CLS
PRINT "*****************************************************************"
PRINT "* LPT8SPEC.BAS is a sample QuickBASIC spectral analysis         *"
PRINT "* program for the LPT:Analog! 8-Channel 12-bit A/D printer      *"
PRINT "* port adapter.                                                 *"
PRINT "*                                                               *"
PRINT "* This program estimates the power spectral distribution of a   *"
PRINT "* data sequence using three different approaches:               *"
PRINT "* 1) the Zero-Padded FFT,                                       *"
PRINT "* 2) the Welch Periodogram Method, and                          *"
PRINT "* 3) Marple's Autoregressive Method.                            *"
PRINT "*                                                               *"
PRINT "* The timing scheme implemented here assumes that the PC's      *"
PRINT "* resources available for the program's execution remain        *"
PRINT "* constant, and as such, the time it takes to execute a uniform *"
PRINT "* section of code remains invariant.  Note that this condition  *"
PRINT "* is not fulfilled when the operating system dynamically        *"
PRINT "* allocates resources to multiple concurrent tasks. For this    *"
PRINT "* reason, this technique will NOT yield uniform timing when     *"
PRINT "* the program runs from within a Windows.                       *"
PRINT "*                                                               *"
PRINT "* (c) Copyright 1997, ADNAV Electronics                         *"
PRINT "*****************************************************************"
PRINT
PRINT "Please press any key to continue ";
'
'  ADNAV Electronics
'  58 Chicory Ct.
'  Lake Jackson, TX   77566
'  (409)292-0988
'  adnav@tgn.net
'  http://www.tgn.net/~adnav/
'
DEFDBL A-Z      'Use double precision
'
' Printer port locations
' ----------------------
CONST prinop1 = &H378     ' Printer Output Port for LPT1
CONST prinstat1 = &H379   ' Printer Status Port for LPT1
CONST printcont1 = &H37A  ' Printer Control Port for LPT1
CONST prinop2 = &H278     ' Printer Output Port for LPT2
CONST prinstat2 = &H279   ' Printer Status Port for LPT2
CONST printcont2 = &H27A  ' Printer Control Port for LPT2
CONST prinop3 = &H3BC     ' Printer Output Port for LPT3
CONST prinstat3 = &H3BD   ' Printer Status Port for LPT3
CONST printcont3 = &H3BE  ' Printer Control Port for LPT3
'
' Define control pin locations
' ----------------------------
CONST power = 12, notcs = 1, sclk = 2, muxd0 = 16, muxd1 = 32, muxd2 = 64
'
' Define data acquisition variables
' ---------------------------------
DIM port, channel AS INTEGER
DIM bit, dat, mux AS INTEGER
DIM vout AS SINGLE
DIM delay AS LONG
DIM delayt AS DOUBLE
DIM convt AS DOUBLE
'
' Define complex arrays for spectral analysis
' -------------------------------------------
REM $DYNAMIC:
DIM ar(1), ai(1), dr(1), di(1), rr(1), ri(1), cr(1), ci(1), xr(1), xi(1)
DIM w(1), xfftr(1), xffti(1), psd(1), win(1), xsegr(1), xsegi(1)

'
' Initialize LPT:Analog! Model 8C
' -------------------------------
'
OUT prinop, power + notcs ' Apply power, *CS high, multiplexer channel 0
'
DO WHILE INKEY$ = ""
LOOP

portdialog:
'
' Port selection dialog
' ---------------------
'
CLS
PRINT "PORT SELECTION"
PRINT "--------------"
PRINT
PRINT "Please select port to which the LPT:Analog! model 8c-I/O is connected"
PRINT "or press <ENTER> to select the default port location (LPT1:)."
PRINT
PRINT "Note:  The BIOS in some computers assumes a different location for"
PRINT "       the LPTx: assignments than the standard.  If you are unable "
PRINT "       to control the LPT:Analog! adapter from your printer port"
PRINT "       using your default LPTx: port, try another selection."
PRINT
PRINT "< 1 > for LPT1: (default)"
PRINT "< 2 > for LPT2: "
PRINT "< 3 > for LPT3: "
INPUT "", port
IF (port = 0 OR port = 1) THEN
      prinop = prinop1: prinstat = prinstat1: princont = princont1
   ELSE
      IF port = 2 THEN
         prinop = prinop2: prinstat = prinstat2: princont = princont2
      ELSE
         IF port = 3 THEN
            prinop = prinop3: prinstat = prinstat3: princont = princont3
         ELSE
            BEEP: GOTO portdialog
         END IF
      END IF
END IF


'
' Compensate for computer speed
' -----------------------------
'
' The timing scheme implemented here assumes that the PC's resources
' available for the program's execution remain constant, and as such,
' the time it takes to execute a uniform section of code remains
' invariant.  Note that this condition is not fulfilled when the
' operating system allocates resources to multiple concurrent tasks.
' For this reason, this technique will NOT yield uniform timing when
' the program is run from within a Windows (or other multitasking
' OS) shell.
'
' Here, the time that it takes to execute a for-to loop will be
' calculated by turning the timer on for 1 second and counting the
' number of for-to loops which executed within that time.
'
'
CLS
PRINT "Calculating loop timing ... DO NOT INTERRUPT!"
TIMER ON
ON TIMER(1) GOSUB delayt
' 1 second / number of for-to loops
' after collecting data for 1 second, break away
FOR delay = 1 TO 40000000: NEXT delay ' dummy delay loop
'
convspeed:
TIMER OFF
CLS
PRINT "Calculating conversion speed (10 seconds) ... DO NOT INTERRUPT!";
TIMER ON
ON TIMER(10) GOSUB convt      ' collect data for 10 seconds and break away
FOR acq = 1 TO 1000000        ' dummy loop containing prototype code
    GOSUB acquire
    xr(1) = vout
    FOR delay = 0 TO 0: NEXT delay
NEXT acq

'
' Acquisition and display control
' -------------------------------
start:
TIMER OFF
PRINT
PRINT "Please enter desired sampling frequency [Samples/s]"
fsamp:
INPUT fsamp
IF fsamp > (1 / convt) THEN
  ' check that desired sampling frequency does not exceed sampling frequency
  ' that can be achieved using this computer running QuickBASIC
  BEEP: PRINT "Maximum sampling frequency for this computer = "; 1 / convt; " [S/s]"
  PRINT "Please re-enter desired sampling frequency !"
  GOTO fsamp
END IF
delsamp = INT(((1 / fsamp) - convt) / delayt)
PRINT "Actual sampling frequency to be used = ";
PRINT USING "####.##"; 1 / (convt + (delsamp * delayt));
PRINT " [S/s]"
PRINT : PRINT
T = convt + (delsamp * delayt)
PRINT "Please input number of samples to be acquired ";

nsamp:
INPUT n
IF n > 1024 THEN
   BEEP: PRINT "Maximum number of samples allowed = 1024"
   GOTO nsamp
END IF

chann:
PRINT
INPUT "please input A/D channel to be used as input [1 ... 8] "; channel
' set up the multiplexer bits according to the selected channel
channel = channel - 1
IF channel < 0 OR channel > 7 THEN
   BEEP: PRINT "Channel number range is 1 ... 8"
   GOTO chann
END IF
mux = (channel AND 1) * muxd0 + ((channel AND 2) / 2) * muxd1 + ((channel AND 4) / 4) * muxd2

CLS
PRINT "SPECTRAL ANALSIS SETUP"
PRINT "----------------------"
PRINT
INPUT "Please input periodogram number of samples per segment ? ", nsamp1
INPUT "Please input periodogram number of samples shift ? ", nshift1
INPUT "Please input auto-regressive model order ? ", ip
BEEP
PRINT : PRINT "Please wait, ACQUIRING DATA !!!"
REDIM xr(n), xi(n)
FOR k = 1 TO n
    GOSUB acquire                            ' acquire a sample
    xr(k) = vout                             ' compute position on screen
    FOR delay = 0 TO delsamp: NEXT delay     ' introduce delay
NEXT k                                       ' next sample
' Detrend by removing average DC
average = 0
FOR k = 1 TO n
    average = average + xr(k)
NEXT k
average = average / n
FOR k = 1 TO n
    xr(k) = xr(k) - average
NEXT k
' Draw display screen on EGA mode 640x350 with 16 colors
SCREEN 9, 0
CLS
LINE (45, 300)-(562, 300), 3, , &H8888
LINE (45, 250)-(562, 250), 3, , &H8888
LINE (45, 200)-(562, 200), 3, , &H8888
LINE (45, 150)-(562, 150), 3, , &H8888
LINE (45, 100)-(562, 100), 3, , &H8888
LINE (45, 50)-(562, 50), 3, , &H8888
LINE (45, 300)-(45, 50), 3, , &H8888
LINE (562, 300)-(562, 50), 3, , &H8888
LINE ((562 - 45) / 2 + 45, 300)-((562 - 45) / 2 + 45, 50), 5, , &H8888
LOCATE 22, 2: PRINT "-100";
LOCATE 18, 2: PRINT "-80";
LOCATE 15, 2: PRINT "-60";
LOCATE 11, 2: PRINT "-40";
LOCATE 8, 2: PRINT "-20";
LOCATE 4, 2: PRINT "0 dB";
LOCATE 3, 28: PRINT "RELATIVE POWER SPECTRUM";
LOCATE 23, 7: PRINT "0";
LOCATE 23, 37: PRINT "0.25";
LOCATE 23, 71: PRINT "0.5";
LOCATE 24, 18: PRINT "FRACTION OF SAMPLING FREQUENCY Fs = ";
PRINT USING "####.##"; 1 / T;
PRINT " (Hz) ";
LOCATE 1, 4: PRINT "PSD ESTIMATORS :  ";
COLOR 11: PRINT "Zero-Padded FFT   ";
COLOR 12: PRINT "Welch's Method   ";
COLOR 10: PRINT "Marple's Method  ";

' Compute zero-padded FFT
npsd = 1024
nshift = 1
nsamp = n            'Set the periodogram for a single segment, and
wintype = 1          'Use rectangular window, in order to
GOSUB periodogram    'Compute the zero-padded FFT through the periodogram
col = 11: GOSUB plot 'Plot results in light blue

' Estimate the PSD through Welch's averaged periodogram method
nshift = nshift1     'Periodogram number of sample shift between segments
nsamp = nsamp1       'Periodogram number of samples per segment
wintype = 0          'Apply Hamming window
GOSUB periodogram    'Estimate PSD through Welch's method
col = 12: GOSUB plot 'Plot results in light red

' Estimate the PSD through Marple's method
GOSUB marplepsd      'Estimate PSD through Marple's AR method
col = 10: GOSUB plot 'Plot results in light green
GOTO progend

marplepsd:
'  Subroutine to estimate the power spectral distribution of a data
'  sequence by Marple's method.  This subroutine first solves Marple's
'  equations for the estimation of complex autoregressive coefficients
'  from complex data.  Then, it evaluates the transfer function of the
'  estimated AR system by using the FFT.
'
'   Input Parameters:
'     n      - Number of data samples (integer)
'     ip     - Order of linear prediction model (integer)
'     xr,xi  - Array of complex data samples xr(1),xi(1) through xr(n),xi(n)
'     npsd   - Power spectral distribution length
'
'   Intermediate Parameters:
'     p      - Real linear prediction variance at order ip
'     ar,ai  - Array of complex linear prediction coefficients
'
'  Output Parameters:
'     psd    - Array containing real power spectral distribution, with
'              a maximum power of psdmax
'
REDIM ar(ip), ai(ip), dr(ip + 1), di(ip + 1), rr(ip), ri(ip)
REDIM cr(ip + 1), ci(ip + 1)

r1 = 0
FOR k = 2 TO n - 1
    r1 = r1 + 2 * (xr(k) ^ 2 + xi(k) ^ 2)
NEXT k
r2 = xr(1) ^ 2 + xi(1) ^ 2
r3 = xr(n) ^ 2 + xi(n) ^ 2
r4 = 1 / (r1 + 2 * (r2 + r3))
p = r1 + r2 + r3
delta = 1 - r2 * r4
gamma = 1 - r3 * r4
lambdar = r4 * (xr(1) * xr(n) - xi(1) * xi(n))
lambdai = -r4 * (xr(1) * xi(n) + xr(n) * xi(1))
cr(1) = xr(n) * r4: ci(1) = xi(n) * r4
dr(1) = r4 * xr(1): di(1) = -r4 * xi(1)
m = 0
IF ip = 0 THEN
  p = (.5 * r1 + r2 + r3) / n
  LOCATE 1, 1: BEEP: PRINT "ERROR: Zero AR model order !                     "
  GOTO progend
END IF
'   Main loop of Marple's Modified Covariance algorithm
marpleloop:
m = m + 1
save1r = 0
save1i = 0
FOR k = m + 1 TO n
    save1r = save1r + xr(k) * xr(k - m) + xi(k) * xi(k - m)
    save1i = save1i - xr(k) * xi(k - m) + xi(k) * xr(k - m)
NEXT k
save1r = 2 * save1r: save1i = 2 * save1i
rr(m) = save1r: ri(m) = -save1i
thetar = xr(n) * dr(1) - xi(n) * di(1): thetai = xr(n) * di(1) + dr(1) * xi(n)
psir = xr(n) * cr(1) - xi(n) * ci(1): psii = xr(n) * ci(1) + cr(1) * xi(n)
xir = xr(1) * dr(1) + xi(1) * di(1): xii = xr(1) * di(1) - xi(1) * dr(1)
IF m <> 1 THEN
   FOR k = 1 TO m - 1
       thetar = thetar + xr(n - k) * dr(k + 1) - xi(n - k) * di(k + 1)
       thetai = thetai + xr(n - k) * di(k + 1) + xi(n - k) * dr(k + 1)
       psir = psir + xr(n - k) * cr(k + 1) - xi(n - k) * ci(k + 1)
       psii = psii + xr(n - k) * ci(k + 1) + xi(n - k) * cr(k + 1)
       xir = xir + xr(k + 1) * dr(k + 1) + xi(k + 1) * di(k + 1)
       xii = xii + xr(k + 1) * di(k + 1) - xi(k + 1) * dr(k + 1)
       rr(k) = rr(k) - (xr(n + 1 - m) * xr(n + 1 - m + k) + xi(n + 1 - m) * xi(n + 1 - m + k))
       ri(k) = ri(k) - (-xr(n + 1 - m) * xi(n + 1 - m + k) + xi(n + 1 - m) * xr(n + 1 - m + k))
       rr(k) = rr(k) - (xr(m) * xr(m - k) + xi(m) * xi(m - k))
       ri(k) = ri(k) - (xr(m) * xi(m - k) - xi(m) * xr(m - k))
       save1r = save1r + rr(k) * ar(m - k) + ri(k) * ai(m - k)
       save1i = save1i + rr(k) * ai(m - k) - ri(k) * ar(m - k)
   NEXT k
END IF
c1r = -save1r / p: c1i = -save1i / p
ar(m) = c1r: ai(m) = c1i
p = p * (1 - c1r ^ 2 - c1i ^ 2)
IF m <> 1 THEN
   FOR k = 1 TO m / 2
       mk = m - k
       save1r = ar(k): save1i = ai(k)
       auxr = save1r + c1r * ar(mk) + c1i * ai(mk)
       ai(k) = save1i - c1r * ai(mk) + c1i * ar(mk)
       ar(k) = auxr
       IF k <> mk THEN
          ar(mk) = ar(mk) + c1r * save1r + c1i * save1i
          ai(mk) = ai(mk) - c1r * save1i + c1i * save1r
       END IF
   NEXT k
END IF
IF m = ip THEN
   p = .5 * p / (n - m)
   GOTO arpsd
END IF
'   Time update of C,D vectors and GAMMA,DELTA,LAMBDA scalars
r1 = 1 / (delta * gamma - lambdar ^ 2 - lambdai ^ 2)
c1r = (thetar * lambdar + thetai * lambdai + psir * delta) * r1
c1i = (-thetar * lambdai + thetai * lambdar + psii * delta) * r1
c2r = (psir * lambdar - psii * lambdai + thetar * gamma) * r1
c2i = (psir * lambdai + psii * lambdar + thetai * gamma) * r1
c3r = (xir * lambdar + xii * lambdai + thetar * delta) * r1
c3i = (-xir * lambdai + xii * lambdar + thetai * delta) * r1
c4r = (thetar * lambdar - thetai * lambdai + xir * gamma) * r1
c4i = (thetar * lambdai + thetai * lambdar + xii * gamma) * r1
FOR k = 1 TO (m - 1) / 2 + 1
    mk = m + 1 - k
    save1r = cr(k): save1i = -ci(k)
    save2r = dr(k): save2i = -di(k)
    save3r = cr(mk): save3i = -ci(mk)
    save4r = dr(mk): save4i = -di(mk)
    cr(k) = cr(k) + c1r * save3r - c1i * save3i + c2r * save4r - c2i * save4i
    ci(k) = ci(k) + c1r * save3i + c1i * save3r + c2r * save4i + c2i * save4r
    dr(k) = dr(k) + c3r * save3r - c3i * save3i + c4r * save4r - c4i * save4i
    di(k) = di(k) + c3r * save3i + c3i * save3r + c4r * save4i + c4i * save4r
    IF k <> mk THEN
       cr(mk) = cr(mk) + c1r * save1r - c1i * save1i + c2r * save2r - c2i * save2i
       ci(mk) = ci(mk) + c1i * save1r + c1r * save1i + c2r * save2i + c2i * save2r
       dr(mk) = dr(mk) + c3r * save1r - c3i * save1i + c4r * save2r - c4i * save2i
       di(mk) = di(mk) + c3r * save1i + c3i * save1r + c4r * save2i + c4i * save2r
    END IF
NEXT k
r2 = psir ^ 2 + psii ^ 2
r3 = thetar ^ 2 + thetai ^ 2
r4 = xir ^ 2 + xii ^ 2
auxr = psir * lambdar - psii * lambdai
auxi = psir * lambdai + psii * lambdar
aux2r = auxr * thetar + auxi * thetai
r5 = gamma - (r2 * delta + r3 * gamma + 2 * aux2r) * r1
auxr = thetar * lambdar - thetai * lambdai
auxi = thetar * lambdai + thetai * lambdar
aux2r = auxr * xir + auxi * xii
r2 = delta - (r3 * delta + r4 * gamma + 2 * aux2r) * r1
gamma = r5
delta = r2
lambdar = lambdar + c3r * psir + c3i * psii + c4r * thetar + c4i * thetai
lambdai = lambdai - c3r * psii + c3i * psir - c4r * thetai + c4i * thetar
IF p <= 0 THEN GOTO arpsderr
IF (delta <= 0 OR delta > 1 OR gamma <= 0 OR gamma > 1) THEN GOTO arpsderr
r1 = 1 / p
r2 = 1 / (delta * gamma - lambdar ^ 2 - lambdai ^ 2)
efr = xr(m + 1): efi = xi(m + 1)
ebr = xr(n - m): ebi = xi(n - m)
FOR k = 1 TO m
   efr = efr + ar(k) * xr(m + 1 - k) - ai(k) * xi(m + 1 - k)
   efi = efi + ar(k) * xi(m + 1 - k) + ai(k) * xr(m + 1 - k)
   ebr = ebr + ar(k) * xr(n - m + k) + ai(k) * xi(n - m + k)
   ebi = ebi + ar(k) * xi(n - m + k) - ai(k) * xr(n - m + k)
NEXT k
c1r = ebr * r1: c1i = ebi * r1
c2r = efr * r1: c2i = -efi * r1
c3r = (ebr * delta + efr * lambdar - efi * lambdai) * r2
c3i = (-ebi * delta + efr * lambdai + efi * lambdar) * r2
auxr = ebr * lambdar - ebi * lambdai
auxi = ebr * lambdai + ebi * lambdar
c4r = (efr * gamma + auxr) * r2: c4i = (efi * gamma - auxi) * r2
FOR k = m TO 1 STEP -1
   save1r = ar(k): save1i = ai(k)
   ar(k) = save1r + c3r * cr(k) - c3i * ci(k) + c4r * dr(k) - c4i * di(k)
   ai(k) = save1i + c3r * ci(k) + c3i * cr(k) + c4r * di(k) + c4i * dr(k)
   cr(k + 1) = cr(k) + c1r * save1r - c1i * save1i
   ci(k + 1) = ci(k) + c1r * save1i + c1i * save1r
   dr(k + 1) = dr(k) + c2r * save1r - c2i * save1i
   di(k + 1) = di(k) + c2r * save1i + c2i * save1r
NEXT k
cr(1) = c1r: ci(1) = c1i
dr(1) = c2r: di(1) = c2i
r3 = ebr ^ 2 + ebi ^ 2
r4 = efr ^ 2 + efi ^ 2
auxr = efr * ebr - efi * ebi: auxi = efr * ebi + efi * ebr
aux2r = auxr * lambdar - auxi * lambdai
p = p - (r3 * delta + r4 * gamma + 2 * aux2r) * r2
delta = delta - r4 * r1
gamma = gamma - r3 * r1
auxr = efr * ebr - efi * ebi: auxi = efr * ebi + efi * ebr
lambdar = lambdar + auxr * r1: lambdai = lambdai - auxi * r1
IF (p > 0 AND delta > 0 AND delta <= 1 AND gamma > 0 AND gamma <= 1) THEN GOTO marpleloop
arpsderr:
LOCATE 1, 6: BEEP
PRINT "ERROR:  Numerical ill-conditioning detected for model order  >  "; m - 1;
GOTO progend
arpsd:   'Evaluate the AR model's transfer function
nfft = npsd
REDIM xfftr(nfft), xffti(nfft), psd(npsd)
xfftr(1) = 1: xffti(1) = 0
FOR k = 1 TO ip
    xfftr(k + 1) = ar(k): xffti(k + 1) = ai(k)
NEXT k
FOR k = ip + 2 TO npsd
   xfftr(k) = 0: xffti(k) = 0   'Zero-pad up to npsd
NEXT k
GOSUB FFT
psdmax = 0
FOR k = 1 TO npsd
   psd(k) = p * T / (xfftr(k) ^ 2 + xffti(k) ^ 2)
   IF psd(k) > psdmax THEN psdmax = psd(k)
NEXT k
RETURN

FFT:
'   Subroutine to compute the complex FFT of a complex data series.
'
'   Input Parameters:
'     nfft        - FFT size
'     t           - Sample interval in seconds
'     xfftr,xffti - Array of nfft complex data samples xfftr(1),xffti(1)
'                   to xfftr(nfft),xffti(nfft)
'
'   Output Parameters:
'     xfftr,xffti - nfft complex transform values replace original data
'                   samples indexed from k=1 to k=nfft, representing the
'                   frequencies (k-1)/nfft*t [Hz]
'
REDIM wr(nfft) AS DOUBLE, wi(nfft) AS DOUBLE
' Set up complex exponential table for FFT
nexp = 1
nt = 2 ^ nexp
WHILE nt < nfft
   nexp = nexp + 1
   nt = 2 ^ nexp
WEND
IF nt <> nfft THEN
   LOCATE 1, 4: BEEP: PRINT "Error!  nfft is not a power of 2                "
   GOTO progend
END IF
s = 8 * ATN(1) / (nt)
c1r = COS(s): c1i = -SIN(s)
c2r = 1: c2i = 0
' Compute complex exponential array
FOR k = 1 TO nt
   wr(k) = c2r: wi(k) = c2i
   auxr = c2r * c1r - c2i * c1i: c2i = c2r * c1i + c2i * c1r
   c2r = auxr
NEXT k
' Main FFT routine
mm = 1
ll = nfft
FOR k = 1 TO nexp
   nn = ll / 2
   jj = mm + 1
   FOR i = 1 TO nfft STEP ll
      kk = i + nn
      c1r = xfftr(i) + xfftr(kk): c1i = xffti(i) + xffti(kk)
      xfftr(kk) = xfftr(i) - xfftr(kk): xffti(kk) = xffti(i) - xffti(kk)
      xfftr(i) = c1r: xffti(i) = c1i
   NEXT i
   IF nn <> 1 THEN
      FOR j = 2 TO nn
         c2r = wr(jj): c2i = wi(jj)
         FOR i = j TO nfft STEP ll
            kk = i + nn
            c1r = xfftr(i) + xfftr(kk): c1i = xffti(i) + xffti(kk)
            auxr = xfftr(i) - xfftr(kk): auxi = xffti(i) - xffti(kk)
            xfftr(kk) = auxr * c2r - auxi * c2i
            xffti(kk) = auxr * c2i + auxi * c2r
            xfftr(i) = c1r: xffti(i) = c1i
         NEXT i
         jj = jj + mm
      NEXT j
      ll = nn
      mm = mm * 2
   END IF
NEXT k
nv2 = nfft / 2
nm1 = nfft - 1
j = 1
FOR i = 1 TO nm1
   IF i < j THEN
      c1r = xfftr(j): c1i = xffti(j)
      xfftr(j) = xfftr(i): xffti(j) = xffti(i)
      xfftr(i) = c1r: xffti(i) = c1i
   END IF
k = nv2
WHILE k < j
   j = j - k
   k = k / 2
WEND
   j = j + k
NEXT i
FOR i = 1 TO nfft
   xfftr(i) = xfftr(i) * T: xffti(i) = xffti(i) * T
NEXT i
RETURN

periodogram:
'   Subroutine to compute the averaged periodogram over nseg segments.
'   Input Parameters:
'     n       - Number of data samples
'     nshift  - Number of samples shift between segments
'     nsamp   - Number of samples per segment (must be even)
'     t       - Sample interval in seconds
'     xr,xi   - Array of complex samples xr(1),xi(1) to xr(n),xi(n)
'     wintype - Window type - 0 = Hamming, other = rectangular
'
'   Output Parameters:
'     nseg   - Number of segments averaged
'     psd    - Array containing real power spectral distribution, with
'              a maximum power of psdmax
'
REDIM win(nsamp), xsegr(npsd), xsegi(npsd), psd(npsd), xfftr(npsd), xffti(npsd)
pi2 = 8 * ATN(1)
' Compute window
FOR k = 1 TO nsamp
   IF wintype = 0 THEN
      ' Hamming window
      win(k) = .538 + .462 * COS(pi2 * (-.5 + (k - 1) / (nsamp - 1)))
   ELSE
      win(k) = 1  'Rectangular window
   END IF
NEXT k
' Compute Welch's averaged periodogram applying window
nseg = (n - nsamp) / nshift + 1
FOR k = 1 TO nseg
   FOR j = 1 TO nsamp
      index = j + (k - 1) * nshift
      xsegr(j) = xr(index) * win(j)
      xsegi(j) = xi(index) * win(j)
   NEXT j
   FOR j = nsamp + 1 TO npsd
      xsegr(j) = 0: xsegi(j) = 0    'Zero-pad up to npsd
   NEXT j
   nfft = npsd
   FOR j = 1 TO nfft
      xfftr(j) = xsegr(j): xffti(j) = xsegi(j)
   NEXT j
   GOSUB FFT
   FOR j = 1 TO npsd
      IF k = 1 THEN
         psd(j) = xfftr(j) ^ 2 + xffti(j) ^ 2
      ELSE
         psd(j) = psd(j) + xfftr(j) ^ 2 + xffti(j) ^ 2
      END IF
   NEXT j
NEXT k
psdmax = 0
FOR k = 1 TO npsd
   psd(k) = psd(k) / (nseg * nsamp)
   IF psd(k) > psdmax THEN psdmax = psd(k)
NEXT k
RETURN

plot:
' Plot results on graph using color col, assuming that npsd = 512 (complex)
' or npsd = 1024 (real)
FOR k = 1 TO npsd
   IF psd(k) = 0 THEN psd(k) = 1E-20
   IF psdmax = 0 THEN psdmax = 1E-20
   psd(k) = 20 * LOG(psd(k) / psdmax) / LOG(10) 'Normalize and transform data to dBs
   IF psd(k) < -100 THEN psd(k) = -100          'Clip at -100 dB
NEXT k
IF (complex$ = "y" OR complex$ = "Y") THEN
   ' Plot PSD for positive frequencies
   FOR k = 2 TO 256
      LINE (44 + k + 256, 50 - 2.5 * psd(k - 1))-(45 + k + 256, 50 - 2.5 * psd(k)), col
   NEXT k
   ' Plot PSD for negative frequencies
   FOR k = 256 TO 512
      LINE (44 + k - 256, 50 - 2.5 * psd(k - 1))-(45 + k - 256, 50 - 2.5 * psd(k)), col
   NEXT k
ELSE
   ' Plot PSD only for positive frequencies
   FOR k = 2 TO 512
      LINE (44 + k, 50 - 2.5 * psd(k - 1))-(45 + k, 50 - 2.5 * psd(k)), col
   NEXT k
END IF
RETURN

progend:
' Wait for any key to be pressed to exit program
WHILE INKEY$ = "": WEND
OUT prinop, 0                          ' send LPT:Analog! to sleep
SCREEN 0                               ' return to text mode screen
END

acquire:
'
' Acquisition Loop
' ----------------
'
OUT prinop, power + notcs + sclk + mux ' power up but keep CS' deasserted
convert:
OUT prinop, power + mux                ' convert by asserting CS', pull SCLK low
loop1:
FOR clocknum = 1 TO 2                  ' two clock pulses
   OUT prinop, power + sclk + mux
   OUT prinop, power + mux
NEXT clocknum
dat = 0                                ' clear A/D accumulator
FOR clocknum = 11 TO 0 STEP -1         ' clock 12 bits serially
   OUT prinop, power + sclk + mux      ' clock pulse rising edge
   OUT prinop, power + mux             ' clock pulse falling edge
   bit = (INP(prinstat) AND 8) / 8     ' read port and filter bit
   dat = dat + (2 ^ clocknum) * bit    ' accumulate from bit 11 to bit 0
NEXT clocknum                          ' next bit
OUT prinop, power + notcs + mux        ' deassert *CS
vout = dat * 2.5 / ((2 ^ 12 - 1))      ' translate A/D data to Volts
                                       ' the 2.5 represents the maximum input
                                       ' for an analog-input channel without
                                       ' prescaling
RETURN

delayt:
BEEP
'
' Calculate number of iterations for conversion wait loop
' -------------------------------------------------------
'
' This is the break-away point for the for-to loop timing routine.
' Notice that the subroutine is not made to return to the line
' below its calling point, but rather to the routine which times
' the data acquisition process
'
delayt = 1 / delay          ' 1 second / number of for-to loops
RETURN convspeed            ' re-enter at the line labeled "convspeed"

convt:
'
' Calculate time that it takes to convert a single data frame
' -----------------------------------------------------------
'
' This is the break-away point for the acquisition loop timing routine.
' Once again, this subroutine does not return to the line following
' its calling point, but rather to the portion of the program which
' requests selection of the sampling rate.
'
SCREEN 0                    ' return to text screen
convt = 10 / acq            ' average over 10 seconds
CLS
PRINT "DATA ACQUISITION SETUP"
PRINT "----------------------"
PRINT
PRINT "Conversion time = "; convt; " seconds"
PRINT "Maximum sampling frequency = "; 1 / convt; " Samples/s per channel"
RETURN start                ' re-enter at the line labeled "start"

