CLS
PRINT "*****************************************************************"
PRINT "* ACQUIRE8.BAS is a sample QuickBASIC acquisition and display   *"
PRINT "* program for the LPT:Analog! 8-Channel 12-bit A/D printer      *"
PRINT "* port adapter.                                                 *"
PRINT "*                                                               *"
PRINT "* The program makes use of controlled delay loops in order to   *"
PRINT "* sample data at desired intervals.                             *"
PRINT "*                                                               *"
PRINT "* The timing scheme implemented here assumes that the PC's      *"
PRINT "* resources available for the program's execution remain        *"
PRINT "* constant, and as such, the time it takes to execute a uniform *"
PRINT "* section of code remains invariant.  Note that this condition  *"
PRINT "* is not fulfilled when the operating system dynamically        *"
PRINT "* allocates resources to multiple concurrent tasks. For this    *"
PRINT "* reason, this technique will NOT yield uniform timing when     *"
PRINT "* the program runs from within a Windows (or other              *"
PRINT "* multitasking OS) shell.                                       *"
PRINT "*                                                               *"
PRINT "* (c) Copyright 1997, ADNAV Electronics                         *"
PRINT "*****************************************************************"
PRINT
'
'  ADNAV Electronics
'  58 Chicory Ct.
'  Lake Jackson, TX   77566
'  (409)292-0988
'  adnav@tgn.net
'  http://www.tgn.net/~adnav/
'
' Printer port locations
' ----------------------
CONST prinop1 = &H378     ' Printer Output Port for LPT1
CONST prinstat1 = &H379   ' Printer Status Port for LPT1
CONST printcont1 = &H37A  ' Printer Control Port for LPT1
CONST prinop2 = &H278     ' Printer Output Port for LPT2
CONST prinstat2 = &H279   ' Printer Status Port for LPT2
CONST printcont2 = &H27A  ' Printer Control Port for LPT2
CONST prinop3 = &H3BC     ' Printer Output Port for LPT3
CONST prinstat3 = &H3BD   ' Printer Status Port for LPT3
CONST printcont3 = &H3BE ' Printer Control Port for LPT3
'
' Define control pin locations
' ----------------------------
CONST power = 12, notcs = 1, sclk = 2, muxd0 = 16, muxd1 = 32, muxd2 = 64
'
' Define Data Acquisition Array and other variables
' -------------------------------------------------
'
DIM port, framelength, channel AS INTEGER
DIM bit, dat, scrpoint, mux, y, status, din1, din2, din3, din4 AS INTEGER
DIM vout AS SINGLE
DIM delay AS LONG
DIM delayt AS DOUBLE
DIM convt AS DOUBLE
'
' Idle while waiting for a key to be pressed
' ------------------------------------------
PRINT "Please press any key to continue"
DO WHILE INKEY$ = ""'
LOOP
portdialog:
'
' Port selection dialog
' ---------------------
'
CLS
PRINT "Please select port to which the LPT:Analog! Model 8C is connected"
PRINT "or press <ENTER> to select the default port location (LPT1:)."
PRINT
PRINT "Note:  The BIOS in some computers assumes a different location for"
PRINT "       the LPTx: assignments than the standard.  If you are unable "
PRINT "       to control the LPT:Analog! adapter from your printer port"
PRINT "       using your default LPTx: port, try another selection."
PRINT
PRINT "< 1 > for LPT1: "
PRINT "< 2 > for LPT2: "
PRINT "< 3 > for LPT3: "
INPUT "", port
IF (port = 0 OR port = 1) THEN
      prinop = prinop1: prinstat = prinstat1: princont = princont1
   ELSE
      IF port = 2 THEN
         prinop = prinop2: prinstat = prinstat2: princont = princont2
      ELSE
         IF port = 3 THEN
            prinop = prinop3: prinstat = prinstat3: princont = princont3
         ELSE
            BEEP: GOTO portdialog
         END IF
      END IF
END IF

framedialog:
'
' Data acquisition frame length dialog
' ------------------------------------
'
CLS
INPUT "How many channels would you like to acquire for display (min 1, max 8) "; framelength
IF (framelength < 1 OR framelength > 8) THEN BEEP: GOTO framedialog
PRINT
'
' Initialize LPT:Analog! Model 8C
' -------------------------------
'
OUT prinop, power + notcs ' Apply power, *CS high, multiplexer channel 0
'
' Compensate for computer speed
' -----------------------------
'
' The timing scheme implemented here assumes that the PC's resources
' available for the program's execution remain constant, and as such,
' the time it takes to execute a uniform section of code remains
' invariant.  Note that this condition is not fulfilled when the
' operating system allocates resources to multiple concurrent tasks.
' For this reason, this technique will NOT yield uniform timing when
' the program is run from within a Windows (or other multitasking
' OS) shell.
'
' Here, the time that it takes to execute a for-to loop will be
' calculated by turning the timer on for 1 second and counting the
' number of for-to loops which executed within that time.
'
PRINT "Calculating loop timing ..."
TIMER ON
ON TIMER(1) GOSUB delayt              ' collect data for 1 second and break away
FOR delay = 1 TO 40000000: NEXT delay ' dummy delay loop
END
'
convspeed:
TIMER OFF
'
' To calculate conversion speed, substitute the following
' by a prototype of the code to be executed while
' acquiring and displaying data
'
SCREEN 8
CLS
GOSUB drawframe
scrpoint = 61
acq = 0
LOCATE 16, 18
PRINT "    Calculating conversion speed (10 seconds) ...";
LOCATE 18, 18
COLOR 14
PRINT "    Please, DO NOT interrupt !"
COLOR 7
TIMER ON
ON TIMER(10) GOSUB convt      ' collect data for 10 seconds and break away
'
' dummy loop containing prototype code
' please see comments on the actual acquisition
' control routine for details on its operation
DO WHILE INKEY$ = ""
IF scrpoint > 638 THEN
   scrpoint = 61
ELSE
END IF
LINE (scrpoint, 66)-(scrpoint, 198), 0
LINE (scrpoint, 1)-(scrpoint, 58), 0
FOR channel = 0 TO framelength - 1
       mux = (channel AND 1) * muxd0 + ((channel AND 2) / 2) * muxd1 + ((channel AND 4) / 4) * muxd2
       GOSUB acquire
       y = INT((2.5 - vout) * 53) + 66
       PSET (scrpoint, y), channel + 1
NEXT channel
GOSUB monitor
PSET (scrpoint, 10 - 8 * din1), 1
PSET (scrpoint, 26 - 8 * din2), 2
PSET (scrpoint, 42 - 8 * din3), 3
PSET (scrpoint, 58 - 8 * din4), 4
FOR delay = 0 TO 0: NEXT delay
acq = acq + 1
scrpoint = scrpoint + 1
LOOP
'
' Acquisition and display control
' -------------------------------
start:
TIMER OFF
PRINT
PRINT "Please enter desired sampling frequency [Samples/s]"
fsamp:
INPUT fsamp
IF fsamp > (1 / convt) THEN
  ' check that desired sampling frequency does not exceed sampling frequency
  ' that can be achieved using this computer running QuickBASIC
  BEEP: PRINT "Maximum sampling frequency for this computer = "; 1 / convt; " [S/s]"
  PRINT "Please re-enter desired sampling frequency !"
  GOTO fsamp
END IF
delsamp = INT(((1 / fsamp) - convt) / delayt)
PRINT "Actual sampling frequency to be used = ";
PRINT USING "####.##"; 1 / (convt + (delsamp * delayt));
PRINT " [S/s]"
PRINT : PRINT
PRINT "Please press any key to start acquiring data"
PRINT
PRINT "While acquiring, please press any key to end"
PRINT
PRINT "Color coding for digital input channels: ";
COLOR 1: PRINT "1  ";   ' blue
COLOR 2: PRINT "2  ";   ' green
COLOR 3: PRINT "3  ";   ' cyan
COLOR 4: PRINT "4  ";   ' red
COLOR 7                 ' return to white text
PRINT
PRINT
PRINT "Color coding for analog input channels:  ";
COLOR 1: PRINT "1  ";   ' blue
COLOR 2: PRINT "2  ";   ' green
COLOR 3: PRINT "3  ";   ' cyan
COLOR 4: PRINT "4  ";   ' red
COLOR 5: PRINT "5  ";   ' magenta
COLOR 6: PRINT "6  ";   ' brown
COLOR 14: PRINT "7  ";  ' yellow
COLOR 8: PRINT "8  "    ' grey
COLOR 7                 ' return to white text
'
' Idle while waiting for a key to be pressed
DO WHILE INKEY$ = ""'
LOOP
'
' Acquisition and display control
' -------------------------------
SCREEN 8                                     ' CGA graphics mode 640x200
scrpoint = 61                                ' initialize x-axis position
CLS                                          ' clear screen
GOSUB drawframe                              ' draw frame for signal acquisition
                                             ' window and place labels
'
' Continuously acquire and display all channels while no key is pressed
'
DO WHILE INKEY$ = ""
IF scrpoint > 638 THEN                       ' if x coordinate on screen is
                                             ' > pixel 239, then reset trace
                                             ' to first horizontal position (61)
   scrpoint = 61
ELSE
   scrpoint = scrpoint                       ' this line is there to balance
                                             ' the delay caused by scrpoint = 61
END IF
'
' Clear prior data from the window on retrace.  Note that this is done
' by painting black (color = 0) vertical lines at the current horizontal
' position on the display windows.
'
LINE (scrpoint, 66)-(scrpoint, 198), 0       ' clear previous data from the
                                             ' current horizontal position on
                                             ' the analog signal display window
LINE (scrpoint, 1)-(scrpoint, 58), 0         ' clear previous data from the
                                             ' current horizontal position on
                                             ' the digital input line display
                                             ' windows.
'
' Acquire and display data for all selected analog-input channels
'
FOR channel = 0 TO framelength - 1               ' the first channel of the
                                                 ' multiplexer is zero, and thus
                                                 ' the channel for-to loop is made
                                                 ' to count from 0 to framelength-1
       ' set up the multiplexer bits according to the selected channel
       mux = (channel AND 1) * muxd0 + ((channel AND 2) / 2) * muxd1 + ((channel AND 4) / 4) * muxd2
       GOSUB acquire                             ' now acquire a sample for the
                                                 ' selected channel
       y = INT((2.5 - vout) * 53) + 66           ' compute position of sampled
                                                 ' point for display on the
                                                 ' analog signal display window
                                                 ' to be displayed on the
                                                 ' screen
       PSET (scrpoint, y), channel + 1           ' place point with color depending
                                                 ' on channel being displayed.
                                                 ' Notice that 1 is added to
                                                 ' compensate for offset introduced
                                                 ' above to allow multiplexer to see
                                                 ' channel 1 as channel 0
NEXT channel                                     ' select next channel of frame
'
' Acquire data for digital-input lines
'
GOSUB monitor
PSET (scrpoint, 10 - 8 * din1), 1               ' place point for input line 1
PSET (scrpoint, 26 - 8 * din2), 2               ' place point for input line 2
PSET (scrpoint, 42 - 8 * din3), 3               ' place point for input line 3
PSET (scrpoint, 58 - 8 * din4), 4               ' place point for input line 4
'
' Introduce delay as needed to achieve desired sampling rate
'
FOR delay = 0 TO delsamp: NEXT delay
acq = acq + 0                                  ' dummy instruction required to
                                               ' compensate same instruction
                                               ' within rate calculation loop
scrpoint = scrpoint + 1                        ' advance to next horizontal point
LOOP                                           ' loop back

progend:
'
' Leave program
' -------------
OUT prinop, 0                          ' send LPT:Analog! to sleep
SCREEN 0                               ' return to text mode screen
END

acquire:
'
' Acquisition Loop
' ----------------
'
OUT prinop, power + notcs + sclk + mux ' power up but keep CS' deasserted
convert:
OUT prinop, power + mux                ' convert by asserting CS', pull SCLK low
loop1:
FOR clocknum = 1 TO 2                  ' two clock pulses
   OUT prinop, power + sclk + mux
   OUT prinop, power + mux
NEXT clocknum
dat = 0                                ' clear A/D accumulator
FOR clocknum = 11 TO 0 STEP -1         ' clock 12 bits serially
   OUT prinop, power + sclk + mux      ' clock pulse rising edge
   OUT prinop, power + mux             ' clock pulse falling edge
   bit = (INP(prinstat) AND 8) / 8     ' read port and filter bit
   dat = dat + (2 ^ clocknum) * bit    ' accumulate from bit 11 to bit 0
NEXT clocknum                          ' next bit
OUT prinop, power + notcs + mux        ' deassert *CS
vout = dat * 2.5 / ((2 ^ 12 - 1))      ' translate A/D data to Volts
                                       ' the 2.5 represents the maximum input
                                       ' for an analog-input channel without
                                       ' prescaling
RETURN

delayt:
BEEP
'
' Calculate number of iterations for conversion wait loop
' -------------------------------------------------------
'
' This is the break-away point for the for-to loop timing routine.
' Notice that the subroutine is not made to return to the line
' below its calling point, but rather to the routine which times
' the data acquisition process
'
delayt = 1 / delay          ' 1 second / number of for-to loops
RETURN convspeed            ' re-enter at the line labeled "convspeed"

convt:
'
' Calculate time that it takes to convert a single data frame
' -----------------------------------------------------------
'
' This is the break-away point for the acquisition loop timing routine.
' Once again, this subroutine does not return to the line following
' its calling point, but rather to the portion of the program which
' requests selection of the sampling rate.
'
SCREEN 0                    ' return to text screen
convt = 10 / acq            ' average over 10 seconds
PRINT "Conversion time = "; convt; " seconds"
PRINT "Maximum sampling frequency = "; 1 / convt; " Samples/s per channel"
PRINT "for a data acquisition frame consisting of "; framelength; " channels"
RETURN start                ' re-enter at the line labeled "start"

drawframe:
'
' Draw frames and labels for display screens
' ------------------------------------------
'
' Draw frame for analog signal display
'
LINE (60, 199)-(639, 199)
LINE (60, 65)-(639, 65)
LINE (60, 199)-(60, 65)
LINE (639, 65)-(639, 199)
' label frame for analog signal display
LOCATE 9, 2: PRINT "2.500V";
LOCATE 17, 2: PRINT "1.250V";
LOCATE 25, 2: PRINT "0.000V";
' label digital input channels
COLOR 1
LOCATE 1, 2: PRINT "D#1";
LOCATE 1, 6: PRINT "1";
LOCATE 2, 6: PRINT "0";
COLOR 2
LOCATE 3, 2: PRINT "D#2";
LOCATE 3, 6: PRINT "1";
LOCATE 4, 6: PRINT "0";
COLOR 3
LOCATE 5, 2: PRINT "D#3";
LOCATE 5, 6: PRINT "1";
LOCATE 6, 6: PRINT "0";
COLOR 4
LOCATE 7, 2: PRINT "D#4";
LOCATE 7, 6: PRINT "1";
LOCATE 8, 6: PRINT "0";
COLOR 7
RETURN

monitor:
'
' Monitor Digital Inputs
' ----------------------
'
status = INP(prinstat)                      ' read printer status port
din1 = (status AND 128) / 128               ' evaluate digital input #1
IF din1 = 1 THEN din1 = 0 ELSE din1 = 1     ' invert din1
din3 = (status AND 32) / 32                 ' evaluate digital input #2
din2 = (status AND 64) / 64                 ' evaluate digital input #3
din4 = (status AND 16) / 16                 ' evaluate digital input #4
RETURN

