CLS
PRINT "*****************************************************************"
PRINT "* ATOD_SL8.BAS is a sample QuickBASIC acquisition and display   *"
PRINT "* program for the LPT:Analog! 8-Channel 12-bit A/D printer      *"
PRINT "* port adapter.                                                 *"
PRINT "*                                                               *"
PRINT "* Timer-based acquisition is implemented, allowing sampling     *"
PRINT "* periods between 1 second and 24 hours.  In addition to the    *"
PRINT "* real-time display of data, the program logs acquired data     *"
PRINT "* to disk.                                                      *"
PRINT "*                                                               *"
PRINT "*                                                               *"
PRINT "* (c) Copyright 1997, ADNAV Electronics                         *"
PRINT "*****************************************************************"
PRINT
'
'  ADNAV Electronics
'  58 Chicory Ct.
'  Lake Jackson, TX   77566
'  (409)292-0988
'  adnav@tgn.net
'  http://www.tgn.net/~adnav/
'
' Printer port locations
' ----------------------
CONST prinop1 = &H378     ' Printer Output Port for LPT1
CONST prinstat1 = &H379   ' Printer Status Port for LPT1
CONST printcont1 = &H37A  ' Printer Control Port for LPT1
CONST prinop2 = &H278     ' Printer Output Port for LPT2
CONST prinstat2 = &H279   ' Printer Status Port for LPT2
CONST printcont2 = &H27A  ' Printer Control Port for LPT2
CONST prinop3 = &H3BC     ' Printer Output Port for LPT3
CONST prinstat3 = &H3BD   ' Printer Status Port for LPT3
CONST printcont3 = &H3BE  ' Printer Control Port for LPT3
'
' Define control pin locations
' ----------------------------
CONST power = 12, notcs = 1, sclk = 2, muxd0 = 16, muxd1 = 32, muxd2 = 64
CONST framelength = 8
'
' Define Data Acquisition Array and other variables
' -------------------------------------------------
'
DIM port, channel AS INTEGER
DIM bit, dat, scrpoint, mux, y, status, din1, din2, din3, din4 AS INTEGER
DIM vout AS SINGLE
DIM period, clocknum, npoint, npoints AS INTEGER
DIM hours, minutes AS DOUBLE
'$DYNAMIC
DIM acqdata(1, 1) AS INTEGER
'
' Port selection dialog
' ---------------------
'
CLS
portdialog:
PRINT "Please select port to which the LPT:Analog! Model 8C is connected"
PRINT "or press <ENTER> to select the default port location (LPT1:)."
PRINT
PRINT "Note:  The BIOS in some computers assumes a different location for"
PRINT "       the LPTx: assignments than the standard.  If you are unable "
PRINT "       to control the LPT:Analog! adapter from your printer port"
PRINT "       using your default LPTx: port, try another selection."
PRINT
PRINT "< 1 > for LPT1: "
PRINT "< 2 > for LPT2: "
PRINT "< 3 > for LPT3: "
INPUT "", port
IF (port = 0 OR port = 1) THEN
      prinop = prinop1: prinstat = prinstat1: princont = princont1
   ELSE
      IF port = 2 THEN
         prinop = prinop2: prinstat = prinstat2: princont = princont2
      ELSE
         IF port = 3 THEN
            prinop = prinop3: prinstat = prinstat3: princont = princont3
         ELSE
            BEEP: GOTO portdialog
         END IF
      END IF
END IF
'
' Initialize LPT:Analog! Model 8C
' -------------------------------
'
OUT prinop, power + notcs ' Apply power, *CS high, multiplexer channel 0
'

'
' Data Acquisition Mode Dialog
' ----------------------------
'
INPUT "Please input name of data file where data are to be stored: ", filename$
OPEN filename$ FOR OUTPUT AS #1
periodd:
INPUT "Please input sampling period [integer seconds] (min = 1, max = 86400) : ", period
IF period < 1 OR period > 86400 THEN BEEP: GOTO periodd
nsampd:
INPUT "please enter number of samples to be acquired (min = 2, max = 2666) : ", nsamp
IF nsamp < 2 OR nsamp > 2666 THEN BEEP: GOTO nsampd
REDIM acqdata(nsamp, 11) AS INTEGER
CLS
PRINT
PRINT " Computer will acquire "; nsamp; " samples or until any key is pressed."
PRINT
PRINT " Sampling period will be: "
IF period < 60 THEN
   PRINT period; " seconds"
   ELSE
      IF period < 3600 THEN
         minutes = period / 60
         PRINT period; " seconds = "; INT(minutes); " minutes ";
         IF INT(minutes) < minutes THEN PRINT "and "; INT((minutes - INT(minutes)) * 60 + .01); " seconds"
      ELSE
         hours = period / 3600
         PRINT period; "seconds = "; INT(hours); " hours ";
         IF INT(hours) < hours THEN
            minutes = (hours - INT(hours)) * 60
            PRINT "and "; INT(minutes); " minutes ";
         END IF
         IF INT(minutes) < minutes THEN PRINT "and "; INT((minutes - INT(minutes)) * 60 + .01); " seconds"
      END IF
END IF
PRINT
PRINT "****************************************************************"
PRINT "*   WARNING:                                                   *"
PRINT "*                                                              *"
PRINT "* Turning the computer off before completion of the programmed *"
PRINT "* number of acquisition events or before stopping the          *"
PRINT "* acquisition process will result in the complete loss of      *"
PRINT "* acquired data !                                              *"
PRINT "*                                                              *"
PRINT "****************************************************************"
PRINT
BEEP

PRINT "Please press any key to start acquiring data"
PRINT
PRINT "While acquiring, please press any key to end"
PRINT
PRINT "Color coding for digital input channels: ";
COLOR 1: PRINT "1  ";   ' blue
COLOR 2: PRINT "2  ";   ' green
COLOR 3: PRINT "3  ";   ' cyan
COLOR 4: PRINT "4  ";   ' red
COLOR 7                 ' return to white text
PRINT
PRINT
PRINT "Color coding for analog input channels:  ";
COLOR 1: PRINT "1  ";   ' blue
COLOR 2: PRINT "2  ";   ' green
COLOR 3: PRINT "3  ";   ' cyan
COLOR 4: PRINT "4  ";   ' red
COLOR 5: PRINT "5  ";   ' magenta
COLOR 6: PRINT "6  ";   ' brown
COLOR 14: PRINT "7  ";  ' yellow
COLOR 8: PRINT "8  "    ' grey
COLOR 7                 ' return to white text
'
' Idle while waiting for a key to be pressed
DO WHILE INKEY$ = ""'
LOOP
'
' Acquisition and display control
' -------------------------------
SCREEN 8                                     ' CGA graphics mode 640x200
scrpoint = 61                                ' initialize x-axis position
CLS                                          ' clear screen
GOSUB drawframe                              ' draw frame for signal acquisition
                                             ' window and place labels
TIMER ON                                     ' initialize timer
ON TIMER(period) GOSUB sample                ' define timer events
DO                                           ' run acquisition loop
LOOP WHILE (INKEY$ = "" AND npoint < nsamp)  ' End program if any key is
                                             ' pressed or specified number
                                             ' of points are acquired
GOTO progend

sample:
'
' Continuously acquire and display all channels while no key is pressed
'
IF scrpoint > 638 THEN                       ' if x coordinate on screen is
                                             ' > pixel 239, then reset trace
                                             ' to first horizontal position (61)
   scrpoint = 61
ELSE
END IF
'
' Clear prior data from the window on retrace.  Note that this is done
' by painting black (color = 0) vertical lines at the current horizontal
' position on the display windows.
'
LINE (scrpoint, 66)-(scrpoint, 198), 0       ' clear previous data from the
                                             ' current horizontal position on
                                             ' the analog signal display window
LINE (scrpoint, 1)-(scrpoint, 58), 0         ' clear previous data from the
                                             ' current horizontal position on
                                             ' the digital input line display
                                             ' windows.
'
' Acquire and display data for all selected analog-input channels
'
FOR channel = 0 TO framelength - 1               ' the first channel of the
                                                 ' multiplexer is zero, and thus
                                                 ' the channel for-to loop is made
                                                 ' to count from 0 to framelength-1
       ' set up the multiplexer bits according to the selected channel
       mux = (channel AND 1) * muxd0 + ((channel AND 2) / 2) * muxd1 + ((channel AND 4) / 4) * muxd2
       GOSUB acquire                             ' now acquire a sample for the
                                                 ' selected channel
       y = INT((2.5 - vout) * 53) + 66           ' compute position of sampled
                                                 ' point for display on the
                                                 ' analog signal display window
                                                 ' to be displayed on the
                                                 ' screen
       PSET (scrpoint, y), channel + 1           ' place point with color depending
                                                 ' on channel being displayed.
                                                 ' Notice that 1 is added to
                                                 ' compensate for offset introduced
                                                 ' above to allow multiplexer to see
                                                 ' channel 1 as channel 0
       acqdata(npoint, channel) = dat            ' store A/D counts for current channel
NEXT channel                                     ' select next channel of frame
'
' Acquire data for digital-input lines
'
GOSUB monitor
acqdata(npoint, 8) = din1
acqdata(npoint, 9) = din2
acqdata(npoint, 10) = din3
acqdata(npoint, 11) = din4
PSET (scrpoint, 10 - 8 * din1), 1               ' place point for input line 1
PSET (scrpoint, 26 - 8 * din2), 2               ' place point for input line 2
PSET (scrpoint, 42 - 8 * din3), 3               ' place point for input line 3
PSET (scrpoint, 58 - 8 * din4), 4               ' place point for input line 4
scrpoint = scrpoint + 1                         ' advance to next horizontal pointnsamp = nsamp + 1                             ' advance samples counter
npoint = npoint + 1                             ' advance acquired sample counter
RETURN




progend:
'
' Store results and leave program
' -------------------------------
OUT prinop, 0                          ' send LPT:Analog! to sleep
SCREEN 0                               ' return to text mode screen
PRINT "STORING ACQUIRED DATA TO DISK"
PRINT #1, "Time", "Analog ch 1", "Analog ch 2", "Analog ch 3", "Analog ch 4",
PRINT #1, "Analog ch 5", "Analog ch 6", "Analog ch 7", "Analog ch 8",
PRINT #1, "Digital ch 1", "Digital ch 2", "Digital ch 3", "Digital ch 4"
PRINT #1, "[seconds]",
FOR channel = 1 TO 8
   PRINT #1, "[A/D counts]",
NEXT channel
FOR channel = 1 TO 4
   PRINT #1, "[Level]",
NEXT channel
PRINT #1, " "
FOR npoints = 0 TO npoint - 1
   PRINT #1, npoints * period,
   FOR chan = 0 TO 11
      PRINT #1, acqdata(npoints, chan),
   NEXT chan
   PRINT #1, ""
NEXT npoints
CLOSE #1
PRINT : PRINT "DATA STORED"
END



acquire:
'
' Acquisition Loop
' ----------------
'
OUT prinop, power + notcs + sclk + mux ' power up but keep CS' deasserted
convert:
OUT prinop, power + mux                ' convert by asserting CS', pull SCLK low
loop1:
FOR clocknum = 1 TO 2                  ' two clock pulses
   OUT prinop, power + sclk + mux
   OUT prinop, power + mux
NEXT clocknum
dat = 0                                ' clear A/D accumulator
FOR clocknum = 11 TO 0 STEP -1         ' clock 12 bits serially
   OUT prinop, power + sclk + mux      ' clock pulse rising edge
   OUT prinop, power + mux             ' clock pulse falling edge
   bit = (INP(prinstat) AND 8) / 8     ' read port and filter bit
   dat = dat + (2 ^ clocknum) * bit    ' accumulate from bit 11 to bit 0
NEXT clocknum                          ' next bit
OUT prinop, power + notcs + mux        ' deassert *CS
vout = dat * 2.5 / ((2 ^ 12 - 1))      ' translate A/D data to Volts
                                       ' the 2.5 represents the maximum input
                                       ' for an analog-input channel without
                                       ' prescaling
RETURN

drawframe:
'
' Draw frames and labels for display screens
' ------------------------------------------
'
' Draw frame for analog signal display
'
LINE (60, 199)-(639, 199)
LINE (60, 65)-(639, 65)
LINE (60, 199)-(60, 65)
LINE (639, 65)-(639, 199)
' label frame for analog signal display
LOCATE 9, 2: PRINT "2.500V";
LOCATE 17, 2: PRINT "1.250V";
LOCATE 25, 2: PRINT "0.000V";
' label digital input channels
COLOR 1
LOCATE 1, 2: PRINT "D#1";
LOCATE 1, 6: PRINT "1";
LOCATE 2, 6: PRINT "0";
COLOR 2
LOCATE 3, 2: PRINT "D#2";
LOCATE 3, 6: PRINT "1";
LOCATE 4, 6: PRINT "0";
COLOR 3
LOCATE 5, 2: PRINT "D#3";
LOCATE 5, 6: PRINT "1";
LOCATE 6, 6: PRINT "0";
COLOR 4
LOCATE 7, 2: PRINT "D#4";
LOCATE 7, 6: PRINT "1";
LOCATE 8, 6: PRINT "0";
COLOR 7
RETURN

monitor:
'
' Monitor Digital Inputs
' ----------------------
'
status = INP(prinstat)                      ' read printer status port
din1 = (status AND 128) / 128               ' evaluate digital input #1
IF din1 = 1 THEN din1 = 0 ELSE din1 = 1     ' invert din1
din3 = (status AND 32) / 32                 ' evaluate digital input #2
din2 = (status AND 64) / 64                 ' evaluate digital input #3
din4 = (status AND 16) / 16                 ' evaluate digital input #4
RETURN

