CLS
PRINT "******************************************************************"
PRINT "* LPTAN8 is a sample QuickBASIC acquisition and display program  *"
PRINT "* for the 8-Channel LPT:Analog! 12-bit A/D printer port adapter. *"
PRINT "*                                                                *"
PRINT "* Channels are acquired and displayed at the maximum rate        *"
PRINT "* supported by your computer running QuickBASIC.                 *"
PRINT "*                                                                *"
PRINT "* (c) Copyright 1996, ADNAV Electronics                          *"
PRINT "*                                                                *"
PRINT "* ADNAV Electronics           (409)292-0988                      *"
PRINT "* 58 Chicory Court, Lake Jackson, TX  77566                      *"
PRINT "******************************************************************"
PRINT
'
' Printer port locations
' ----------------------
CONST prinop = &H378   ' Printer Output Port (could be &H278 or &H3BC)
CONST prinstat = &H379 ' Printer Status Port (could be &H279 or &H3BD)
'
' Define control pin locations
' ----------------------------
CONST power = 12, notcs = 1, sclk = 2, muxd0 = 16, muxd1 = 32, muxd2 = 64
'
' Define Data Acquisition Array and other variables
' -------------------------------------------------
'
DIM bit, dat, scrpoint, mux, status, din1, din2, din3, din4 AS INTEGER
DIM vout AS SINGLE
'
'
' Initialize
' ----------
OUT prinop, power + notcs ' Apply power, *CS high, multiplexer channel 0
PRINT
PRINT "Please press any key to start acquiring data"
PRINT
PRINT "While acquiring, please press any key to end"
PRINT
PRINT "Color coding for digital input channels: ";
COLOR 1: PRINT "1  ";   ' blue
COLOR 2: PRINT "2  ";   ' green
COLOR 3: PRINT "3  ";   ' cyan
COLOR 4: PRINT "4  ";   ' red
COLOR 7                 ' return to white text
PRINT
PRINT
PRINT "Color coding for analog input channels:  ";
COLOR 1: PRINT "1  ";   ' blue
COLOR 2: PRINT "2  ";   ' green
COLOR 3: PRINT "3  ";   ' cyan
COLOR 4: PRINT "4  ";   ' red
COLOR 5: PRINT "5  ";   ' magenta
COLOR 6: PRINT "6  ";   ' brown
COLOR 14: PRINT "7  ";  ' yellow
COLOR 8: PRINT "8  "    ' grey
COLOR 7                 ' return to white text
'
' Idle while waiting for a key to be pressed
DO WHILE INKEY$ = ""'
LOOP
'
' Acquisition and display control
' -------------------------------
SCREEN 8                                     ' CGA graphics mode 640x200
scrpoint = 641                               ' initialize x-axis position
CLS                                          ' clear screen
'
' Continuously acquire and display all channels while no key is pressed
'
DO WHILE INKEY$ = ""
'
' If the current point is the last on the screen (x = 640) then
' clear the screen and return to the first plotting position (x = 60)
'
IF scrpoint > 639 THEN
   CLS
   ' draw graticule for analog signal display
   LINE (60, 198)-(639, 198)
   LINE (60, 66)-(639, 66)
   LINE (60, 133)-(639, 133)
   LINE (60, 198)-(60, 66)
   LINE (639, 66)-(639, 198)
   ' label graticule for analog signal display
   LOCATE 9, 2: PRINT "2.500V";
   LOCATE 17, 2: PRINT "1.250V";
   LOCATE 25, 2: PRINT "0.000V";
   ' label digital input channels
   COLOR 1
   LOCATE 1, 2: PRINT "D#1";
      LOCATE 1, 6: PRINT "1";
      LOCATE 2, 6: PRINT "0";
   COLOR 2
   LOCATE 3, 2: PRINT "D#2";
      LOCATE 3, 6: PRINT "1";
      LOCATE 4, 6: PRINT "0";
   COLOR 3
   LOCATE 5, 2: PRINT "D#3";
      LOCATE 5, 6: PRINT "1";
      LOCATE 6, 6: PRINT "0";
   COLOR 4
   LOCATE 7, 2: PRINT "D#4";
      LOCATE 7, 6: PRINT "1";
      LOCATE 8, 6: PRINT "0";
   COLOR 7
   scrpoint = 60
END IF
' Acquire point for channel 1
mux = 0
GOSUB acquire
y1 = INT((2.5 - vout) * 53) + 66
' Acquire point for channel 2
mux = muxd0
GOSUB acquire
y2 = INT((2.5 - vout) * 53) + 66
' Acquire point for channel 3
mux = muxd1
GOSUB acquire
y3 = INT((2.5 - vout) * 53) + 66
' Acquire point for channel 4
mux = muxd0 + muxd1
GOSUB acquire
y4 = INT((2.5 - vout) * 53) + 66
' Acquire point for channel 5
mux = muxd2
GOSUB acquire
y5 = INT((2.5 - vout) * 53) + 66
' Acquire point for channel 6
mux = muxd0 + muxd2
GOSUB acquire
y6 = INT((2.5 - vout) * 53) + 66
' Acquire point for channel 7
mux = muxd1 + muxd2
GOSUB acquire
y7 = INT((2.5 - vout) * 53) + 66
' Acquire point for channel 8
mux = muxd0 + muxd1 + muxd2
GOSUB acquire
y8 = INT((2.5 - vout) * 53) + 66
' Display data
PSET (scrpoint, y1), 1
PSET (scrpoint, y2), 2
PSET (scrpoint, y3), 3
PSET (scrpoint, y4), 4
PSET (scrpoint, y5), 5
PSET (scrpoint, y6), 6
PSET (scrpoint, y7), 14
PSET (scrpoint, y8), 8
GOSUB monitor
PSET (scrpoint, 10 - 8 * din1), 1
PSET (scrpoint, 26 - 8 * din2), 2
PSET (scrpoint, 42 - 8 * din3), 3
PSET (scrpoint, 58 - 8 * din4), 4
scrpoint = scrpoint + 1             ' increment screen horizontal position
LOOP
SCREEN 0
CLS
END

monitor:
'
' Monitor Digital Inputs
' ----------------------
'
status = INP(prinstat)                      ' read printer status port
din1 = (status AND 128) / 128               ' evaluate digital input #1
IF din1 = 1 THEN din1 = 0 ELSE din1 = 1     ' invert din1
din3 = (status AND 32) / 32                 ' evaluate digital input #2
din2 = (status AND 64) / 64                 ' evaluate digital input #3
din4 = (status AND 16) / 16                 ' evaluate digital input #4
RETURN


acquire:
'
' Acquisition Loop
' ----------------
'
OUT prinop, power + notcs + sclk + mux ' power up but keep CS' deasserted
convert:
OUT prinop, power + mux                ' convert by asserting CS', pull SCLK low
loop1:
FOR clocknum = 1 TO 2                  ' two clock pulses
   OUT prinop, power + sclk + mux
   OUT prinop, power + mux
NEXT clocknum
dat = 0                                ' clear A/D accumulator
FOR clocknum = 11 TO 0 STEP -1         ' clock 12 bits serially
   OUT prinop, power + sclk + mux      ' clock pulse rising edge
   OUT prinop, power + mux             ' clock pulse falling edge
   bit = (INP(prinstat) AND 8) / 8     ' read port and filter bit
   dat = dat + (2 ^ clocknum) * bit    ' accumulate from bit 11 to bit 0
NEXT clocknum                          ' next bit
OUT prinop, power + notcs + mux        ' deassert *CS
vout = dat * 2.5 / ((2 ^ 12))          ' translate A/D data to Volts
RETURN

