CLS
PRINT "******************************************************************"
PRINT "* SCALE.BAS is a sample QuickBASIC program to read a load cell   *"
PRINT "* using the LPT:Analog! model 8c-I/O 12-bit Data Acquisition     *"
PRINT "* adapter for the printer port.                                  *"
PRINT "* Using a two strain-gage load cell, this program allows the PC  *"
PRINT "* to act as an electronic scale calibrated in ounces             *"
PRINT "*                                                                *"
PRINT "* (c) Copyright 1997, ADNAV Electronics                          *"
PRINT "*                                                                *"
PRINT "* ADNAV Electronics           (409)292-0988                      *"
PRINT "* 58 Chicory Court, Lake Jackson, TX  77566                      *"
PRINT "******************************************************************"
PRINT
'
' Printer port locations
' ----------------------
CONST prinop = &H378   ' Printer Output Port (could be &H278 or &H3BC)
CONST princon = &H37A  ' Printer Control Port (could be &H27A or &H3BE)
CONST prinstat = &H379 ' Printer Status Port (could be &H279 or &H3BD)
'
' Define control pin locations
' ----------------------------
CONST notdaclk = 1, notdacs = 2, dadat = 4
CONST power = 12, notcs = 1, sclk = 2, muxd0 = 16, muxd1 = 32, muxd2 = 64
'
' Define variables
' ----------------
DIM dtoa1, dtoa2, clocknum, dat, mux AS INTEGER
'
' Main Routine
' ------------
main:
dtoa1 = 0        ' initialize D/A #1 to zero
dtoa2 = 2500     ' set load cell excitation voltage to 2.5V
GOSUB dtoa       ' update D/A channels
mux = 0          ' set mux to acquire from channel #1
GOSUB acquire    ' acquire one sample from channel #1
offset = 0
'
' Autozero Routine
' ----------------
' This loop increments the offset to the sensor output
' until the output value is close to the A/D's zero
'
PRINT
PRINT "Please press any key to auto-zero"
DO WHILE INKEY$ = ""  ' wait for any key to be pressed
LOOP
BEEP
CLS
PRINT "Please wait ... autozero in progress ...";
DO WHILE vout > .1      ' loop while A/D input > 100 mV
    dtoa1 = dtoa1 + 1   ' increment offset by 1 mV
    IF dtoa1 > 4094 THEN
       BEEP
       PRINT
       PRINT "Autozero not possible."
       PRINT "Please check connections and try again."
       END
    END IF
    GOSUB dtoa          ' update D/A converter
    GOSUB acquire       ' acquire one sample from channel #1
    LOCATE 6, 10: PRINT "D/A output  = "; dtoa1; " mV       "
                        ' display current D/A output
    LOCATE 7, 10: PRINT "A/D reading = "; vout; " V         "
                        ' display current A/D input
LOOP
FOR offloop = 1 TO 100  ' average 100 zero-offset readings
  GOSUB acquire
  offset = offset + vout
  FOR dummy = 1 TO 1000: NEXT dummy
NEXT offloop
offset = offset / 100   ' divide sum by number of samples to
                        ' yield average
CLS
BEEP
PRINT "Please place calibration sample on load cell"
PRINT
INPUT "Please enter weight of calibration sample [ounces] = ", cal
GOSUB weight            ' obtain average voltage corresponding to
                        ' current weight place on scale's load cell
factor = cal / (vweight - offset)  ' calculate linear load cell
                                   ' calibration factor
PRINT "Load cell calibration = "; factor; "[ounces/Volt]"
DO WHILE INKEY$ = ""    ' while no key is pressed keep on sampling
                        ' scale
GOSUB weight            ' obtain average voltage corresponding to
                        ' current weight place on scale's load cell

LOCATE 15, 15
PRINT "weight = ";
PRINT USING "###.#"; (vweight - offset) * factor; ' display measured weight
PRINT " [ounces]           ";
LOOP
END

dtoa:
'
' D/A Output Routine
' ------------------
'
' dtoa1 contains the code for the desired D/A channel 1 output
' dtoa2 contains the code for the desired D/A channel 2 output
'
OUT princon, notdaclk                      ' set D/A clock line low
OUT princon, notdaclk + notdacs            ' assert D/A CS' line
FOR clocknum = 11 TO 0 STEP -1             ' clock 12 bits of D/A channel 1
                                           ' from MSB to LSB
   IF (dtoa1 AND (2 ^ clocknum)) = 0 THEN  ' if current bit converts to 0
      OUT princon, notdacs                 ' then do not set data bit high
                                           ' while causing a clock rising edge
      FOR delay = 1 TO 5: NEXT delay
      OUT princon, notdacs + notdaclk      ' return clock line to low
   ELSE                                    ' if, on the other hand, current
                                           ' bit converts to one
      OUT princon, notdacs + notdaclk + dadat  ' then set data bit high while
                                               ' maintaining the 'CS and clock
                                               ' lines low
      FOR delay = 1 TO 5: NEXT delay
      OUT princon, notdacs + dadat             ' cause clock rising edge to
                                               ' clock data bit in
      FOR delay = 1 TO 5: NEXT delay
      OUT princon, notdacs + notdaclk + dadat  ' return clock line to low
   END IF
NEXT clocknum                               ' next bit
FOR delay = 1 TO 5: NEXT delay
FOR clocknum = 11 TO 0 STEP -1              ' clock 12 bits of D/A channel 2
                                            ' in the same way
   IF (dtoa2 AND (2 ^ clocknum)) = 0 THEN
      OUT princon, notdacs
      FOR delay = 1 TO 5: NEXT delay
      OUT princon, notdacs + notdaclk
   ELSE
      OUT princon, notdacs + notdaclk + dadat
      FOR delay = 1 TO 5: NEXT delay
      OUT princon, notdacs + dadat
      FOR delay = 1 TO 5: NEXT delay
      OUT princon, notdacs + notdaclk + dadat
   END IF
FOR delay = 1 TO 5: NEXT delay
NEXT clocknum
OUT princon, 0                              ' update D/A data by deasserting 'CS
                                            ' under this condition, clock and
                                            ' data states are of no relevance
RETURN

acquire:
'
' Acquisition Loop
' ----------------
'
OUT prinop, power + notcs + sclk + mux ' power up but keep CS' deasserted
convert:
OUT prinop, power + mux                ' convert by asserting CS', pull SCLK low
loop1:
FOR clocknum = 1 TO 2                  ' two clock pulses
   OUT prinop, power + sclk + mux
   OUT prinop, power + mux
NEXT clocknum
dat = 0                                ' clear A/D accumulator
FOR clocknum = 11 TO 0 STEP -1         ' clock 12 bits serially
   OUT prinop, power + sclk + mux      ' clock pulse rising edge
   OUT prinop, power + mux             ' clock pulse falling edge
   bit = (INP(prinstat) AND 8) / 8     ' read port and filter bit
   dat = dat + (2 ^ clocknum) * bit    ' accumulate from bit 11 to bit 0
NEXT clocknum                          ' next bit
OUT prinop, power + notcs + mux        ' deassert *CS
vout = dat * 2.5 / ((2 ^ 12 - 1))      ' translate A/D data to Volts
RETURN

weight:
'
' Scale measurement routine
' -------------------------
vweight = 0                            ' initialize variable
FOR aveloop = 1 TO 100                 ' obtain and average 100 samples
   GOSUB acquire                       ' acquire sample point through A/D
   vweight = vweight + vout            ' update sum
   FOR dummy = 1 TO 100: NEXT dummy    ' time delay loop
NEXT aveloop
vweight = vweight / 100                ' divide sum by number of samples
                                       ' to yield average
RETURN


