CLS
PRINT "*******************************************************************"
PRINT "* THERMOM.BAS is a sample QuickBASIC program to read a thermistor *"
PRINT "* using the LPT:Analog! model 8c-I/O 12-bit Data Acquisition      *"
PRINT "* adapter for the printer port.                                   *"
PRINT "* Using a Radio Shack 271-110A thermistor, this program allows    *"
PRINT "* the PC to act as an electronic thermometer.                     *"
PRINT "*                                                                 *"
PRINT "* (c) Copyright 1997, ADNAV Electronics                           *"
PRINT "*                                                                 *"
PRINT "* ADNAV Electronics           (409)292-0988                       *"
PRINT "* 58 Chicory Court, Lake Jackson, TX  77566                       *"
PRINT "*******************************************************************"
PRINT
'
' Printer port locations
' ----------------------
CONST prinop = &H378   ' Printer Output Port (could be &H278 or &H3BC)
CONST princon = &H37A  ' Printer Control Port (could be &H27A or &H3BE)
CONST prinstat = &H379 ' Printer Status Port (could be &H279 or &H3BD)
'
' Define control pin locations
' ----------------------------
CONST power = 12, notcs = 1, sclk = 2, muxd0 = 16, muxd1 = 32, muxd2 = 64
'
' Define variables
' ----------------
DIM clocknum, dat, mux, tindex AS INTEGER
DIM r(33) AS SINGLE
'
' Define Thermistor Table
' -----------------------
'
' (from back of Radio Shack blister package)
'
r(1) = 329200      ' -50 Degrees C
r(2) = 247500      ' -45 Degrees C
r(3) = 188400      ' -40 Degrees C
r(4) = 144000      ' -35 Degrees C
r(5) = 111300      ' -30 Degrees C
r(6) = 86390       ' -25 Degrees C
r(7) = 67740       ' -20 Degrees C
r(8) = 53390       ' -15 Degrees C
r(9) = 42450       ' -10 Degrees C
r(10) = 33890       ' -5 Degrees C
r(11) = 27280      ' 0 Degrees C
r(12) = 22050      ' +5 Degrees C
r(13) = 17960      ' +10 Degres C
r(14) = 14680      ' +15 Degrees C
r(15) = 12090      ' +20 Degrees C
r(16) = 10000      ' +25 Degrees C
r(17) = 8313       ' +30 Degrees C
r(18) = 6941       ' +35 Degrees C
r(19) = 5828       ' +40 Degrees C
r(20) = 4912       ' +45 Degrees C
r(21) = 4161       ' +50 Degrees C
r(22) = 3537       ' +55 Degrees C
r(23) = 3021       ' +60 Degrees C
r(24) = 2589       ' +65 Degrees C
r(25) = 2229       ' +70 Degrees C
r(26) = 1924       ' +75 Degrees C
r(27) = 1669       ' +80 Degrees C
r(28) = 1451       ' +85 Degrees C
r(29) = 1366       ' +90 Degrees C
r(30) = 1108       ' +95 Degrees C
r(31) = 973.5      ' +100 Degrees C
r(32) = 857.5      ' +105 Degrees C
r(33) = 757.9      ' +110 Degrees C
'
' Main Routine
' ------------
PRINT
INPUT "Please input excitation current [uA] ", current
PRINT
main:
DO WHILE INKEY$ = ""    ' while no key is pressed keep on sampling
                        ' thermistor
GOSUB temperature       ' obtain average voltage corresponding to
                        ' current temperature
res = vtemp / (current * .000001)
FOR tindex = 1 TO 30                     ' Search table to look for
    IF r(tindex) >= res THEN t = tindex  ' temperature range
NEXT tindex
' Find piece-wise linear interpolation
temp = (5 / (r(t + 1) - r(t))) * (res - r(t)) + t * 5 - 55
LOCATE 20, 15
PRINT "temperature = ";
PRINT USING "###.#"; temp; ' display measured temperature
PRINT " [degrees Celsius]           ";
LOCATE 23, 15
PRINT "temperature = ";
PRINT USING "###.#"; (9 / 5) * temp + 32;
PRINT " [degrees Fahrenheit]        ";
LOOP
END

dtoa:
'
' D/A Output Routine
' ------------------
'
' dtoa1 contains the code for the desired D/A channel 1 output
' dtoa2 contains the code for the desired D/A channel 2 output
'
OUT princon, notdaclk                      ' set D/A clock line low
OUT princon, notdaclk + notdacs            ' assert D/A CS' line
FOR clocknum = 11 TO 0 STEP -1             ' clock 12 bits of D/A channel 1
                                           ' from MSB to LSB
   IF (dtoa1 AND (2 ^ clocknum)) = 0 THEN  ' if current bit converts to 0
      OUT princon, notdacs                 ' then do not set data bit high
                                           ' while causing a clock rising edge
      FOR delay = 1 TO 5: NEXT delay
      OUT princon, notdacs + notdaclk      ' return clock line to low
   ELSE                                    ' if, on the other hand, current
                                           ' bit converts to one
      OUT princon, notdacs + notdaclk + dadat  ' then set data bit high while
                                               ' maintaining the 'CS and clock
                                               ' lines low
      FOR delay = 1 TO 5: NEXT delay
      OUT princon, notdacs + dadat             ' cause clock rising edge to
                                               ' clock data bit in
      FOR delay = 1 TO 5: NEXT delay
      OUT princon, notdacs + notdaclk + dadat  ' return clock line to low
   END IF
NEXT clocknum                               ' next bit
FOR delay = 1 TO 5: NEXT delay
FOR clocknum = 11 TO 0 STEP -1              ' clock 12 bits of D/A channel 2
                                            ' in the same way
   IF (dtoa2 AND (2 ^ clocknum)) = 0 THEN
      OUT princon, notdacs
      FOR delay = 1 TO 5: NEXT delay
      OUT princon, notdacs + notdaclk
   ELSE
      OUT princon, notdacs + notdaclk + dadat
      FOR delay = 1 TO 5: NEXT delay
      OUT princon, notdacs + dadat
      FOR delay = 1 TO 5: NEXT delay
      OUT princon, notdacs + notdaclk + dadat
   END IF
FOR delay = 1 TO 5: NEXT delay
NEXT clocknum
OUT princon, 0                              ' update D/A data by deasserting 'CS
                                            ' under this condition, clock and
                                            ' data states are of no relevance
RETURN

acquire:
'
' Acquisition Loop
' ----------------
'
OUT prinop, power + notcs + sclk + mux ' power up but keep CS' deasserted
convert:
OUT prinop, power + mux                ' convert by asserting CS', pull SCLK low
loop1:
FOR clocknum = 1 TO 2                  ' two clock pulses
   OUT prinop, power + sclk + mux
   OUT prinop, power + mux
NEXT clocknum
dat = 0                                ' clear A/D accumulator
FOR clocknum = 11 TO 0 STEP -1         ' clock 12 bits serially
   OUT prinop, power + sclk + mux      ' clock pulse rising edge
   OUT prinop, power + mux             ' clock pulse falling edge
   bit = (INP(prinstat) AND 8) / 8     ' read port and filter bit
   dat = dat + (2 ^ clocknum) * bit    ' accumulate from bit 11 to bit 0
NEXT clocknum                          ' next bit
OUT prinop, power + notcs + mux        ' deassert *CS
vout = dat * 2.5 / ((2 ^ 12 - 1))      ' translate A/D data to Volts
RETURN

temperature:
'
' Thermistor measurement routine
' ------------------------------
vtemp = 0                              ' initialize variable
FOR aveloop = 1 TO 100                 ' obtain and average 100 samples
   GOSUB acquire                       ' acquire sample point through A/D
   vtemp = vtemp + vout                ' update sum
   FOR dummy = 1 TO 100: NEXT dummy    ' time delay loop
NEXT aveloop
vtemp = vtemp / 100                    ' divide sum by number of samples
                                       ' to yield average
RETURN


