// General Description
// -------------------
// Types for mapping I/O pins.
// Default values of the ports.
// Useful constants.


typedef struct {
	unsigned char Freq_SelIn:1 ;			// AD used to select the desired frequency	 (AD)
	unsigned char PWidth_SelIn:1 ;			// AD used to select the desired pulse width (AD)
	unsigned char Filler:4 ;				// Not used
} BitsPORTA ;
#define DefaultPortA	(0x00)
// AD Convertion Clock = FOSC/32	
#define AD_FREQ_SEL_IN		0x80
#define AD_PWIDTH_SEL_IN	0x88


typedef struct {
	unsigned char SensingIn:1 ;				// Sense when the pin goes from 0 to 1  (INPUT)
	unsigned char Filler1:1 ;				// Not used
	unsigned char ReadingEnable:1 ;			// 0 - Enable parameter reading			(OUTPUT)
	unsigned char HighAmplitudePacing:1 ;	// 1 - Before pace for 2*VDD, 0 - VDD	(OUTPUT)
	unsigned char Refractory_SelIn:1 ;		// 0 - 250 ms, 1 - 350 ms				(INPUT)
	unsigned char Mode_SelIn:1 ;			// 0 - VOO, 1 - VVI						(INPUT)
	unsigned char Amplitude_SelIn:1 ;		// 0 - VDD, 1 - 2*VDD					(INPUT)
	unsigned char Filler2:1 ;				// Not used
} BitsPORTB;
#define DefaultPortB	(0x04)

typedef struct {
	unsigned char OscIn:1 ;					// Oscillator	Input
	unsigned char OscOut:1 ;				// Oscillator	Output
	unsigned char Pacing:1 ;				// 1 - For pace							(OUTPUT)
	unsigned char QuickDischarge:1 ;		// 0 - Quick discharge during refractory(OUTPUT)
	unsigned char Filler:4 ;
} BitsPORTC ;
#define DefaultPortC	(0x08)

typedef enum { ALERT, WAIT_END_ALERT, REFRACTORY, NOISE_WINDOW, FAILURE } STATE ;
typedef enum { TOUT, SENSE, UNKNOWN_EVENT } EVENT ;



extern BitsPORTA bPORTA ;
extern bank1 BitsPORTA bTRISA ;
extern BitsPORTA bCopyPORTA ;
extern BitsPORTB bPORTB ;
extern BitsPORTB bCopyPORTB ;
extern bank1 BitsPORTB bTRISB ;
extern BitsPORTC bPORTC ;
extern BitsPORTC bCopyPORTC ;
extern bank1 BitsPORTC bTRISC ;


// Constants
#define TIMER_BASE	(1000.0/32768.0)	// TIMER 0,030517578125 ms per tick
#define T_100ms		(unsigned int)(100.0 / TIMER_BASE )
#define T_250ms		(unsigned int)(250.0 / TIMER_BASE )
#define T_350ms		(unsigned int)(350.0 / TIMER_BASE )




// Periods (in ms) 
#define T_1500ms	(unsigned int)(1500.0 / TIMER_BASE )
#define T_1200ms	(unsigned int)(1200.0 / TIMER_BASE )
#define T_1091ms	(unsigned int)(1091.0 / TIMER_BASE )
#define T_1000ms	(unsigned int)(1000.0 / TIMER_BASE )
#define T_923ms		(unsigned int)( 923.0 / TIMER_BASE )
#define T_857ms		(unsigned int)( 857.0 / TIMER_BASE )
#define T_750ms		(unsigned int)( 750.0 / TIMER_BASE )
#define T_667ms		(unsigned int)( 667.0 / TIMER_BASE )
#define T_600ms		(unsigned int)( 600.0 / TIMER_BASE )
#define T_545ms		(unsigned int)( 545.0 / TIMER_BASE )
#define T_500ms		(unsigned int)( 500.0 / TIMER_BASE )
#define T_429ms 	(unsigned int)( 429.0 / TIMER_BASE )


// Pulse Width (in ms)
#define	T_0_125ms	(unsigned int)(	0.125 / TIMER_BASE )
#define	T_0_250ms	(unsigned int)(	0.250 / TIMER_BASE )
#define	T_0_375ms	(unsigned int)(	0.375 / TIMER_BASE )
#define	T_0_500ms	(unsigned int)(	0.500 / TIMER_BASE )
#define	T_0_625ms	(unsigned int)(	0.625 / TIMER_BASE )
#define	T_0_750ms	(unsigned int)(	0.750 / TIMER_BASE )
#define	T_0_875ms	(unsigned int)(	0.875 / TIMER_BASE )
#define	T_1_000ms	(unsigned int)(	1.000 / TIMER_BASE )
#define	T_1_125ms	(unsigned int)(	1.125 / TIMER_BASE )
#define	T_1_250ms	(unsigned int)(	1.250 / TIMER_BASE )
#define	T_1_375ms	(unsigned int)(	1.375 / TIMER_BASE )
#define	T_1_500ms	(unsigned int)(	1.500 / TIMER_BASE )



// Macros
#define Sleep()			{ asm("SLEEP"); asm("NOP"); }
