;********************************************************
; FILENAME = SB001M.asm
; SHOCK BOX CONTROL PROGRAM
;(c) 2003 David Prutchi and Michael Norris
;********************************************************

	LIST	p=16C77,F=INHX8M,R=DEC

	#include "P16C77.INC"

	__CONFIG _HS_OSC&_WDT_OFF&_PWRTE_ON&_CP_OFF&_BODEN_OFF
	__IDLOCS 0100			;version 1.0
	ERRORLEVEL 1,-302

	#include "sbdef6.inc"		;MEMORY DEFINITIONS FILE
;*********************************************************
; MACRO DEFINITIONS
;*********************************************************
BANK0	MACRO
	BCF	STATUS,RP0
	BCF	STATUS,RP1
	ENDM

BANK1	MACRO
	BSF	STATUS,RP0
	BCF	STATUS,RP1
	ENDM

BANK2	MACRO
	BCF	STATUS,RP0
	BSF	STATUS,RP1
	ENDM

BANK3	MACRO
	BSF	STATUS,RP0
	BSF	STATUS,RP1
	ENDM

;***********************************************************
; RESET VECTOR
;***********************************************************
	ORG	0x00
	GOTO	STARTUP
;***********************************************************
;INTERRUPT VECTOR
;***********************************************************
	ORG	0x04		;SAVE THE REGISTERS
	MOVWF	W_TEMP
	SWAPF	STATUS,W
	CLRF	STATUS
	MOVWF	STATUS_TEMP

	BANK0
	MOVF	PCLATH,W	;SAVE THE UPPER PROGRAM COUNTER
	MOVWF	PCLATH_TEMP
	MOVF	FSR,W		;SAVE THE FSR REGISTER 
	MOVWF	FSR_TEMP

	BCF	IRP_BIT		;SAVE THE FSR BANK SELECT BIT
	BTFSC	STATUS,IRP
	BSF	IRP_BIT

;ISR STARTS HERE
	BTFSC	PIR1,TMR1IF	;WAS IT A TIMER 1 INTERRUPT?
	GOTO	TIMER1_INT	;YES THEN DO THE TIMER 1 INTERRUPT ROUTINE

	BTFSS	INTCON,INTF	;WAS IT AN EXTERNAL INTERRUPT? (RTS)
	GOTO	EXIT_INT	;NO THEN WHAT WAS IT!!?

;*****************************************************************
; EXTERNAL INTERRUPT ROUTINE
;*****************************************************************
	BANK0
	BCF	INTCON,INTF	;CLEAR THE INTERRUPT FLAG
	MOVLW	RX_BUF		;GET THE RX BUFFER ADDRESS
	MOVWF	RX_PTR		;INIT THE PTR

	MOVF	RCREG,W		;CLEAR THE RX BUFFER
	MOVF	RCREG,W

	MOVLW	d'13'		;WAIT 7.8125ms * 13 = 101.56ms
	MOVWF	TEMP1		;FOR THE CLOCK TO STABILIZE
	MOVF	TMR1H,W
	ADDWF	TEMP1,F
WAIT_INT
	MOVF	TMR1H,W
	SUBWF	TEMP1,W
	BTFSS	_Z
	GOTO	WAIT_INT

	CALL	GET_RX_DATA
	GOTO	EXIT_INT
		
;*****************************************************************
; TIMER 1 INTERRUPT SERVICE ROUTINE
;*****************************************************************
TIMER1_INT
	BCF	T1CON,TMR1ON		;TIMER 1 OFF
	BCF	PIR1,TMR1IF		;CLEAR THE INTERRUPT FLAG

	MOVLW	LOW TIMER1_QRTR_SEC	;RELOAD THE TIMER
	MOVWF	TMR1L
	MOVLW	HIGH TIMER1_QRTR_SEC
	MOVWF	TMR1H

	BSF	T1CON,TMR1ON		;TIMER 1 ON
	CALL	DO_ONE_QTR_SEC		;DO THE 1/4 SECOND ROUTINE

	DECF	QRTR_SEC_TIMER,F	;1/4 SEC TIMER = 0
	BTFSS	_Z
	GOTO	EXIT_INT		;NO THEN EXIT

	MOVLW	d'4'			;YES THEN RELOAD THE 1/4 SECOND TIMER
	MOVWF	QRTR_SEC_TIMER
	CALL	DO_ONE_SECOND		;AND DO THE 1 SECOND ROUTINE

	DECF	SEC60_TIMER,F		;DEC THE 1 SEC TIMER
	BTFSS	_Z
	GOTO	EXIT_INT

	MOVLW	d'60'
	MOVWF	SEC60_TIMER
	CALL	DO_ONE_MIN
			
EXIT_INT
	BANK0				;RESTORE THE REGISTERS
	MOVF	FSR_TEMP,W
	MOVWF	FSR

	BCF	STATUS,IRP
	BTFSC	IRP_BIT
	BSF	STATUS,IRP
	
	MOVF	PCLATH_TEMP,W
	MOVWF	PCLATH
	SWAPF	STATUS_TEMP,W
	MOVWF	STATUS
	SWAPF	W_TEMP,F
	SWAPF	W_TEMP,W

	RETFIE
;***************************************************************
; RX MESSAGE STATE TABLE
; ALL ROUTINES SHOULD END WITH A RETURN
;***************************************************************
RX_MSG_TABLE
	ADDWF	PCL,F
	GOTO	GET_COMMAND
	GOTO	GET_DATA_COUNT
	GOTO	GET_DATA
	GOTO	GET_CHK_LOW
	GOTO	GET_CHK_HIGH

#define	GET_COMMAND_INDEX	0
#define	GET_DATA_COUNT_INDEX	1
#define	GET_DATA_INDEX		2
#define	GET_CHK_LOW_INDEX	3
#define	GET_CHK_HIGH_INDEX	4


;***************************************************************
; LOOK-UP TABLE FOR THE TMR0 VALUES USED FOR THE 11 PHASE PULSE WIDTHS
;***************************************************************
GET_TIMER_VAL

	ADDWF	PCL,F
	RETLW	0	;0
	RETLW   d'15'	;1ms
	RETLW	d'31'	;2ms
	RETLW	d'47'	;3ms
	RETLW   d'62'	;4ms
	RETLW	d'78'	;5ms
	RETLW	d'94'	;6ms
	RETLW	d'109'	;7ms
	RETLW	d'125'	;8ms
	RETLW	d'141'	;9ms
	RETLW	d'156'	;10ms

;***************************************************************
; COMMAND EXECUTE TABLE
; THESE ROUTINES SHOULD END WITH A RETURN TO EXIT PROPERLY
;***************************************************************
COMMAND_EX_TABLE
	ADDWF	PCL,F
	GOTO	RESEND		;0	RESEND THE LAST REPLY
	GOTO	READ_IMPED	;1	READ 8 IMPED AND AVERAGE
	GOTO	READ_CAP	;2	READ 8 CAP VOLTAGES AND AVERAGE
	GOTO	CHARGE_TO	;3	CHARGE TO XX(lsb) XX(msb) VOLTS
	GOTO	SET_PW		;4	SET PHASE 1 AND PHASE 2 PW ( XX XX )
	GOTO	SET_VOLT	;5	SET THE CHARGE TO VOLTAGE ( XX XX )
	GOTO	SET_POL		;6	SET THE POLARITY, 1 = POSITIVE
	GOTO	DEFIB_ARM	;7	ARM THE DEFIB
	GOTO	DO_A_SHOCK	;8	SHOCK! (MUST BE ARMED FIRST)
	GOTO	DO_A_REFORM	;9	CHARGE TO MAX V, MAX PW, DUMP
				;	AND CLEAR THE REFORM TIME.
	GOTO	DO_A_DUMP	;A	DUMP ALL CHARGE
	GOTO	CLEAR_TOTAL	;B	CLEAR THE SHOCK TOTAL
	GOTO	REPORT_TOTAL	;C	SEND THE SHOCK TOTAL
	GOTO	REPORT_TIME	;D	SEND TIME SINCE POWER-UP (mins)
	GOTO	REPORT_REFORM	;E	TIME SINCE LAST REFORM (mins)
	GOTO	SHOCK_AT	;F	SHOCK AT VOLTS, PW1, PW2, POLARITY (MUST BE ARMED FIRST)
	GOTO	DO_MAX_SLEEP	;0x10	GOTO MAX SLEEP MODE (min. current usage)
				;	THIS MODE IS CLEARED BY ANY RX COMM.

	GOTO	READ_RUNAWAY	;0x11	SEND BACK THE LAST RUNAWAY VOLTAGE MEAS.
				;	USED FOR DEBUG ONLY


#define	LAST_COMMAND	0x11
#define	SHOCK_COMMAND	0x08
	
;***************************************************************
#define	AD_CONFIG_ANALOG	b'00000100'	;RA0=A, RA1=A, RA3=A, ALL OTHERS=D, VREF=VDD
#define	AD_CONFIG_DIG		b'00000110'	;ALL DIGITAL
;***********************************************************
; INITIALIZATION ROUTINE
;***********************************************************
STARTUP
	CLRF	INTCON		;ALL INTERRUPTS OFF

	BANK0
	MOVLW	0x00		;CLEAR ALL PORT DATA REGISTERS
	MOVWF	PORTA
	MOVWF	PORTA_DATA
	MOVWF	PORTB
	MOVWF	PORTB_DATA
	MOVWF	PORTC
	MOVWF	PORTC_DATA
	MOVWF	PORTD
	MOVWF	PORTD_DATA
	MOVWF	PORTE
	MOVWF	PORTE_DATA

	CALL	SW_5V_OFF	;SWITCH THE +5V OFF (ACTIVE LOW SIGNAL)

	BSF	CTS_NOT		;CTS OFF (ACTIVE LOW)
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

	BSF	TX_OUT		;LEAVE THE TX LINE HIGH
	MOVF	PORTC_DATA,W
	MOVWF	PORTC


;SETUP THE INTERNAL A/D CONVERTER
	CLRF	ADCON0
	MOVLW	b'11000000'	;CONVERSION TIME = FOSC/32
	MOVWF	ADCON0		;SELECT A/D CHAN. = 0
				;CLEAR GO/*DONE
				;TURN OFF THE A/D CONVERTER FOR NOW
	BANK1
	MOVLW	AD_CONFIG_DIG	;ALL DIGITAL 	
	MOVWF	ADCON1

;SETUP ALL THE PORT DATA DIRECTION REGISTERS
	MOVLW	PORTA_DATA_DIR	;PORTA
	MOVWF	TRISA
	MOVLW	PORTB_DATA_DIR	;PORTB
	MOVWF	TRISB
	MOVLW	PORTC_DATA_DIR	;PORTC
	MOVWF	TRISC
	MOVLW	PORTD_DATA_DIR	;PORTD
	MOVWF	TRISD
	MOVLW	PORTE_DATA_DIR	;PORTE
	MOVWF	TRISE

;SETUP THE OPTION REGISTER
	MOVLW	b'10000111'	;NO PULLUPS, INTERRUPT RB0 ON FALLING EDGE				
	MOVWF	OPTION_REG	;TMR0 CLK IS INTERNAL	
				;PRESCALER ASSIGNED TO TMR0
				;PRESCALE 1:256

;CLEAR THE PERIPERHAL INTERRUPT ENABLE REGISTERS (for now)
	CLRF	PIE1
	CLRF	PIE2
	CLRF	PCON

;CLEAR ALL PERIPHERAL INTERRUPT FLAGS
	BANK0
	CLRF	PIR1
	CLRF	PIR2

;SETUP THE 32KHZ TIMER OSCILLATOR
	CLRF	T1CON			;CLEAR ALL BITS IN THE TIMER CONTROL REG.
					;1:1 PRESCALE (1/32.768KHZ * 65536 = 2 secs max.)
					;TIMER 1 IS OFF

	BSF	T1CON,TMR1CS		;EXTERNAL CLOCK INPUT
	BSF	T1CON,NOT_T1SYNC	;CLOCK SYNC OFF (active low)
	BSF	T1CON,T1OSCEN		;ENABLE THE 32KHZ OSCILLATOR

;SETUP THE INTERNAL RAM TIMERS
	MOVLW	0x04
	MOVWF	QRTR_SEC_TIMER
	MOVLW	d'60'
	MOVWF	SEC60_TIMER

;CLEAR MEMORY
	CLRF	TEMP1			;CLEAR THE SCRATCHPAD AREA
	CLRF	TEMP2
	CLRF	TEMP3
	CLRF	TEMP4
	CLRF	TEMP5
	CLRF	TEMP6
	
	CLRF	DAC_LSB			;CLEAR THE DAC DATA STORAGE AREA
	CLRF	DAC_MSB

	CLRF	AD_1_LSB		;CLEAR THE A/D DATA STORAGE AREA
	CLRF	AD_1_MSB
	CLRF	AD_2_LSB
	CLRF	AD_2_MSB	

	CLRF	PWR_ON_1		;CLEAR THE MINUTE TIMER
	CLRF	PWR_ON_2
	CLRF	PWR_ON_3
	CLRF	PWR_ON_4

	CLRF	REF_1			;CLEAR THE REFORM TIMER
	CLRF	REF_2
	CLRF	REF_3
	CLRF	REF_4

	CLRF	SYS_INT_FLAGS		;CLEAR THE SYSTEM FLAGS
	BSF	POWER_UP		;SIGNAL TO THE DO_ONE_SECOND ROUTINE
					;THAT THE POWER JUST CAME UP.
	CLRF	HV_STATE_FLAGS

	CLRF	RX_COMM_STATE
	CLRF	COMM_STATUS
	CLRF	COMM_TIMER
	CLRF	RX_FLAGS

	CLRF	BLINK_ON_TIME
	CLRF	BLINK_OFF_TIME

;CLEAR THE COMM BUFFERS
	MOVLW	BUF_SPACE_LEN		;GET THE NUMBER OF BYTES TO BE CLEARED
	MOVWF	TEMP1			;STORE IT IN TEMP1

	MOVLW	TX_BUF			;GET THE STARTING ADDRESS (1ST BUFFER)
	MOVWF	FSR			;PLACE ADDRESS IN PTR

NEXT_MEM
	CLRF	INDF			;CLEAR MEMORY LOCATION @FSR
	INCF	FSR,F			;PTR += 1
	DECFSZ	TEMP1,F			;BYTE COUNT -= 1, BYTE COUNT = 0?
	GOTO	NEXT_MEM		;NO THEN DO AGAIN

	MOVLW	RX_BUF			;SETUP THE RX AND TX BUFFER POINTERS
	MOVWF	RX_PTR
	MOVWF	FSR
	MOVLW	REPLY_BUF
	MOVWF	REPLY_PTR

	MOVLW	HV_CHARGE_TIME_INIT	;TIME IN 0.25 SEC INCREMENTS
	MOVWF	HV_CHARGE_TIME

	CALL	RESET_LATCH		;RESET THE LATCHES FOR THE MANUAL SWITCHES
	CALL	CHARGE_SW_CAP_FULL

;WAIT 50ms FOR THE 32KHZ CLOCK TO STABILIZE
	CLRF	TMR0

WAIT_FOR_CLOCK
	MOVF	TMR0,W		;EACH TMR0 COUNT = 256us
	SUBLW	d'195'	
	BTFSC	_C		;RESULT POSITVE
	GOTO	WAIT_FOR_CLOCK	;YES THEN DO AGAIN

;SETUP TIMER1
	MOVLW	LOW TIMER1_QRTR_SEC
	MOVWF	TMR1L
	MOVLW	HIGH TIMER1_QRTR_SEC
	MOVWF	TMR1H

	CLRF	PIR1
	BSF	T1CON,TMR1ON		;TIMER 1 = ON

;SETUP THE SPI PORT
	MOVLW	SSPCON_INIT
	MOVWF	SSPCON

	BANK1
	BCF	SSPSTAT,SMP
	BSF	SSPSTAT,CKE
	BANK0

;SETUP TIMER2
	MOVLW	b'00000011'		;1:16 PRESCALE, TIMER OFF,1:1 POST
	MOVWF	T2CON
	BSF	T2CON,TMR2ON		;TIMER2 ON
	CLRF	PR2

;SETUP THE USART
	BANK1
	MOVLW	d'25'			;2400 BAUD
	MOVWF	SPBRG	

	CLRF	TXSTA			;TRANSMITS OFF, LOW SPEED

	BANK0
	MOVLW	b'10000000'		;ENABLE THE SERIAL PORT
	MOVWF	RCSTA

	MOVF	RCREG,W			;READ THE RX PORT TWICE TO CLEAR IT
	MOVF	RCREG,W

;ENABLE RX AND CLEAR ANY OVERRUN ERRORS

	BCF	RCSTA,CREN		
	BSF	RCSTA,CREN		;THIS CLEARS THE FERR AND OERR BITS

	BANK1	
	CLRF	PIE1
	CLRF	PIE2
	
;ENABLE PERIPHERAL INTERRUPTS	
	BSF	PIE1,TMR1IE		;ENABLE TIMER 1 INTERRUPTS

	BANK0
	CLRF	INTCON			;CLEAR ALL INTERRUPT FLAGS
	BSF	INTCON,INTE		;ENABLE INTERRUPT ON RB0 (RTS)
	BSF	INTCON,PEIE		;ENABLE PERIPHERAL INTERRUPTS
	BSF	INTCON,GIE		;ENABLE GLOBAL INTERRUPTS	
	
;*****************************************************************
; MAIN STARTS HERE
;*****************************************************************
MAIN	
SLEEP_LOOP
	SLEEP
	NOP
	GOTO	SLEEP_LOOP
;*******************************************************************
; END OF MAIN
;*******************************************************************

;*******************************************************************
; SUBROUTINES START HERE
;*******************************************************************

;*******************************************************************
; RESEND THE REPLY BUFFER
; COPY THE REPLY BUFFER TO THE TRANSMIT BUFFER AND REQUEST A TX
;*******************************************************************
RESEND
	BANK0
	MOVLW	REPLY_BUF
	MOVWF	TEMP1		;SOURCE

	MOVLW	TX_BUF
	MOVWF	TEMP2		;DESTINATION

	MOVF	REPLY_MSG_LEN,W
	MOVWF	TEMP3

RESEND_LOOP
	MOVF	TEMP1,W		;GET THE SOURCE DATA PTR
	MOVWF	FSR	
	
	MOVF	INDF,W		;GET THE DATA
	MOVWF	TEMP4

	MOVF	TEMP2,W		;GET THE DEST PTR
	MOVWF	FSR

	MOVF	TEMP4,W		;STORE THE DATA AT THE DEST.
	MOVWF	INDF

	DECFSZ	TEMP3,F
	GOTO 	RESEND_LOOP

	CALL	TRANSMIT_TX_BUF

	RETURN

;*******************************************************************
; SEND BACK THE AVERAGE OF 8 IMPEDANCE VALUES
;*******************************************************************
READ_IMPED
	BANK0
	CALL	TEST_LEAD_IMPED

	BANK0
	MOVLW	TX_BUF
	MOVWF	FSR

	CLRF	TX_MSG_LEN	;CLEAR THE TX MESSAGE LENGTH

	MOVF	RX_COMMAND,W	;COMMAND
	MOVWF	INDF
	INCF	TX_MSG_LEN,F	;MESSAGE LEN = 1
	INCF	FSR,F

	MOVLW	d'2'		;DATA COUNT
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F	;LEN = 2

	MOVF	TEMP2,W		;GET THE AVERAGE (LSB)
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F	;LEN = 3

	MOVF	TEMP3,W		;GET THE AVERAGE (MSB)
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F	;LEN = 4
	
	CALL	CALC_TX_CHKSUM
	CALL	COPY_TX_TO_REPLY
	CALL	TRANSMIT_TX_BUF

	RETURN

;*******************************************************************
; SEND BACK THE AVERAGE OF 8 CAP. VOLTAGE READINGS
;*******************************************************************
READ_CAP
	BANK0
	CALL	READ_CAP_AVG8	;GET AVERAGED 8 READINGS
				;RESULT RETURNED IN TEMP2,TEMP3 (LSB,MSB)

BUILD_CAP_MSG
	CLRF	TX_MSG_LEN
	MOVLW	TX_BUF
	MOVWF	FSR

	MOVF	RX_COMMAND,W	;COMMAND
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVLW	0x02		;DATA COUNT
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVF	TEMP2,W		;LSB DATA
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVF	TEMP3,W		;MSB DATA
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	CALL	CALC_TX_CHKSUM	
	CALL	COPY_TX_TO_REPLY
	CALL	TRANSMIT_TX_BUF

	RETURN

;*******************************************************************
; CHARGE TO XX(lsb)XX(msb) VOLTS USING TOLERANCE MIN XXXX(lsb,msb) 
; MAX XXXX(LSB,MSB) AND HOLD
;*******************************************************************
CHARGE_TO
	BANK0
	MOVLW	RX_BUF	;GET THE START OF THE RX BUFFER
	ADDLW	0x02	;ADD TWO TO GET PAST THE HEADER
	MOVWF	FSR	
	
	MOVF	INDF,W		;GET THE LSB CHARGE TO VALUE
	MOVWF	HV_CNTRLV_LOW
	
	INCF	FSR,F
	MOVF	INDF,W		;GET THE MSB CHARGE TO VALUE
	MOVWF	HV_CNTRLV_HIGH

	INCF	FSR,F
	MOVF	INDF,W
	MOVWF	HV_MIN_LOW	;LSB MIN

	INCF	FSR,F
	MOVF	INDF,W
	MOVWF	HV_MIN_HIGH	;MSB MIN

	INCF	FSR,F
	MOVF	INDF,W
	MOVWF	HV_MAX_LOW	;LSB MAX

	INCF	FSR,F
	MOVF	INDF,W
	MOVWF	HV_MAX_HIGH	;MSB MAX
	

	CALL	INIT_CHARGE	;START CHARGING
	MOVWF	TEMP4		;SAVE THE RETURN VALUE
;********************************
;BUILD THE REPLY MESSAGE
;********************************
	CLRF	TX_MSG_LEN

	MOVLW	TX_BUF
	MOVWF	FSR

	MOVF	RX_COMMAND,W	;COMMAND
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVLW	d'1'		;DATA LENGTH
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVF	TEMP4,W		;DATA
	MOVWF	INDF
	INCF	TX_MSG_LEN,F

	CALL	CALC_TX_CHKSUM
	CALL	TRANSMIT_TX_BUF

	RETURN

;*******************************************************************
; SET THE PHASE 1 (lsb) AND PHASE 2 (msb) PULSE WIDTHS
;*******************************************************************
SET_PW
	BANK0
	MOVLW	RX_BUF		;STEP OVER HEADER
	ADDLW	0x02
	MOVWF	FSR

	MOVF	INDF,W		;GET PHASE 1 VALUE
	MOVWF	HV_PHASE1_PW
	INCF	FSR,F
	MOVF	INDF,W		;GET PHASE 2 VALUE
	MOVWF	HV_PHASE2_PW

	CALL	ECHO_RX

	RETURN

;*******************************************************************
; SET THE CHARGE VOLTAGE XXXX (lsb,msb) 12 BITS
;*******************************************************************
SET_VOLT
	BANK0
	MOVLW	RX_BUF		;STEP OVER THE HEADER
	ADDLW	0x02
	MOVWF	FSR

	MOVF	INDF,W		;GET LSB
	MOVWF	HV_CNTRLV_LOW
	INCF	FSR,F
	MOVF	INDF,W
	MOVWF	HV_CNTRLV_HIGH	;GET MSB

	CALL	ECHO_RX

	RETURN

;*******************************************************************
; SET THE PHASE 1 WAVEFORM POLARITY, 1 = POSITIVE, ELSE NEG.
;*******************************************************************
SET_POL
	BANK0
	MOVLW	RX_BUF
	ADDLW 	0x02
	MOVWF	FSR

	MOVF	INDF,W		;GET THE POLARITY
	MOVWF	HV_POLARITY

	CALL	ECHO_RX

	RETURN

;*******************************************************************
; ARM THE DEFIB (FOR SAFETY) THE SHOCK COMMAND MUST BE PROCEEDED BY
; THIS COMMAND.
;*******************************************************************
DEFIB_ARM
	BANK0
	BSF	HV_ARM
	CALL	ECHO_RX

	RETURN

;*******************************************************************
; DO A SHOCK
; CHARGE IF NEEDED AND SHOCK IF THE ARM COMMAND WAS PREVIOUSLY SENT
;*******************************************************************
DO_A_SHOCK
	BANK0
	BTFSS	HV_READY	;ARE WE CHARGED AND READY
	CALL	INIT_CHARGE	;NO THEN CHARGE

	BSF	HV_SHOCK

	RETURN

;***********************************************************************
; SEND THE DATA COLLECTED DURING THE SHOCK
;***********************************************************************
SEND_PHASE_DATA
	CLRF	TX_MSG_LEN
	MOVLW	TX_BUF
	MOVWF	FSR

	MOVLW	SHOCK_COMMAND		;ECHO THE SHOCK COMMAND
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVLW	d'9'		;DATA COUNT
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVF	TEMP4,W		;STATUS BYTE
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

;PHASE 1
	MOVF	P1_STARTL,W	;LSB STARTING VOLTAGE, PHASE 1
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVF	P1_STARTH,W	;MSB STARTING VOLTAGE, PHASE 1
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVF	P1_ENDL,W	;LSB ENDING VOLTAGE, PHASE 1
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVF	P1_ENDH,W	;MSB ENDING VOLTAGE, PHASE 1
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F


;PHASE 2
	MOVF	P2_STARTL,W	;LSB PHASE 2 STARTING VOLTAGE
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVF	P2_STARTH,W	;MSB PHASE 2 STARTING VOLTAGE
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVF	P2_ENDL,W	;LSB PHASE 2 ENDING VOLTAGE
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVF	P2_ENDH,W	;MSB PHASE 2 ENDING VOLTAGE
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	CALL	CALC_TX_CHKSUM
	CALL	COPY_TX_TO_REPLY

	CALL	TRANSMIT_TX_BUF

	RETURN
;*******************************************************************
; DO A REFORM, REFROM THE CAPS. AT MAX PW AND MAX V., CLEAR THE REFORM
; TIMER.
;******************************************************************
DO_A_REFORM
	
	BANK0
	BSF	HV_MAX		;CHARGE TO MAX VOLTAGE
	CALL	INIT_CHARGE	;THIS WILL CLEAR THE REFORM TIMER
	
	CALL	ECHO_RX
	RETURN

;*******************************************************************
; DUMP THE CHARGE, LEAVE THE DUMP CIRCUIT ___ON___
;*******************************************************************
DO_A_DUMP
	BANK0
	CALL	DO_DUMP		;DUMP THREE TIMES TO BE SURE
	CALL	DO_DUMP
	CALL	DO_DUMP
	CALL	ECHO_RX

	RETURN

;*******************************************************************
; CLEAR THE TOTAL NUMBER OF SHOCKS
;*******************************************************************
CLEAR_TOTAL
	BANK0
	CLRF	TOTAL_1
	CLRF	TOTAL_2
	CALL	ECHO_RX
	RETURN

;*******************************************************************
; REPORT THE TOTAL NUMBER OF SHOCKS
;*******************************************************************
REPORT_TOTAL
	CLRF	TX_MSG_LEN
	MOVLW	TX_BUF
	MOVWF	FSR

	MOVLW	RX_COMMAND	;COMMAND
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVLW	d'2'		;DATA COUNT
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVF	TOTAL_1,W	;LSB DATA
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVF	TOTAL_2,W	;MSB DATA
	MOVWF	INDF
	INCF	TX_MSG_LEN,F

	CALL	CALC_TX_CHKSUM	
	CALL	TRANSMIT_TX_BUF

	RETURN

;*******************************************************************
; REPORT THE TIME IN MINUTES SINCE POWER UP
;*******************************************************************
REPORT_TIME
	CLRF	TX_MSG_LEN
	MOVLW	TX_BUF
	MOVWF	FSR

	MOVLW	RX_COMMAND	;COMMAND
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVLW	d'4'		;DATA COUNT
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVF	PWR_ON_1,W	;DATA	1
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVF	PWR_ON_2,W	;DATA	2
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVF	PWR_ON_3,W	;DATA	3
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVF	PWR_ON_4,W	;DATA	4
	MOVWF	INDF
	INCF	TX_MSG_LEN,F

	CALL	CALC_TX_CHKSUM	
	CALL	TRANSMIT_TX_BUF

	RETURN

;*******************************************************************
; REPORT THE TIME SINCE THE LAST REFORM
;*******************************************************************
REPORT_REFORM
	CLRF	TX_MSG_LEN
	MOVLW	TX_BUF
	MOVWF	FSR

	MOVLW	RX_COMMAND	;COMMAND
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVLW	d'4'		;DATA COUNT
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVF	REF_1,W		;DATA	1
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVF	REF_2,W		;DATA	2
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVF	REF_3,W		;DATA	3
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVF	REF_4,W		;DATA	4
	MOVWF	INDF
	INCF	TX_MSG_LEN,F

	CALL	CALC_TX_CHKSUM	
	CALL	TRANSMIT_TX_BUF

	RETURN

;*******************************************************************
; SHOCK AT: VOLTAGE, PW1, PW2, POLARITY
;*******************************************************************
SHOCK_AT
	MOVLW	RX_BUF		;GET THE RX BUFFER ADDRESS
	ADDLW	0x02		;STEP PAST THE HEADER
	MOVWF	FSR

	MOVF	INDF,W		;GET THE VOLTAGE LSB
	MOVWF	HV_CNTRLV_LOW	
	INCF	FSR,F
	MOVF	INDF,W
	MOVWF	HV_CNTRLV_HIGH	;GET THE VOLTAGE MSB
	INCF	FSR,F

	MOVF	INDF,W		;GET PW1
	MOVWF	HV_PHASE1_PW
	INCF	FSR,F

	MOVF	INDF,W		;GET PW 2
	MOVWF	HV_PHASE2_PW
	INCF	FSR,F

	MOVF	INDF,W		;GET THE POLARITY
	MOVWF	HV_POLARITY
	INCF	FSR,F

	MOVF	INDF,W		;GET THE MIN AND MAX LIMITS
	MOVWF	HV_MIN_LOW
	INCF	FSR,F

	MOVF	INDF,W
	MOVWF	HV_MIN_HIGH
	INCF	FSR,F

	MOVF	INDF,W
	MOVWF	HV_MAX_LOW
	INCF	FSR,F

	MOVF	INDF,W
	MOVWF	HV_MAX_HIGH

	BTFSS	HV_READY	;ARE WE CHARGED AND READY?
	CALL	INIT_CHARGE	;NO THEN CHARGE
	BSF	HV_SHOCK	;REQUEST A SHOCK
				;THE REPLY WILL BE SENT BACK BY THE 
				;DO_SHOCK ROUTINE
	
	RETURN

;**********************************************************************
; MAX SLEEP MODE - SWITCH TO MAX SLEEP MODE.
; THE OPERATIONS DONE EVERY 1/4 OF A SECOND ARE MINIMIZED TO THE FOLLOWING:
; 1. DEC THE 1 SEC COUNTER, IF = 0 THEN
;    a. TEST THE RUNAWAY CAP.
;    b. UPDATE THE 60 SEC TIMERS
; ELSE
;    a. GO BACK TO SLEEP
; 2. MAX SLEEP MODE IS CANCELLED BY A RESET OR BY ANY RX COMM.
;**********************************************************************
DO_MAX_SLEEP
	MOVF	HV_STATE_FLAGS,W	;ARE ALL ACTIONS = 0?
	BTFSC	_Z
	GOTO	SET_LP_MODE		;YES THEN CONTINUE

;BUILD THE ERROR MESSAGE
	CLRF	TX_MSG_LEN		;CLEAR MSG LENGTH
	MOVLW	TX_BUF			;GET THE BUFFER ADDRESS
	MOVWF	FSR

	MOVF	RX_COMMAND,W		;GET THE COMMAND
	MOVWF	INDF			;PUT IT IN THE BUFFER
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVLW	d'1'			;GET THE DATA COUNT
	MOVWF	INDF			;PUT IT IN THE BUFFER
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVLW	0xFF			;PUT A -1 (error) IN THE BUFFER
	MOVWF	INDF
	INCF	TX_MSG_LEN,F

	CALL	CALC_TX_CHKSUM		;CALC THE CHECKSUM
	CALL	COPY_TX_TO_REPLY	;COPY THE TRANSMIT BUFFER
	CALL	TRANSMIT_TX_BUF		;AND SEND THE TRANSMIT DATA
	RETURN
	
SET_LP_MODE	
	CALL	DO_DUMP			;MAKE SURE WE ARE DISCHARGED
	CALL	DO_DUMP

	CALL	CHARGE_LED_OFF		;TURN THE POWER AND LEDS OFF
	CALL	READY_LED_OFF
	CALL	SW_5V_OFF
	CALL	SW_24V_OFF

	CALL	CHARGE_SW_CAP_OFF	;SWITCH THE CAP CHARGING OFF
	BSF	MAX_SLEEP_FLAG		;SET THE MAX SLEEP FLAG
	CALL	ECHO_RX			;ECHO THE RX MESSAGE

	RETURN

;*******************************************************************
; READ BACK THE RUNAWAY CAP VOLTAGE, THIS CAP IS USED FOR DEBUG PURPOSES
; ONLY
;*******************************************************************
READ_RUNAWAY
	BANK0
	
	CLRF	TX_MSG_LEN	;CLEAR THE MESSAGE LENGTH
	MOVLW	TX_BUF		;GET THE TX BUFFER ADDRESS
	MOVWF	FSR

	MOVF	RX_COMMAND,W	;STORE THE COMMAND IN THE BUFFER
	MOVWF	INDF
	INCF	FSR,F
	INCF	TX_MSG_LEN,F

	MOVLW	d'1'
	MOVWF	INDF		;STORE THE REPLY DATA LENGTH
	INCF	TX_MSG_LEN,F

	INCF	FSR,F
	MOVF	RUNAWAY_V,W	;STORE THE DATA
	MOVWF	INDF
	INCF	TX_MSG_LEN,F

	CALL	CALC_TX_CHKSUM		;CALC THE CHECKSUM FOR THE TX MESSAGE
	CALL	COPY_TX_TO_REPLY	;COPY THE TRANSMIT BUFFER TO REPLY BUFFER
	CALL	TRANSMIT_TX_BUF		;SEND THE TRANSMIT BUFFER (MESSAGE)
	
	RETURN

;*******************************************************************
; TEST TO SEE IF THE CHARGING OF THE CAP FAILED, AND IF SO, THEN BLINK
; THE CHARGE LED.
;*******************************************************************
TEST_CHARGE_FAIL
	BTFSS	CHARGE_FAIL		;CHARGE FAIL BIT SET?
	RETURN				;NO THEN EXIT

	MOVF	BLINK_ON_TIME,W		;GET THE ON TIME
	BTFSS	_Z			;IS IT ZERO?
	GOTO	DEC_ON_TIME		;NO THEN DEC THE OFF TIME

	MOVF	BLINK_OFF_TIME,W	;GET THE OFF TIME
	BTFSS	_Z			;IS IT ZERO?
	GOTO	DEC_OFF_TIME

CHARGE_ON
	CALL	CHARGE_LED_ON		;YES THEN TURN ON THE LED
	MOVLW	BLINK_TIME		;GET THE TIME TO BLINK
	MOVWF	BLINK_ON_TIME
	RETURN

DEC_ON_TIME
	DECF	BLINK_ON_TIME,F		;ON TIME -= 1
	BTFSS	_Z			;TIME = 0?
	RETURN				;NO THEN WAIT

	CALL	CHARGE_LED_OFF		;YES THEN TURN THE LED OFF
	MOVLW	BLINK_TIME		;LOAD THE OFF TIME
	MOVWF	BLINK_OFF_TIME
	RETURN				;AND EXIT

DEC_OFF_TIME
	DECF	BLINK_OFF_TIME,F	;OFF TIME -= 1
	BTFSS	_Z			;OFF TIEM = 0?
	RETURN				;NO THEN WAIT
	GOTO	CHARGE_ON		;YES THEN TURN THE LED ON
;*******************************************************************
; GET THE FIRST CHARACTER, THE COMMAND
;*******************************************************************
GET_COMMAND
	MOVLW	LAST_COMMAND	;MAX COMMAND NUMBER + 1
	ADDLW	0x01
	SUBWF	RX_DATA,W	;F-W (COMMAND - (MAX + 1))
	BTFSC	_C		;RESULT POSITIVE?
	RETLW	0xFF		;YES, THEN THIS IS AN ERROR, RETURN -1

	MOVF	RX_DATA,W
	MOVWF	RX_COMMAND

	MOVLW	GET_DATA_COUNT_INDEX	;NEXT STATE
	MOVWF	RX_COMM_STATE

	RETLW	0			;RETURN A ZERO, ALL OK
;*******************************************************************
; GET THE NUMBER OF DATA BYTES
;*******************************************************************
GET_DATA_COUNT
	MOVLW	0x0D		;DATA COUNT > 12
	SUBWF	RX_DATA,W
	BTFSC	_C		;RESULT POSITIVE?
	RETLW	0xFF		;YES THEN THIS IS AN ERROR

	MOVF	RX_DATA,W	;THIS IS THE DATA COUNT
	MOVWF	RX_DATA_COUNT
	MOVWF	RX_COUNT_COPY

	ADDLW	0x02		;THIS IS THE TOTAL MESSAGE LENGTH + 2 BYTES 
	MOVWF	REC_MSG_LEN	;FOR THE CHKSUM - (THE COMMAND AND DATA COUNT)

	MOVLW	GET_DATA_INDEX
	MOVWF	RX_COMM_STATE
	
	RETLW	0

;*******************************************************************
; GET THE DATA
;*******************************************************************
GET_DATA
	DECFSZ	RX_COUNT_COPY,F		;COUNT -= 1, COUNT = 0?
	RETLW	0			;NO THEN EXIT

	MOVLW	GET_CHK_LOW_INDEX	;YES THEN GET THE CHKSUM
	MOVWF	RX_COMM_STATE
	RETLW	0

;*******************************************************************
;GET THE FIRST BYTE OF THE CHKSUM
;*******************************************************************
GET_CHK_LOW
	MOVF	RX_DATA,W
	MOVWF	RX_CHK_LOW

	MOVLW	GET_CHK_HIGH_INDEX
	MOVWF	RX_COMM_STATE

	RETLW	0
;********************************************************************
; GET THE 2ND BYTE OF THE CHKSUM AND TEST THE MESSAGE FOR THE CORRECT
; CHECKSUM VALUE. SET THE CHKSUM_OK FLAG IF OK.
;********************************************************************
GET_CHK_HIGH
	MOVF	RX_DATA,W
	MOVWF	RX_CHK_HIGH
	
	BSF	LAST_CHAR		;YES THEN SET LAST CHAR FLAG,
	RETLW	0x00			;RETURN A 0x00

;*******************************************************************
; CALCULATE THE TX MESSAGE CHECKSUM AND PLACE IT AT THE END OF THE 
; MESSAGE. ADD 2 TO TX_MSG_LEN.
;*******************************************************************
CALC_TX_CHKSUM
	BANK0

	MOVLW	TX_BUF
	MOVWF	FSR

	MOVF	TX_MSG_LEN,W
	MOVWF	TEMP1		;MSG LENGTH

	CLRF	TEMP2		;CALCULATED CHKSUM LOW
	CLRF	TEMP3		;CALCULATED CHKSUM HIGH

NEXT_TX_CHAR
	MOVF	INDF,W
	ADDWF	TEMP2,F

	BTFSC	_C		;WAS THERE A CARRY?
	INCF	TEMP3,F		;YES THEN ADD 1 TO THE HIGH BYTE

	INCF	FSR,F		;PTR++

	DECFSZ	TEMP1,F		;HAVE WE DONE ALL THE CHARACTERS?
	GOTO	NEXT_TX_CHAR	;NO THEN DO NEXT

	MOVF	TEMP2,W		;LOW CHECK SUM
	MOVWF	INDF
	INCF	FSR,F

	MOVF	TEMP3,W		;HIGH CHKSUM
	MOVWF	INDF

	INCF	TX_MSG_LEN,F	;ADD TWO TO MESSAGE LENGTH
	INCF	TX_MSG_LEN,F

	RETURN

;*******************************************************************
; READ THE CAP VOLTAGE 8 TIMES AND AVERAGE THE READINGS
;*******************************************************************
READ_CAP_AVG8
	BANK0

	BTFSS	ENABLE_5V	;5 VOLTS ON? 	ACTIVE LOW
	GOTO	RC_2		;YES IT IS ON

	CALL	SW_5V_ON	;NO THEN SWITCH IT ON
	CALL	WAIT_50MS	;AND WAIT FOR IT TO STABILIZE
RC_2
	CLRF	TEMP2
	CLRF	TEMP3

	MOVLW	d'8'
	MOVWF	TEMP6

READ_CAP_8
	CALL	READ_AD_1	;GET AN CAP READING 

	MOVF	AD_1_MSB,W	;ADD THE 2 HIGH BYTES
	ADDWF	TEMP3,F

	MOVF	AD_1_LSB,W	;ADD THE 2 LOW BYTES
	ADDWF	TEMP2,F

	BTFSC	_C		;WAS THERE A CARRY?
	INCF	TEMP3,F		;ADD 1 TO THE HIGH BYTE

	DECFSZ	TEMP6,F		;HAVE WE TAKEN 8 READINGS?
	GOTO	READ_CAP_8	;NO THEN DO AGAIN

;DIVIDE THE RESULT BY 8
	MOVLW	0x03		;DIVIDE BY 2, THREE TIMES
	MOVWF	TEMP6

DIV_8_CAP
	BCF	_C
	RRF	TEMP3,F			;ROTATE RIGHT HIGH BYTE
	RRF	TEMP2,F			;ROTATE RIGHT LOW BYTE

	DECFSZ	TEMP6,F
	GOTO	DIV_8_CAP

	RETURN

;*******************************************************************
; READ THE CAP VOLTAGE 4 TIMES AND AVERAGE THE READINGS
;*******************************************************************
READ_CAP_AVG4
	BANK0

	BTFSS	ENABLE_5V	;5 VOLTS ON? 	ACTIVE LOW
	GOTO	RC_3		;YES IT IS ON

	CALL	SW_5V_ON	;NO THEN SWITCH IT ON
	CALL	WAIT_50MS	;AND WAIT FOR IT TO STABILIZE
RC_3
	CLRF	TEMP2
	CLRF	TEMP3

	MOVLW	d'4'		;TAKE 4 READINGS
	MOVWF	TEMP6

READ_CAP_4
	CALL	READ_AD_1	;GET AN CAP READING 

	MOVF	AD_1_MSB,W	;ADD THE 2 HIGH BYTES
	ADDWF	TEMP3,F

	MOVF	AD_1_LSB,W	;ADD THE 2 LOW BYTES
	ADDWF	TEMP2,F

	BTFSC	_C		;WAS THERE A CARRY?
	INCF	TEMP3,F		;ADD 1 TO THE HIGH BYTE

	DECFSZ	TEMP6,F		;HAVE WE TAKEN 8 READINGS?
	GOTO	READ_CAP_4	;NO THEN DO AGAIN

	BCF	_C
	RRF	TEMP3,F		;DIVIDE BY 2 (HIGH THEN LOW)		
	RRF	TEMP2,F			

	BCF	_C		;DIVIDE BY 2
	RRF	TEMP3,F
	RRF	TEMP2,F

	RETURN

;*******************************************************************
; TEST THE RX CHECKSUM
;*******************************************************************
TEST_RX_CHKSUM
	BANK0

	MOVLW	RX_BUF
	MOVWF	FSR

	MOVF	RX_MSG_LEN,W
	MOVWF	TEMP1		;MSG LENGTH
	DECF	TEMP1,F		;MSG LEN - CHKSUM = LEN
	DECF	TEMP1,F

	CLRF	TEMP2		;CALCULATED CHKSUM LOW
	CLRF	TEMP3		;CALCULATED CHKSUM HIGH

NEXT_CHK_CHAR
	MOVF	INDF,W
	ADDWF	TEMP2,F

	BTFSC	_C		;WAS THERE A CARRY?
	INCF	TEMP3,F		;YES THEN ADD 1 TO THE HIGH BYTE

	INCF	FSR,F		;PTR++

	DECFSZ	TEMP1,F		;HAVE WE DONE ALL THE CHARACTERS?
	GOTO	NEXT_CHK_CHAR	;NO THEN DO NEXT

	MOVF	RX_CHK_LOW,W	;YES THEN COMPARE THE CHKSUM LOW
	SUBWF	TEMP2,W
	BTFSS	_Z		;WAS IT THE SAME?
	RETLW	0xFF		;NO THEN RETURN ERROR

	MOVF	RX_CHK_HIGH,W	;YES THEN CHECK THE HIGH CHKSUM
	SUBWF	TEMP3,W
	BTFSS	_Z		;WAS IT THE SAME?
	RETLW	0xFF		;NO THEN RETURN AN ERROR

	RETLW	0x00		;ELSE RETURN OK
	
;*******************************************************************
; PARSE THE RX MESSAGE
;*******************************************************************
PARSE_RX_DATA
	MOVF	RX_COMM_STATE,W
	CALL	RX_MSG_TABLE
	
	IORLW	0x00	;NEEDED TO SET Z FLAG
	BTFSS	_Z	;RESULT = 0?
	RETLW	0xFF	;NO THEN RETURN ERROR
	RETLW	0	;ELSE RETURN OK

;*******************************************************************
; GET THE RX MESSAGE
;*******************************************************************
GET_RX_DATA
	BANK0
	BCF	CTS_NOT		;SET CTS on (active low)
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

	CLRF	RX_MSG_LEN

	MOVLW	COMM_TIMEOUT
	MOVWF	COMM_TIMER
	MOVF	TMR1H,W
	ADDWF	COMM_TIMER,F


WAIT_FOR_RX
	MOVF	COMM_TIMER,W	;GET THE TIMEOUT VALUE
	SUBWF	TMR1H,W		;HAVE WE TIMED OUT?
	BTFSC	_Z
	GOTO	RESET_RX_STATE	;YES THEN THIS IS AN ERROR

	MOVF	RX_PTR,W	;GET THE CURRENT BUFFER ADDRESS
	MOVWF	FSR		

	BTFSS	PIR1,RCIF	;HAVE WE RECEIVED A CHARACTER YET?
	GOTO	WAIT_FOR_RX	;NO THEN WAIT

	BTFSC	RCSTA,FERR	;WAS THERE A FRAMING ERROR?
	GOTO	RESET_RX_STATE	;YES THEN RESET RX

	MOVF	RCREG,W		;NO, THEN GET THE RX CHARACTER
	MOVWF	RX_DATA		;SAVE A COPY OF IT

	BTFSC	RCSTA,OERR	;WAS THERE AN OVERRUN ERROR?
	GOTO	RESET_RX_STATE	;YES THEN RESET RX

	CALL	PARSE_RX_DATA	;RETURNS A ZERO IF OK
	IORLW	0x00		;NEEDED TO SET Z FLAG
	BTFSS	_Z		;RESULT = 0?
	GOTO	RESET_RX_STATE	;NO THEN ERROR

	MOVF	RX_DATA,W	;ELSE SAVE THE DATA IN THE BUFFER
	MOVWF	INDF
	
	INCF	RX_PTR,F	;ELSE ADD 1 TO THE BUFFER POINTER
	INCF	RX_MSG_LEN,F	;ADD 1 TO THE MSG LENGTH

	BTFSS	LAST_CHAR	;WAS THIS THE LAST CHARACTER WE WERE EXPECTING?
	GOTO	WAIT_FOR_RX	;NO THEN GET ANOTHER CHARACTER.
	

	CALL	TEST_RX_CHKSUM	;RETURNS A 0 FOR OK
	IORLW	0x00		;NEEDED TO SET Z FLAG
	BTFSS	_Z		;RETURN VAL = 0?
	GOTO	RESET_RX_STATE	;NO THEN ERROR

	BTFSS	MAX_SLEEP_FLAG	;ARE WE IN MAX SLEEP MODE
	GOTO	EXEC_RX_COMMAND	;NO THEN EXECUTE THE COMMAND

	BCF	MAX_SLEEP_FLAG		;CLEAR THE SLEEP FLAG
	CALL	RESET_LATCH		;CLEAR THE FRONT PANEL SWITCH LATCHES
	CALL	CHARGE_SW_CAP_FULL	;START CHARGING THE FRONT PANEL CAP

EXEC_RX_COMMAND
	MOVF	RX_COMMAND,W
	CALL	COMMAND_EX_TABLE


RESET_RX_STATE
	MOVF	RCREG,W		;MAKE SURE THE RX REGISTER IS EMPTY
	MOVF	RCREG,W

	BCF	RCSTA,CREN	;CLEAR ANY OVERRUN ERRORS
	BSF	RCSTA,CREN

	MOVLW	GET_COMMAND_INDEX
	MOVWF	RX_COMM_STATE

	CLRF	RX_FLAGS
	CLRF	COMM_TIMER
		
EXIT_RX
	BSF	CTS_NOT		;CTS OFF (ACTIVE LOW)
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

	BCF	INTCON,INTF	;MAKE SURE WE DIDN'T GET MULIPLE INTS.
	RETURN

;*******************************************************************
; TRANSMIT TX BUF
;*******************************************************************
TRANSMIT_TX_BUF
	BANK0
	BCF	TX_DONE	
	
	MOVLW	TX_BUF		;LOAD THE BUFFER ADDRESS
	MOVWF	FSR		;INTO FSR

	MOVF	TX_MSG_LEN,W	;COPY THE MSG LEN INTO TEMP1
	MOVWF	TEMP1

	MOVF	INDF,W		;GET THE 1ST CHAR
	MOVWF	TXREG

	BANK1
	BSF	TXSTA,TXEN

	BANK0
	INCF	FSR,F
	DECF	TEMP1,F

WAIT_FOR_TXIF
	BTFSS	PIR1,TXIF
	GOTO	WAIT_FOR_TXIF

	MOVF	INDF,W		;GET THE CHARACTER
	MOVWF	TXREG		;AND SEND IT

	INCF	FSR,F		;PTR++

	DECFSZ	TEMP1,F		;MSG COUNT -= 1. COUNT = 0?
	GOTO	WAIT_FOR_TXIF	;NO THEN DO AGAIN

	BCF	TX_RQST		;CLEAR THE TX REQUEST
	BCF	TX_DONE		;AND THE TX DONE FLAGS


WAIT_FOR_TXIF2
	BTFSS	PIR1,TXIF	;IS TXIF SET?
	GOTO	WAIT_FOR_TXIF2	;NO THEN WAIT

	BANK1

WAIT_FOR_TRMT	
	BTFSS	TXSTA,TRMT	;IS THE LAST CHAR GONE?
	GOTO	WAIT_FOR_TRMT	;NO THEN WAIT

	BCF	TXSTA,TXEN	;YES THEN DISABLE TRANSMITS
	BANK0

	RETURN
	
	
;*******************************************************************
;RESET THE MANUAL CHARGE AND DEFIB. LATCHES
;*******************************************************************
RESET_LATCH
	BANK0
	BSF	SWITCH_RESET		;SET THE RESET LINE HIGH
	MOVF	PORTB_DATA,W
	MOVWF	PORTB

	BCF	SWITCH_RESET		;SET THE RESET LINE LOW
	MOVF	PORTB_DATA,W
	MOVWF	PORTB

	RETURN

;********************************************************************
; TURN ON THE CHARGING OF THE CAP FOR THE MANUAL SWITCHES
;********************************************************************
CHARGE_SW_CAP_ON
	BANK0

	BSF	SWITCH_CHARGE		;SET THE LINE HIGH
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

;LOAD THE OFF TIME
	MOVLW	SW_CHRG_ON_INIT		;LOAD THE ON TIME
	MOVWF	SW_CHRG_ON_TIME
	
	CLRF	SW_CHRG_OFF_TIME	;MAKE SURE THE OFF TIME IS ZERO

	RETURN

;********************************************************************
; TURN ON THE CHARGING OF THE CAP FOR THE MANUAL SWITCHES (FULL CHARGE)
;********************************************************************
CHARGE_SW_CAP_FULL
	BANK0

	BSF	SWITCH_CHARGE		;SET THE LINE HIGH
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

;LOAD THE OFF TIME
	MOVLW	SW_CHRG_FULL		;LOAD THE ON TIME
	MOVWF	SW_CHRG_ON_TIME
	
	CLRF	SW_CHRG_OFF_TIME	;MAKE SURE THE OFF TIME IS ZERO

	RETURN

;***********************************************************************
; TURN OFF THE CHARGING OF THE CAP FOR THE MANUAL SWITCHES
;***********************************************************************
CHARGE_SW_CAP_OFF
	BANK0

	BCF	SWITCH_CHARGE		;SET THE LINE LOW
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

;LOAD THE ON TIME
	MOVLW	SW_CHRG_OFF_INIT		;LOAD THE OFF TIME
	MOVWF	SW_CHRG_OFF_TIME

	CLRF	SW_CHRG_ON_TIME	;MAKE SURE THE OFF TIME IS ZERO

	RETURN

;*************************************************************************
; TEST THE CAP CHARGE STATE AND DEC THE CURRENT STATE TIME BY 1.
; IF THE CURRENT STATE TIME = 0, CHANGE THE STATE.
; THIS ROUTINE IS CALLED EVERY 0.25 SECS.
;*************************************************************************
TEST_CAP_TIME
	BANK0
	MOVF	SW_CHRG_ON_TIME,W	;CAP CHARGE TIME = 0?
	BTFSC	_Z
	GOTO	TEST_CAP_OFF		;YES, THEN NOT ON, CHECK TIME OFF

	DECF	SW_CHRG_ON_TIME,F	;ELSE DEC THE ON TIME
	MOVF	SW_CHRG_ON_TIME,W	;CAP CHARGE ON TIME = 0?
	BTFSC	_Z
	CALL	CHARGE_SW_CAP_OFF	;YES THEN STOP CHARGING
	GOTO	EXIT_ONE_QTR

TEST_CAP_OFF
	MOVF	SW_CHRG_OFF_TIME,W	;CAP CHARGE OFF TIME = 0?
	BTFSS	_Z
	GOTO	TEST_OFF_ZERO
	CALL	CHARGE_SW_CAP_ON	;YES THEN THIS IS AN ERROR SO START CHARGING.
	GOTO	EXIT_ONE_QTR		;THE ONLY WAY TO GET HERE IS IF BOTH THE OFF
					;AND ON CAP TIMERS ARE = 0. THAT IS ILLEGAL, SO
					;START CHARGING AND EXIT.
TEST_OFF_ZERO
	DECF	SW_CHRG_OFF_TIME,F	;ELSE DEC THE OFF TIME
	MOVF	SW_CHRG_OFF_TIME,W	;OFF TIME = 0?
	BTFSC	_Z
	CALL	CHARGE_SW_CAP_ON	;YES THEN START CHARGING THE CAP.
					;AND EXIT
EXIT_ONE_QTR
	RETURN

;************************************************************************
; TEST_MAN_SW
;************************************************************************
TEST_MAN_SW
	BANK0
	BTFSS	DEFIB_SW		;HAS THE MAN DEFIB SWITCH BEEN PRESSED?
	GOTO	TEST_CHARGE		;NO THEN TEST THE MAN CHARGE SWITCH

	BTFSS	HV_READY		;YES ARE WE READY TO SHOCK?
	GOTO	EXIT_TEST_MAN		;NO THEN JUST EXIT

	BSF	HV_ARM			;SET THE ARMED AND SHOCK BITS
	BSF	HV_SHOCK		
	CALL	DO_SHOCK		;AND DO A SHOCK	
	GOTO	EXIT_TEST_MAN

TEST_CHARGE
	BTFSS	CHARGE_SW		;HAS THE MAN CHARGE SWITCH BEEN PRESSED
	RETURN				;NO THEN DO NOTHING

	BSF	HV_MAX			;SIGNAL WE WANT TO CHARGE TO MAX
	CALL	INIT_CHARGE		;AND CHARGE

EXIT_TEST_MAN	
	CALL	CHARGE_SW_CAP_FULL	;RECHARGE THE CAP
	CALL	RESET_LATCH		;RESET THE LATCH
	RETURN

;*************************************************************************
; DUMP ALL THE REMAINING CHARGE
;*************************************************************************
DO_DUMP
	BANK0
	BTFSC	AC_ON_IN	;AC POWER ON?
	GOTO	DUMP2		;YES THEN CONTINUE

	CALL	SW_5V_ON	;NO THEN SWITCH THE 5V ON
	
DUMP2	
	CALL	SW_24V_OFF	;24 VOLT POWER OFF (should be off, make sure)
	CALL	HV_SUPPLY_OFF	;DISABLE THE HIGH VOLTAGE SUPPLY
	
	BCF	DUMP_DISABLE	;START THE CHARGE DUMP
	BSF	DUMP_ENABLE
	MOVF	PORTB_DATA,W
	MOVWF	PORTB

	CLRF	HV_STATE_FLAGS	;CLEAR ALL THE FLAGS
	

	CLRF	TEMP1		;WRITE 0 VOLTS TO HV CHARGER
	CLRF	TEMP2
	CALL	WRITE_TO_DAC

	CALL	WAIT_50MS	;WAIT 50ms

	BCF	DUMP_ENABLE	;STOP THE DUMP
	BSF	DUMP_DISABLE
	MOVF	PORTB_DATA,W
	MOVWF	PORTB

	MOVLW	d'33'
	MOVWF	TEMP1

DUMP_W1
	DECFSZ	TEMP1,F		;WAIT 100us FOR THE STOP TO OVER
	GOTO	DUMP_W1

	BCF	DUMP_DISABLE	;ALL DUMP LINES OFF
	MOVF	PORTB_DATA,W
	MOVWF	PORTB

	BTFSS	AC_ON_IN	;AC POWER ON?
	CALL	SW_5V_OFF	;NO THEN SWITCH THE +5V OFF

	CALL	READY_LED_OFF	;TURN OFF BOTH CHARGE AND READY (CHARGE MAY HAVE BEEN TURN ON
	CALL	CHARGE_LED_OFF  ;BY THE DO_ONE_SECOND ROUTINE
	CALL	RESET_LATCH	;CLEAR THE PUSH BUTTON LATCHES
	RETURN

;*************************************************************************
; DO A SHOCK!!
;*************************************************************************
DO_SHOCK
	BANK0

	BTFSC	HV_ARM		;SYSTEM ARMED?
	GOTO	TEST_SHOCK	;YES THEN MAKE SURE WE GOT A SHOCK REQUEST

	MOVLW	0xFF		;NO THEN WE GOT A SHOCK WITHOUT ARM COMMAND
	GOTO	SHOCK_EXIT

TEST_SHOCK
	BTFSC	HV_SHOCK	;DO WE HAVE THE SHOCK COMMAND?
	GOTO	SHOCK_TEST_LEAD	;YES THEN TEST THE LEAD
	
	MOVLW	0xFE		;NO THEN HOW DID WE GET HERE?
	GOTO	SHOCK_EXIT	
	
	
SHOCK_TEST_LEAD
	CALL 	TEST_LEAD_IMPED	;CHECK THE LEAD IMPED ONE LAST TIME
	IORLW	0x00		;WILL RETURN 0 IF OK
	BTFSC	_Z		;VAL = 0?
	GOTO	SHOCK_PWR_ON	;YES THEN POWER UP

	MOVLW	0xFD		;NO THEN RETURN A STATUS OF -3
	GOTO	SHOCK_EXIT

SHOCK_PWR_ON
	BTFSS	ENABLE_5V	;5V POWER ON? (ACTIVE LOW)
	GOTO	STOP_CHARGE	;YES THEN GO

	CALL	SW_5V_ON	;NO THEN POWER UP AND WAIT 50ms
	CALL	WAIT_50MS


STOP_CHARGE
	CALL	HV_SUPPLY_OFF	;DISABLE THE HV SUPPLY

	MOVF	HV_PHASE1_PW,W	;LOAD THE USER SET PULSE WIDTHS
	MOVWF	SYS_PHASE1_PW

	MOVF	HV_PHASE2_PW,W
	MOVWF	SYS_PHASE2_PW

	MOVF	HV_POLARITY,W	;AND POLARITY
	MOVWF	SYS_POLARITY

	BTFSS	HV_MAX		;IS THIS A MAX ENERGY SHOCK (FROM FRONT PANEL)
	GOTO	FIRE    	;NO THEN TEST THE POLARITY

	MOVLW	d'5'		;YES THEN LOAD THE MAX PULSE WIDTH OF 5ms
	MOVWF	SYS_PHASE1_PW
	MOVWF	SYS_PHASE2_PW	

	MOVLW	DEFAULT_POLARITY
	MOVWF	SYS_POLARITY

FIRE
	CLRF	TEMP6
	CALL	CLOSE_HV_SWITCHES	;FIRE THE IGBTS, COLLECT PHASE DATA
	MOVWF	TEMP6

SHOCK_EXIT
	MOVWF	TEMP4
	CALL	SEND_PHASE_DATA

	CALL	READY_LED_OFF
	CALL	SW_24V_OFF	;SWITCH THE 24 VOLTS OFF

	INCF	TOTAL_1,F	;ADD 1 TO THE SHOCK TOTAL
	BTFSC	_Z		;DID IT ROLL OVER TO ZERO
	INCF	TOTAL_2,F	;YES THEN ADD 1 TO THE HIGH BYTE

	CALL	DO_DUMP		;DUMP REMAINING ENERGY

	RETURN

;*************************************************************************
; CLOSE THE SWITCHES (IGBTS) TO DO A SHOCK
; RETURN AN ERROR IF NEEDED
; ELSE RETURN A 0
;*************************************************************************
CLOSE_HV_SWITCHES
	BANK1
	MOVF	OPTION_REG,W	;SETUP TMR0 FOR A 1:64 PRESCALER
	ANDLW	b'11111000'
	IORLW	b'00000101'
	MOVWF	OPTION_REG
	BANK0

	CLRF	P1_STARTL	;CLEAR THE PHASE STARTING AND ENDING VOLTAGES
	CLRF	P1_STARTH
	CLRF	P1_ENDL
	CLRF	P1_ENDH

	CLRF	P2_STARTL
	CLRF	P2_STARTH
	CLRF	P2_ENDL
	CLRF	P2_ENDH

	MOVF	SYS_POLARITY,W
	BTFSC	_Z			;POLARITY = 0
	GOTO	FIRE_2_1		;YES THEN FIRE 2 THEN 1
	

;***************************************************************************
; FIRE PHASE 1 AND THEN PHASE 2 OF THE H-BRIDGE
;***************************************************************************
FIRE_1_2
	CALL	READ_CAP_AVG8		;GET A CAP VOLTAGE READING
	MOVF	TEMP2,W			;STORE THE READING
	MOVWF	P1_STARTL
	MOVF	TEMP3,W
	MOVWF	P1_STARTH
	
	MOVF	SYS_PHASE1_PW,W		;GET THE PHASE 1 PULSE WIDTH
	CALL	GET_TIMER_VAL		;CONVERT IT TO A TIMER 1 VALUE
	MOVWF	PHASE_TVAL		;SAVE IT 
	IORLW	0x00
	BTFSS	_Z			;IS IT ZERO?
	CALL	FIRE_1			;NO THEN FIRE PHASE 1
					;THERE IS A 100us DELAY AT THE END OF THE FIRE_1
					;ROUTINE

	CALL	READ_CAP_AVG4		;READ THE CAP. VOLTAGES (4 AVERAGES)

	MOVF	TEMP2,W			;GET THE LSB RESULT
	MOVWF	P1_ENDL			;SAVE IT AS THE PHASE 1 ENDING VOLTAGE (lsb)
	MOVF	TEMP3,W			;GET THE MSB RESULT
	MOVWF	P1_ENDH			;SAVE IT AS THE PHASE 1 ENDING VOLTAGE	(msb)

	MOVF	SYS_PHASE2_PW,W		;GET THE PHASE 2 PULSE WIDTH
	CALL	GET_TIMER_VAL		;CONVERT IT TO A TIMER 1 VALUE
	MOVWF	PHASE_TVAL
	IORLW	0x00
	BTFSS	_Z			;IS IT ZERO?
	CALL	FIRE_2			;NO THEN FIRE PHASE 2

	CALL	READ_CAP_AVG8		;GET A CAP VOLTAGE READING
	MOVF	TEMP2,W			;STORE THE READING
	MOVWF	P2_ENDL
	MOVF	TEMP3,W
	MOVWF	P2_ENDH

	BANK1
	MOVF	OPTION_REG,W	;SETUP TMR0 FOR A 1:256 PRESCALER
	IORLW	b'00000111'
	MOVWF	OPTION_REG
	BANK0
	
	RETLW	0		;*NOT IMPLEMENTED* AN ERROR IN ONE OF THE PHASES
				;WOULD BE RETURNED VALUE OTHER THAN 0. THIS RETURN VALUE
				;WOULD BE SENT BACK IN THE PHASE DATA AS THE STATUS
				;BYTE.

;***************************************************************************
; FIRE PHASE 2 AND THEN PHASE 1 OF THE H-BRIDGE
;***************************************************************************
FIRE_2_1
	CALL	READ_CAP_AVG8		;GET A CAP VOLTAGE READING
	MOVF	TEMP2,W			;STORE THE READING
	MOVWF	P2_STARTL
	MOVF	TEMP3,W
	MOVWF	P2_STARTH
	
	MOVF	SYS_PHASE2_PW,W		;GET THE PHASE 2 PULSE WIDTH
	CALL	GET_TIMER_VAL		;CONVERT IT TO A TIMER0 VALUE
	MOVWF	PHASE_TVAL		;SAVE IT
	IORLW	0x00
	BTFSS	_Z			;IS IT ZERO?
	CALL	FIRE_2			;NO THEN FIRE PHASE 2

	CALL	READ_CAP_AVG4		;GET 4 AVERAGED READINGS

	MOVF	TEMP2,W			;GET THE LSB RESULT
	MOVWF	P2_ENDL			;SAVE IT AS THE PHASE 2 ENDING VOLTAGE (lsb)
	MOVF	TEMP3,W			;GET THE MSB RESULT
	MOVWF	P2_ENDH			;SAVE IT AS THE PHASE 2 ENDING VOLTAGE	(msb)


	MOVF	SYS_PHASE1_PW,W		;GET THE PHASE 1 PULSE WIDTH
	CALL	GET_TIMER_VAL		;CONVERT IT TO A TIMER0 VALUE
	MOVWF	PHASE_TVAL		;SAVE IT
	IORLW	0x00
	BTFSS	_Z			;IS IT ZERO?
	CALL	FIRE_1			;NO THEN FIRE PHASE 1

	CALL	READ_CAP_AVG8		;GET AN AVERAGE OF 8 READINGS
	MOVF	TEMP2,W			;SAVE THE RESULT
	MOVWF	P1_ENDL
	MOVF	TEMP3,W
	MOVWF	P1_ENDH

	BANK1
	MOVF	OPTION_REG,W	;SETUP TMR0 FOR A 1:256 PRESCALER
	IORLW	b'00000111'
	MOVWF	OPTION_REG
	BANK0
	
	RETLW	0		;*NOT IMPLEMENTED* AN ERROR IN ONE OF THE PHASES
				;WOULD BE RETURNED VALUE OTHER THAN 0. THIS RETURN VALUE
				;WOULD BE SENT BACK IN THE PHASE DATA AS THE STATUS
				;BYTE.

;****************************************************************************
; FIRE PHASE 1 OF THE H-BRIDGE
;****************************************************************************
FIRE_1
	MOVLW	d'30'		;WAIT A MAX OF 90us
	MOVWF	TEMP1

	CLRF	TMR0		;CLEAR TIMER 0

	BSF	KICK_1		;KICK AND ENABLE	(+4us = 94us)
	BSF	ENABLE_1
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

WFK_1
	DECFSZ	TEMP1,F		;WAIT 90us FOR KICK TO BE OVER
	GOTO	WFK_1		

	BCF	KICK_1		;STOP THE KICK
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

WAIT_P1
	MOVF	TMR0,W		;GET THE TIMER VALUE
	SUBWF	PHASE_TVAL,W	;F-W
	BTFSC	_C		;RESULT POSITVE?
	GOTO	WAIT_P1		;YES THEN WAIT

	BCF	ENABLE_1	;TURN OFF THE IGBTS
	BSF	DISABLE_1
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

	MOVLW	d'17'		;WAIT 50us 17 * 1us * 3
	MOVWF	TEMP1

WAIT_1
	DECFSZ	TEMP1,F
	GOTO 	WAIT_1

	BCF	DISABLE_1	;STOP THE DISABLE
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

	RETURN

;****************************************************************
; FIRE PHASE 2 OF THE H-BRIDGE
;****************************************************************
FIRE_2
	MOVLW	d'30'
	MOVWF	TEMP1
	
	CLRF	TMR0		;CLEAR TIMER 0

	BSF	KICK_2		;KICK AND ENABLE
	BSF	ENABLE_2
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

WFK_2
	DECFSZ	TEMP1,F		;WAIT 90us FOR KICK TO BE OVER
	GOTO	WFK_2		

	BCF	KICK_2		;STOP THE KICK
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

WAIT_P2
	MOVF	TMR0,W		;GET THE TIMER VALUE
	SUBWF	PHASE_TVAL,W	;F-W
	BTFSC	_C		;RESULT POSITVE?
	GOTO	WAIT_P2		;YES THEN WAIT

	BCF	ENABLE_2	;TURN OFF THE IGBTS
	BSF	DISABLE_2
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

	MOVLW	d'17'		;WAIT 50us
	MOVWF	TEMP1

WAIT_2
	DECFSZ	TEMP1,F
	GOTO 	WAIT_2

	BCF	DISABLE_2	;STOP THE DISABLE
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

	RETURN


;*************************************************************************
; TEST THE CHARGE TIME
; 1. DECREMENT CHARGE TIME IF NOT = 0
; 2. IF AFTER DEC. THE CHARGE TIME IT IS = 0 THEN TEST THE CAP VOLTAGE
;*************************************************************************
TEST_CHARGE_TIME
	BANK0
	BTFSC	HV_READY		;ARE WE WAITING TO SHOCK OR DUMP?
	RETLW	-1			;YES THEN EXIT

	BTFSS	HV_CIP			;IS THERE A CHARGE IN PROGRESS?
	RETLW	-2			;NO THEN EXIT

	MOVF	SYS_HV_CHARGE_TIME,W	;GET CHARGE TIME
	BTFSC	_Z			;CHARGE TIME = 0?
	RETLW	-2			;YES THEN DO NOTHING

	DECFSZ	SYS_HV_CHARGE_TIME,F	;CHARGE TIME -= 1, = 0?
	RETLW 	-3			;NO THEN EXIT
					;YES THEN ...

	CALL 	TEST_CAP_V		;TEST THE CAP VOLTAGE
	IORLW	0x00			;NEEDED TO SET THE Z FLAG
	BTFSC	_Z			;RETURNED VAL = 0?
	GOTO	SET_READY_ON		;YES THEN CONTINUE

	CLRF	HV_STATE_FLAGS		;CLEAR ALL FLAGS
	BSF	CHARGE_FAIL		;SIGNAL THE CHARGE FAILED
	CALL	READY_LED_OFF
	CALL	CHARGE_LED_OFF
	RETLW	-4			;RET VAL != 0, RETURN A -4

SET_READY_ON
	BSF	READY_LED		;TURN ON THE READY LED
	BCF	CHARGE_LED		;TURN OFF THE CHARGE LED
	MOVF	PORTA_DATA,W
	MOVWF	PORTA

	BTFSS	HV_SHOCK		;IS THERE A SHOCK REQUEST?
	CALL	TRIGGER_TALKER		;NO THEN TRIGGER THE TALKING MODULE

	BSF	HV_READY		;AND SIGNAL THAT WE ARE READY
	BCF	HV_CIP			;SIGNAL NO CHARGE IN PROGRESS

	BTFSS	HV_MAX			;MAX CHARGE?
	GOTO	TC_EXIT			;NO THEN JUST EXIT

	CLRF	REF_1			;YES THEN CLEAR THE REFORM TIMERS
	CLRF	REF_2
	CLRF	REF_3
	CLRF	REF_4
TC_EXIT
	MOVLW	MAX_HOLD_TIME		;CHARGED, NOW HOLD FOR
	MOVWF	HV_HOLD_TIMER		;THIS TIME
	
	RETLW 0

;*************************************************************************
; TEST THE HV CAP VOLTAGE VALUE. IF IT IS WITHIN THE RANGE SPECIFIED THEN
; RETURN A VALUE OF 0, ELSE RETURN A -1
;*************************************************************************
TEST_CAP_V
	CALL	READ_CAP_AVG8    	;GET A CAP READING

	MOVF	TEMP2,W			;MAKE A COPY OF THE VALUE FOR THE TEST
	MOVWF	TEMP4
	MOVF	TEMP3,W
	MOVWF	TEMP5

	MOVF	HV_MIN_HIGH,W		;READING - MIN
	SUBWF	TEMP5,F			;SUBTRACT THE HIGH BYTES

MIN_TEST
	MOVF	HV_MIN_LOW,W		;SUBTRACT THE LOW BYTES
	SUBWF	TEMP4,F
	BTFSS	_C			;RESULT POSITIVE?
	DECF	TEMP5,F			;NO THEN SUBTRACT ONE FROM THE HIGH BYTE
	RLF	TEMP5,F			;GET THE UPPER BIT OF THE HIGH BYTE
	BTFSC	_C			;UPPER BIT SET?	
	GOTO	FAILED_V		;YES THEN WE FAILED THE VOLTAGE TEST

MAX_TEST
	MOVF	TEMP2,W			;MAKE A COPY OF THE VALUE FOR THE TEST
	MOVWF	TEMP4
	MOVF	TEMP3,W
	MOVWF	TEMP5	

	MOVF	TEMP5,W			;MAX - READING, THE HIGH BYTES
	SUBWF	HV_MAX_HIGH,W
	MOVWF	TEMP1			;SAVE THE RESULT

	MOVF	TEMP4,W			;MAX - READING, THE LOW BYTES
	SUBWF	HV_MAX_LOW,W		
	BTFSS	_C			;RESULT POSIITVE?
	DECF	TEMP1,F			;NO THEN SUBTRACT ONE FROM THE HIGH BYTE
	RLF	TEMP1,F			;SHIFT THE UPPER BIT INTO THE CARRY
	BTFSS	_C			;CARRY SET?
	RETLW	0			;NO THEN WE ARE WITHIN BOTH RANGES, SO WE PASSED
FAILED_V	
	CALL	DO_DUMP
	RETLW	0xFF
;*************************************************************************
; TEST FOR CLOCK RUNAWAY
;*************************************************************************
TEST_RUNAWAY
	BANK0
	BTFSC	POWER_UP		;DID WE JUST POWER UP (FIRST TIME)?
	GOTO	INIT_RUNAWAY		;YES THEN INIT ONLY
					;ELSE ....
	BANK1
	MOVLW	b'00000100'		;CONFIGURE THE A/D CONVERTER INPUTS
	MOVWF	ADCON1
	BANK0
	
	MOVLW	b'01000001'		;AD CONVERTER ON, CHANNEL 0 SELECTED
	MOVWF	ADCON0			;FOSC/8

	MOVLW	d'34'		;34 * 3 *  1us (~100us)
	MOVWF	TEMP1

WAIT_AD				;WAIT FOR THE INPUT SAMPLE CAP TO GET CHARGED UP
	DECFSZ	TEMP1,F		;AFTER THE A/D WAS TURNED ON
	GOTO	WAIT_AD		;WAIT APPROX. 100us. THIS IS BECAUSE WE HAVE A 
				;10 MEG RESISTOR IN SERIES WITH THE +5V

	BSF	ADCON0,GO		;START AN A/D CONVERSION

WAIT_CONV
	BTFSC	ADCON0,GO		;IS THE GO BIT STILL HIGH?
	GOTO	WAIT_CONV		;YES, WAIT FOR THE CONVERSION TO COMPLETE

	MOVF	ADRES,W			;GET THE A/D CONVERTER VALUE
	MOVWF	RUNAWAY_V		;STORE IT

	CLRF	ADCON0			;TURN OFF THE A/D CONVERTER

	BANK1				;CONFIGURE PORTA AS ALL DIGITAL
	MOVLW	b'00000111'
	MOVWF	ADCON1

;----------------------------------------------
; TEST THE VOLTAGE TO SEE IF IT IS WITHIN RANGE
; IF IT IS NOT THEN RETURN A VALUE OF -1
; ***** THIS IS NOT IMPLEMENTED ***************
;----------------------------------------------
	
INIT_RUNAWAY

	BANK0
	BCF	RUNAWAY			;MAKE THE OUTPUT DATA LOW (TO SHORT IT OUT)
	MOVF	PORTA_DATA,W
	MOVWF	PORTA

	BANK1
	MOVLW	PORTA_DATA_DIR		;GET THE PORT A DATA DIRECTION VALUE
	ANDLW	b'11111110'		;MAKE BIT 0 AN OUTPUT
	MOVWF	TRISA
	
	MOVLW	d'17'			;WAIT TO MAKE SURE THE CAP IS DISCHARGED
	MOVWF	TEMP1			

WAIT_CLEAR
	DECFSZ	TEMP1,F			;WAIT 3 * 17 * 1us (~50us)
	GOTO	WAIT_CLEAR
	
	MOVLW	PORTA_DATA_DIR
	IORLW	b'00000001'		;MAKE BIT 0 AN INPUT
	MOVWF	TRISA

	BANK0
	BCF	POWER_UP		;AND CLEAR THE POWER UP BIT
	RETLW	0x00
;*************************************************************************
; SHUT OFF THE CHARGE DUMP
;*************************************************************************
DUMP_OFF
	BCF	DUMP_ENABLE	;STOP CHARGE DUMP
	BSF	DUMP_DISABLE
	MOVF	PORTB_DATA,W
	MOVWF	PORTB

	MOVLW	d'16'		;WAIT 50us
	MOVWF	TEMP1
	
WAIT_DUMP
	DECFSZ	TEMP1,F
	GOTO	WAIT_DUMP	

	BCF	DUMP_DISABLE	;STOP 
	MOVF	PORTB_DATA,W
	MOVWF	PORTB	
	RETURN
;*************************************************************************
; INIT CHARGE
;*************************************************************************
INIT_CHARGE
	BANK0
	
	BCF	CHARGE_FAIL	;CLEAR THE CHARGE FAIL BIT TO RESTART

	BTFSC	HV_CIP		;CHARGE IN PROGRESS? 
	RETLW	-1		;YES THEN DON'T CHARGE

	BTFSC	HV_READY	;ARE WE WAITING TO SHOCK?
	RETLW	-2		;YES THEN DON'T CHARGE

	CALL	TEST_LEAD_IMPED		;RETURN 0 IF OK
	IORLW	0x00			;NEEDED FOR Z STATUS
	BTFSS	_Z			;RESULT = 0?
	RETLW	-3			;NO THEN EXIT

	MOVF	HV_CNTRLV_LOW,W		;COPY THE USER DATA TO SYSTEM
	MOVWF	SYS_CNTRLV_LOW
	MOVF	HV_CNTRLV_HIGH,W
	MOVWF	SYS_CNTRLV_HIGH

	BTFSS	HV_MAX			;HV MAX FLAG SET BY FRONT PANEL SWITCH?
	GOTO    START_CHARGE		;NO THEN DO NORMAL CHARGE

	MOVLW	LOW MAX_CNTRLV		;YES THEN SET FOR MAX VOLTAGE
	MOVWF	SYS_CNTRLV_LOW
	MOVLW	HIGH MAX_CNTRLV
	MOVWF	SYS_CNTRLV_HIGH

;SINCE THIS IS A CHARGE TO MAX THE UPPER AND LOWER RANGE FOR THE 
;CAP. TO BE CHARGED TO MUST BE LOADED NOW. THIS IS TO TEST THE VOLTAGE
;THAT THE CAP. CHARGES TO.

	MOVLW	LOW HV_MIN_ALLOWED
	MOVWF	HV_MIN_LOW
	MOVLW	HIGH HV_MIN_ALLOWED
	MOVWF	HV_MIN_HIGH

	MOVLW	LOW HV_MAX_ALLOWED
	MOVWF	HV_MAX_LOW
	MOVLW	HIGH HV_MAX_ALLOWED
	MOVWF	HV_MAX_HIGH
	
START_CHARGE
	CALL	SW_5V_ON		;5 VOLTS ON	
	CALL	SW_24V_ON		;24 VOLTS ON
	BSF	HV_CIP			;SIGNAL CHARGE IN PROGRESS
	CALL	DUMP_OFF		;TURN OFF THE DUMP

;WAIT 50ms FOR EVERYTHING TO STABILIZE
	CALL	WAIT_50MS

	;OUTPUT THE SYS_CHARGE_V TO THE DAC
	MOVF	SYS_CNTRLV_LOW,W
	MOVWF	TEMP1
	MOVF	SYS_CNTRLV_HIGH,W
	MOVWF	TEMP2
	CALL	WRITE_TO_DAC

	CALL	HV_SUPPLY_ON		;ENABLE THE HV SUPPLY

	CALL	CHARGE_LED_ON		;TURN THE CHARGE LED ON
	CALL	READY_LED_OFF		;MAKE SURE THE READY IS OFF
	CALL	TRIGGER_TALKER		;SPEAK

LOAD_TIME
	MOVF	HV_CHARGE_TIME,W	;LOAD THE HV CHARGE TIME
	MOVWF	SYS_HV_CHARGE_TIME

	RETLW	0

;*************************************************************************
; TRIGGER THE TALKING MODULE
;*************************************************************************
TRIGGER_TALKER
	BTFSS	CHECK_LEAD_LED		;LEAD LED ON?
	GOTO	CHK_LED_OFF		;NO THE BIT IS LOW

	BCF	CHECK_LEAD_LED		;SET THE LINE LOW TO TRIGGER THE "TALKER MODULE"
	MOVF	PORTA_DATA,W		
	MOVWF	PORTA

	BSF	CHECK_LEAD_LED		;AND BACK HIGH AGAIN FOR THE LED
	MOVF	PORTA_DATA,W		
	MOVWF	PORTA
	RETURN

CHK_LED_OFF	
	BSF	CHECK_LEAD_LED		;SET THE CHECK LEAD LED
	MOVF	PORTA_DATA,W		
	MOVWF	PORTA

	BCF	CHECK_LEAD_LED		;SET THE LINE LOW TO TRIGGER THE "TALKER MODULE"
	MOVF	PORTA_DATA,W		
	MOVWF	PORTA
	RETURN

;*************************************************************************
; COPY THE TX BUFFER TO THE REPLY BUFFER
; THE TX_MSG_LEN MUST BE SET TO THE CORRECT VALUE OF THE MESSAGE LENGTH
;*************************************************************************
COPY_TX_TO_REPLY
	MOVF	TX_MSG_LEN,W	;GET A COPY OF THE MESSAGE LENGTH
	MOVWF	TEMP1

	MOVLW	TX_BUF		;GET THE SOURCE ADDRESS.
	MOVWF	TEMP2

	MOVLW	REPLY_BUF	;GET THE DEST. ADDRESS.
	MOVWF	TEMP3

COPY_TX_REPLY
	MOVF	TEMP2,W		;GET THE SOURCE DATA
	MOVWF	FSR

	MOVF	INDF,W		;SAVE IT TEMPORARILY
	MOVWF	TEMP4

	MOVF	TEMP3,W		;GET THE DEST PTR
	MOVWF	FSR

	MOVF	TEMP4,W		;STORE THE DATA @FSR
	MOVWF	INDF

	INCF	TEMP2,F		;SOURCE PTR++, DEST. PTR++
	INCF	TEMP3,F

	DECFSZ	TEMP1,F		;MESSAGE COUNT -= 1, = 0?
	GOTO	COPY_TX_REPLY	;NO THEN DO AGAIN

	RETURN

;*************************************************************************
; TEST TO SEE IF WE HAVE A SHOCK REQUEST
;*************************************************************************
TEST_SHOCK_RQST
	BTFSS	HV_SHOCK	;DO WE HAVE A SHOCK REQUEST
	RETURN			;NO THEN JUST RETURN

	BTFSS	HV_READY	;ARE WE CHARGED?
	RETURN			

	CALL	DO_SHOCK
	MOVWF	TEMP4

	RETURN

;*************************************************************************
;1/4 SECOND ROUTINE
;CALLED BY THE INTERRUPT SERVICE ROUTINE
;*************************************************************************
DO_ONE_QTR_SEC
	BTFSC	MAX_SLEEP_FLAG		;ARE WE IN MAX SLEEP MODE?
	RETURN				;YES THEN EXIT
					;ELSE...

	CALL	TEST_MAN_SW		;CHECK TO SEE IF THE MAN SW HAS BEEN PRESSED
	CALL	TEST_CAP_TIME		;RECHARGE THE MAN SWITCH CAP IF NEEDED
	CALL	TEST_CHARGE_TIME 	;IF WE ARE CHARGING THE HV CAP, CHECK THE 
					;CHARGE TIME
	CALL	TEST_SHOCK_RQST		;SHOCK IF WE GOT A SHOCK REQUEST
	CALL 	TEST_CHARGE_FAIL	;IF CHARGING FAILED THEN BLINK
					;THE CHARGE LED
	
	RETURN

;*************************************************************************
;ONE SECOND ROUTINE
;CALLED BY THE INTERRUPT SERVICE ROUTINE
;*************************************************************************
DO_ONE_SECOND
	CALL	TEST_RUNAWAY

	BANK0
	BTFSS	MAX_SLEEP_FLAG		;MAX SLEEP MODE?
	GOTO	TEST_LIMP		;NO THEN CONTINUE

	CALL	READY_LED_ON		;YES THEN TURN ON THE CHARGE LED
	CALL	WAIT_50MS		;FOR 50ms
	CALL	READY_LED_OFF
	GOTO	EXIT_ONE_SEC		;AND THEN EXIT

TEST_LIMP		
	CALL	TEST_LEAD_LED		;ELSE TEST THE LEAD ...
					;THE +5V IS SWITCHED ON BY THIS CALL
	
TEST_HV_READY
	BTFSS	HV_READY		;READY TO FIRE?
	GOTO	TEST_HV_CIP		;NO THEN SKIP THIS

	MOVF	HV_HOLD_TIMER,W		;AND DOES THE HOLD TIMER = 0?
	BTFSC	_Z			
	GOTO	TEST_HV_CIP		;YES THEN SKIP THIS

	DECF	HV_HOLD_TIMER,F		;HOLD TIMER -= 1
	BTFSS	_Z			;HOLD TIMER = 0?				
	GOTO	EXIT_ONE_SEC		;YES READY, NO HOLD_TIMER !=0
					;SO DON'T TEST CIP OR AC POWER ON

;MAKE TALKER SPEAK
	CALL	CHARGE_LED_ON		;YES THEN SPEAK IT
	CALL	READY_LED_ON
	CALL	TRIGGER_TALKER		;TRIGGER THE TALKER MODULE		
	CALL	DO_DUMP			;DO A DUMP, DO DUMP WILL TURN THE READY LED OFF
	GOTO	EXIT_ONE_SEC		;EXIT
					
TEST_HV_CIP
	BTFSC	HV_CIP			;IS THERE A CHARGE IN PROGRESS?
	GOTO	EXIT_ONE_SEC		;YES THEN EXIT

	BTFSS	AC_ON_IN		;IS THE AC POWER ON?
	GOTO	FIVE_OFF		;NO THEN TURN THE +5VSW OFF

	CALL	SW_5V_ON		;ELSE TURN THE +5VSW ON
	CALL	WAIT_50MS		;WAIT 50MS FOR POWER UP
	GOTO	EXIT_ONE_SEC

FIVE_OFF
	CALL	SW_5V_OFF

EXIT_ONE_SEC	
	RETURN	

;*************************************************************************
; TEST THE LEAD IMPEDANCE
; 1. SEND 5 TRIGGER PULSES TO THE IMPED MEAS BOARD TO MAKE SURE HAS AN
; UP TO DATE VALUE.
; 2. RETURN A ZERO IF LEAD IMPEDANCE IS WITHIN RANGE
; ELSE RETURN A -1.
;*************************************************************************
TEST_LEAD_IMPED
	BANK0
	
	BTFSS	ENABLE_5V	;IS THE 5V ON (ACTIVE LOW)
	GOTO	TL1		;YES THEN GO

	CALL	SW_5V_ON	;NO THEN SWITCH THE 5V ON
	CALL	WAIT_50MS	;WAIT 50ms FOR A/D CONVERTER TO STABILIZE

TL1
	CLRF	TEMP2
	CLRF	TEMP3

	MOVLW	d'5'
	MOVWF	TEMP6
	
SEND_START_SIG
	CALL	SEND_IMPED_START
	CALL	WAIT_2MS

	DECFSZ	TEMP6,F		;HAVE WE SENT ALL THE TRIGGER SIGNALS?
	GOTO	SEND_START_SIG	;NO THEN DO AGAIN

	MOVLW	d'8'		;TAKE 8 READINGS AND AVERAGE THEM
	MOVWF	TEMP6

AVG_8_AD2
	CALL	READ_AD_2	;USES TEMP1!
	CALL	WAIT_2MS	;WAIT 2MS
	
	MOVF	AD_2_MSB,W	;ADD THE HIGH BYTES
	ADDWF	TEMP3,F	

	MOVF	AD_2_LSB,W	;ADD THE LOW BYTES
	ADDWF	TEMP2,F

	BTFSC	_C		;WAS THERE A CARRY?
	INCF	TEMP3,F		;YES THEN ADD 1 TO THE HIGH BYTE

	DECFSZ	TEMP6,F		;HAVE WE GOT 8 AVERAGES?
	GOTO	AVG_8_AD2	;NO THEN DO AGAIN

	MOVLW	d'3'		;DIVIDE BY 2, 3 TIMES
	MOVWF	TEMP6

DIV_8_TL
	BCF	_C		;CLEAR THE CARRY
	RRF	TEMP3,F		;DIVIDE BY 2 (SHIFT MSB)
	RRF	TEMP2,F		;SHIFT LSB
	DECFSZ	TEMP6,F		;ARE WE DONE?
	GOTO	DIV_8_TL	;NO THEN DO AGAIN

TEST_MIN_MAX
;TEST FOR MINIMUM
	MOVLW	HIGH MIN_IMPED_MV	;GET THE MINIMUM (HIGH BYTE)
	SUBWF	TEMP3,W			;(F-W) IMPED HIGH BYTE - MIN HIGH BYTE 
	MOVWF	TEMP4	

	MOVLW	LOW MIN_IMPED_MV	;GET THE MIN. (LOW BYTE)
	SUBWF	TEMP2,W			;(F-W) IMPED LOW - MIN LOW
	BTFSS	_C
	DECF	TEMP4,F
	RLF	TEMP4,F
	BTFSC	_C
	GOTO	FAILED_TEST

;TEST FOR MAX
	MOVF	TEMP3,W			;HIGH BYTE READING
	SUBLW	HIGH MAX_IMPED_MV	;(K-W) MAX - READING
	MOVWF	TEMP4			;SAVE RESULT

	MOVF	TEMP2,W			;LOW BYTE READING
	SUBLW	LOW MAX_IMPED_MV	;(K-W) MAX - READING
	BTFSS	_C			;RESULT POS?
	DECF	TEMP4,F			;NO THEN DEC THE HIGH BYTE
	RLF	TEMP4,F			
	BTFSC	_C			;WAS THERE A BORROW IN THE HIGH BYTE?

FAILED_TEST
	RETLW	0xFF			;YES THEN WE FAILED	
	RETLW	0			;ELSE PASSED


;*************************************************************************
; TEST THE LEAD IMPEDANCE AND TURN ON OR OFF THE CHECK LEAD LED
;*************************************************************************
TEST_LEAD_LED
	CALL	TEST_LEAD_IMPED		;TEST THE LEAD IMPEDANCE
	IORLW	0x00			;RESULT = 0?
	BTFSC	_Z			
	GOTO	LEAD_OK			;YES THEN THE LEAD IS OK	

	BSF	CHECK_LEAD_LED		;NO THEN SET THE CHECK LEAD LED
	MOVF	PORTA_DATA,W		
	MOVWF	PORTA

	BCF	CHECK_LEAD_LED		;SET THE LINE LOW TO TRIGGER THE "TALKER MODULE"
	MOVF	PORTA_DATA,W		
	MOVWF	PORTA

	BSF	CHECK_LEAD_LED		;AND BACK HIGH AGAIN FOR THE LED
	MOVF	PORTA_DATA,W		
	MOVWF	PORTA
	RETURN				;AND EXIT


LEAD_OK
	BCF	CHECK_LEAD_LED		;MAKE IT LOW TO TRIGGER THE TALKING MODULE
	MOVF	PORTA_DATA,W
	MOVWF	PORTA
	RETURN
		
;*************************************************************************
; TRIGGER THE IMPEDANCE MEASUREMENT CIRCUIT
; MEM USED:NONE
;*************************************************************************
SEND_IMPED_START

	BSF	IMPED_START	;START AN IMPEDANCE MESUREMENT
	MOVF	PORTA_DATA,W
	MOVWF	PORTA

	NOP	;WAIT 10us FOR THE IMPEDANCE MEAS. PULSE
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP

	BCF	IMPED_START	;IMPED START SIGNAL LOW
	MOVF	PORTA_DATA,W
	MOVWF	PORTA

	RETURN
;*************************************************************************
; SWITCH ON THE +5 VOLTS
;*************************************************************************
SW_5V_ON
	BANK0
	BCF	ENABLE_5V		;(ACTIVE LOW)
	MOVF	PORTC_DATA,W
	MOVWF	PORTC
	RETURN

;*************************************************************************
; SWITCH OFF THE +5 VOLTS
;*************************************************************************
SW_5V_OFF
	BANK0
	BSF	ENABLE_5V		;(ACTIVE LOW)
	MOVF	PORTC_DATA,W
	MOVWF	PORTC
	RETURN

;*************************************************************************
; SWITCH THE 24 VOLTS ON
;*************************************************************************
SW_24V_ON
	BANK0
	BSF	ENABLE_24V	;TURN ON THE 24V
	MOVF	PORTB_DATA,W
	MOVWF	PORTB
	RETURN

;*************************************************************************
; SWITCH THE 24 VOLTS OFF
;*************************************************************************
SW_24V_OFF
	BANK0
	BCF	ENABLE_24V	;TURN OFF THE 24V
	MOVF	PORTB_DATA,W
	MOVWF	PORTB
	RETURN

;*************************************************************************
; TURN OFF THE READY LED
;*************************************************************************
READY_LED_OFF
	BCF	READY_LED	;TURN OFF THE READY LED
	MOVF	PORTA_DATA,W
	MOVWF	PORTA
	RETURN

;*************************************************************************
; TURN ON THE READY LED
;*************************************************************************
READY_LED_ON
	BSF	READY_LED	;TURN ON THE READY LED
	MOVF	PORTA_DATA,W
	MOVWF	PORTA
	RETURN

;*************************************************************************
; TURN ON THE CHARGE LED
;*************************************************************************
CHARGE_LED_ON
	BSF	CHARGE_LED	;TURN OFF THE CHARGE LED
	MOVF	PORTA_DATA,W
	MOVWF	PORTA
	RETURN

;*************************************************************************
; TURN OFF THE CHARGE LED
;*************************************************************************
CHARGE_LED_OFF

	BCF	CHARGE_LED	;TURN OFF THE READY LED
	MOVF	PORTA_DATA,W
	MOVWF	PORTA
	RETURN

;*************************************************************************
; ONE MINUTE ROUTINE
; CALLED BY THE INTERRUPT SERVICE ROUTINE
;*************************************************************************
DO_ONE_MIN
	BANK0	

;INCREMENT THE POWER ON TIMER
	INCF	PWR_ON_1,F
	BTFSS	_Z
	GOTO	INC_REF
	
	INCF	PWR_ON_2,F
	BTFSS	_Z
	GOTO	INC_REF

	INCF	PWR_ON_3,F
	BTFSC	_Z
	INCF	PWR_ON_4,F

;INCREMENT THE REFORM TIMER
INC_REF
	INCF	REF_1,F
	BTFSS	_Z
	RETURN
	
	INCF	REF_2,F
	BTFSS	_Z
	RETURN

	INCF	REF_3,F
	BTFSS	_Z
	RETURN

	INCF	REF_4,F
	
	RETURN

;*************************************************************************
; ECHO THE RX MESSAGE
; 1. COPY THE RX BUFFER TO THE TX BUFFER.
;*************************************************************************
ECHO_RX
	CLRF	TX_MSG_LEN

	MOVF	RX_MSG_LEN,W	;GET THE NUMBER OF BYTES
	MOVWF	TEMP1

	MOVLW	RX_BUF		;LOAD THE SOURCE PTR
	MOVWF	TEMP2
	
	MOVLW	TX_BUF		;LOAD THE DEST. PTR
	MOVWF	TEMP3

COPY_RX_TX
	MOVF	TEMP2,W		;GET THE SOURCE PTR
	MOVWF	FSR
	
	MOVF	INDF,W		;GET THE DATA @PTR
	MOVWF	TEMP4		;SAVE IT TEMP

	MOVF	TEMP3,W
	MOVWF	FSR

	MOVF	TEMP4,W		;STORE THE DATA @FSR
	MOVWF	INDF

	INCF	TEMP2,F		;SOURCE PTR++
	INCF	TEMP3,F		;DEST PTR++

	INCF	TX_MSG_LEN,F	;ADD 1 TO THE TRANSMIT MESSAGE LENGTH

	DECFSZ	TEMP1,F		;ALL BYTES DONE
	GOTO	COPY_RX_TX	;NO THEN DO AGAIN

	CALL	TRANSMIT_TX_BUF
		
	RETURN
;*************************************************************************
; WRITE TO THE D/A CONVERTER
; VALUES TO BE WRITTEN ARE IN TEMP1 (lsb), TEMP2 (msb)
;*************************************************************************
WRITE_TO_DAC
	BANK0
	BSF	DA_CS			;ENABLE THE D/A (CS LINE LOW)
	MOVF	PORTE_DATA,W
	MOVWF	PORTE

	BCF	PIR1,SSPIF		;CLEAR THE INTERRUPT FLAG
	MOVF	TEMP2,W			;WRITE OUT THE MSB DATA
	MOVWF	SSPBUF

WAIT_SPI_1
	BTFSS	PIR1,SSPIF		;SPI INTERRUPT FLAG SET?
	GOTO	WAIT_SPI_1		;NO THEN WAIT UNTIL IT IS

	BCF	PIR1,SSPIF		;CLEAR THE INTERRUPT FLAG
	MOVF	TEMP1,W			;WRITE OUT THE LSB DATA
	MOVWF	SSPBUF

WAIT_SPI_2
	BTFSS	PIR1,SSPIF		;SPI INTERRUPT FLAG SET?
	GOTO	WAIT_SPI_2		;NO THEN WAIT

	BCF	DA_CS			;YES THEN SET THE CS LINE HIGH
	MOVF	PORTE_DATA,W
	MOVWF	PORTE
	RETURN
;****************************************************************************
; ENABLE THE HV POWER SUPPLY
;****************************************************************************
HV_SUPPLY_ON

	BSF	ENABLE_HV		;ENABLE THE HV POWER SUPPLY
	MOVF	PORTB_DATA,W
	MOVWF	PORTB
	RETURN

;****************************************************************************
; DISABLE THE HV POWER SUPPLY
;****************************************************************************
HV_SUPPLY_OFF

	BCF	ENABLE_HV		;DISABLE THE HV POWER SUPPLY
	MOVF	PORTB_DATA,W
	MOVWF	PORTB
	RETURN

;****************************************************************************
; READ THE CAP VOLTAGE
; TURN ON THE +5V IF NEEDED BEFORE READING THE A/D CONVERTER
;****************************************************************************
READ_CAP_V
	BANK0
	BTFSS	ENABLE_5V	;5 VOLTS ON? 	(ACTIVE LOW)
	GOTO	RC_1		;YES IT IS ON

	CALL	SW_5V_ON	;NO IT IS OFF, SO SWITCH IT ON 
	CALL	WAIT_50MS	;AND WAIT 50ms
	BSF	FIVE_NOT_ON	

RC_1
	CALL	READ_AD_1
	RETURN
;****************************************************************************
; READ A TO D CONVERTER #1 (CAP VOLTAGE)
; VALUES RETURNED IN AD_1_LSB, AND AD_2_MSB
;****************************************************************************
READ_AD_1
	BANK0

	BSF	AD_CS_1		;START A CONVERSION
	MOVF	PORTE_DATA,W
	MOVWF	PORTE

	NOP			;WAIT 10us TOTAL FOR CONVERSION TO COMPLETE
	NOP			
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP

	MOVF	SSPBUF,W	;READ STATUS TO CLEAR BIT
	BANK0			;2us

	CLRF	SSPBUF		;DO A DUMMY WRITE TO GET MSB

	BANK1

WAIT_SPI_RX_1
	BTFSS	SSPSTAT,BF	;BUFFER FULL?
	GOTO	WAIT_SPI_RX_1	;NO THEN WAIT	

	BANK0			;YES THEN GET THE DATA
	MOVF	SSPBUF,W
	MOVWF	AD_1_MSB	;AND SAVE IT	

	CLRF	SSPBUF		;DUMMY WRITE

	BANK1

WAIT_SPI_RX_2
	BTFSS	SSPSTAT,BF	;BUFFER FULL?
	GOTO	WAIT_SPI_RX_2	;NO THEN WAIT

	BANK0			;YES THEN GET THE DATA
	MOVF	SSPBUF,W
	MOVWF	AD_1_LSB	

	BCF	AD_CS_1		;SET CS HIGH
	MOVF	PORTE_DATA,W
	MOVWF	PORTE

	MOVLW	d'3'
	MOVWF	TEMP1

AD_ADJ1
	BCF	_C		;SHIFT RIGHT 3 TIMES
	RRF	AD_1_MSB,F
	RRF	AD_1_LSB,F
	DECFSZ	TEMP1,F
	GOTO	AD_ADJ1

	MOVLW	0x0F		;MASK OFF THE UPPER 4 BITS
	ANDWF	AD_1_MSB,F

	RETURN

;****************************************************************************
; READ A TO D CONVERTER #2 (IMPEDANCE)
; MEM USED: TEMP1
;****************************************************************************
READ_AD_2
	BANK0	

	CALL	SEND_IMPED_START

	BSF	AD_CS_2		;START A CONVERSION
	MOVF	PORTE_DATA,W
	MOVWF	PORTE

	NOP			;WAIT 10us TOTAL FOR CONVERSION TO COMPLETE
	NOP			
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP

	MOVF	SSPBUF,W	;READ STATUS TO CLEAR BIT
	BANK0			
	CLRF	SSPBUF		;DO A DUMMY WRITE TO GET MSB
	BANK1

WAIT_SPI_RX_3
	BTFSS	SSPSTAT,BF	;BUFFER FULL?
	GOTO	WAIT_SPI_RX_3	;NO THEN WAIT	

	BANK0			;YES THEN GET THE DATA
	MOVF	SSPBUF,W
	MOVWF	AD_2_MSB	;AND SAVE IT
	
	CLRF	SSPBUF		;DUMMY WRITE

	BANK1

WAIT_SPI_RX_4
	BTFSS	SSPSTAT,BF	;BUFFER FULL?
	GOTO	WAIT_SPI_RX_4	;NO THEN WAIT

	BANK0			;YES THEN GET THE DATA
	MOVF	SSPBUF,W
	MOVWF	AD_2_LSB	

	BCF	AD_CS_2		;SET CS HIGH
	MOVF	PORTE_DATA,W
	MOVWF	PORTE

	MOVLW	d'3'
	MOVWF	TEMP1

AD_ADJ2
	BCF	_C		;SHIFT RIGHT 3 TIMES
	RRF	AD_2_MSB,F
	RRF	AD_2_LSB,F
	DECFSZ	TEMP1,F
	GOTO	AD_ADJ2

	MOVLW	0x0F		;MASK OFF THE UPPER 4 BITS
	ANDWF	AD_2_MSB,F

	RETURN
;****************************************************************************
; WAIT 50 milliseconds
;****************************************************************************
WAIT_50MS
	BANK0

	CLRF	TMR0


DELAY_50
	MOVF	TMR0,W			;WAIT 50ms
	SUBLW	d'195'
	BTFSC	_C
	GOTO	DELAY_50

	RETURN
;*****************************************************************************
; WAIT 2.048 milliseconds
;*****************************************************************************
WAIT_2MS
	BANK0

	CLRF	TMR0

WAIT_2048
	MOVF	TMR0,W		;WAIT 2.048ms 
	SUBLW	d'8'		
	BTFSC	_C
	GOTO	WAIT_2048

	RETURN

;***********************************************************************************
	END
