;********************************************************
;(c) 2003 David Prutchi and Michael Norris
;FILE NAME = SBDEF6.INC
;SHOCK BOX DEFINITION FILE
;********************************************************

#define	_Z	STATUS,Z
#define	_C	STATUS,C

#define	TIMER1_QRTR_SEC		d'57344'
#define	TIMER1_HALF_SEC		d'49152'
#define	TIMER1_1SEC		d'32768'

#define	SSPCON_INIT		b'00100000'	;SPI PORT CONFIG VALUE
						;ENABLE THE PORT, CLOCK PARITY LOW,
						;MASTER MODE, FOSC/4

#define	BAUD_2400	d'25'			;SPRG SETTING FOR 2400 baud (USING BRGH=0)
#define	BAUD_19_2K	d'12'			;SPRG SETTING FOR 19.2K BAUD (USING BRGH=1)

#define	MAX_HOLD_TIME	d'30'			;MAX 30 SECS TO HOLD CHARGE BEFORE DUMPING
#define	COMM_TIMEOUT	d'128'			;7.8125ms * THIS VALUE
						;7.8125ms * 128 = 1.0 SEC.

#define	MIN_IMPED_MV	0x264		;25 OHMS
#define MAX_IMPED_MV	0x344		;150 OHMS

#define	HV_MAX_ALLOWED	d'3420'		;MIN AND MAX ALLOWED FOR THE CAP VOLTAGE AT
#define	HV_MIN_ALLOWED	d'3094'		;750 VOLT CHARGE 
					;** CHANGED FROM 800 TO 750
					;   FOR VER. SB001M, 8/17/99	

#define	MAX_CNTRLV	d'3488'		;750VDC!	** CHANGED FROM 800 TO 750 
					;		   FOR VER. SB001M, 8/17/99
					;(0.215V/mv)
;***********************************************************
; PORT PIN DEFINITIONS
;***********************************************************
#define	PORTA_DATA_DIR	b'11001001'
#define	PORTB_DATA_DIR	b'01100001'		;** CHANGED 4-19-99 TO ADD DUMP_DISABLE TO RB7
#define	PORTC_DATA_DIR	b'10010011'
#define	PORTD_DATA_DIR	b'00000000'
#define	PORTE_DATA_DIR	b'00001000'

#define	CHARGE_SW	PORTB,5
#define	DEFIB_SW	PORTB,6
#define	AC_ON_IN	PORTA,3
#define	RTS_IN		PORTB,0
;***********************************************************
; DATA MEMORY DEFINITIONS START HERE
;***********************************************************
RAM_START	EQU		0x20

W_TEMP		EQU		RAM_START
STATUS_TEMP	EQU		W_TEMP+1
PCLATH_TEMP	EQU		STATUS_TEMP+1
FSR_TEMP	EQU		PCLATH_TEMP+1

PORTA_DATA	EQU		FSR_TEMP+1
#define 	RUNAWAY		PORTA_DATA,0	;IN AND OUT
#define		CHARGE_LED	PORTA_DATA,1	;OUT
#define		READY_LED	PORTA_DATA,2	;OUT
#define		POWER_DET	PORTA_DATA,3	;IN
#define		IMPED_START	PORTA_DATA,4	;OUT
#define		CHECK_LEAD_LED	PORTA_DATA,5	;OUT

PORTB_DATA	EQU		PORTA_DATA+1
#define		RTS		PORTB_DATA,0
#define		ENABLE_HV	PORTB_DATA,1
#define		DUMP_ENABLE	PORTB_DATA,2
#define		ENABLE_24V	PORTB_DATA,3
#define		SWITCH_RESET	PORTB_DATA,4
#define		MAN_CHARGE	PORTB_DATA,5
#define		MAN_DEFIB	PORTB_DATA,6
#define		DUMP_DISABLE	PORTB_DATA,7

PORTC_DATA	EQU		PORTB_DATA+1
#define		OSC_32KHZ_1	PORTC_DATA,0
#define		OSC_32KHZ_2	PORTC_DATA,1
#define		ENABLE_5V	PORTC_DATA,2
#define		SCLK		PORTC_DATA,3
#define		SDIN		PORTC_DATA,4
#define		SDOUT		PORTC_DATA,5
#define		TX_OUT		PORTC_DATA,6
#define		RX_IN		PORTC_DATA,7

PORTD_DATA	EQU		PORTC_DATA+1
#define		KICK_1		PORTD_DATA,0
#define		KICK_2		PORTD_DATA,1
#define		ENABLE_1	PORTD_DATA,2
#define		ENABLE_2	PORTD_DATA,3
#define		DISABLE_1	PORTD_DATA,4
#define		DISABLE_2	PORTD_DATA,5
#define		CTS_NOT		PORTD_DATA,6
#define		SWITCH_CHARGE	PORTD_DATA,7

PORTE_DATA	EQU		PORTD_DATA+1
#define		AD_CS_1		PORTE_DATA,0
#define		AD_CS_2		PORTE_DATA,1
#define		DA_CS		PORTE_DATA,2

QRTR_SEC_TIMER	EQU		PORTE_DATA+1
SEC60_TIMER	EQU		QRTR_SEC_TIMER+1

;ELAPSED TIME IN MINUTES SINCE LAST RESET
PWR_ON_1	EQU		SEC60_TIMER+1  ;(lsb)
PWR_ON_2	EQU		PWR_ON_1+1	
PWR_ON_3	EQU		PWR_ON_2+1	
PWR_ON_4	EQU		PWR_ON_3+1   	;(msb)

REF_1		EQU		PWR_ON_4+1      ;(lsb)
REF_2		EQU		REF_1+1
REF_3		EQU		REF_2+1
REF_4 		EQU		REF_3+1		;(msb)

TOTAL_1		EQU		REF_4+1		;TOTAL NUMBER OF SHOCKS SINCE POWER UP
TOTAL_2		EQU		TOTAL_1+1	;CAN BE CLEARED BY USER

SYS_HV_CHARGE_TIME	EQU	TOTAL_2+1
SYS_CNTRLV_LOW		EQU	SYS_HV_CHARGE_TIME+1
SYS_CNTRLV_HIGH		EQU	SYS_CNTRLV_LOW+1
SYS_PHASE1_PW		EQU	SYS_CNTRLV_HIGH+1
SYS_PHASE2_PW		EQU	SYS_PHASE1_PW+1
SYS_POLARITY		EQU	SYS_PHASE2_PW+1


TEMP1			EQU	SYS_POLARITY+1		;SCRATCH PAD AREA
TEMP2			EQU	TEMP1+1
TEMP3			EQU	TEMP2+1
TEMP4			EQU	TEMP3+1
TEMP5			EQU	TEMP4+1
TEMP6			EQU	TEMP5+1


SYS_INT_FLAGS	EQU		TEMP6+1
#define		IRP_BIT		SYS_INT_FLAGS,0
#define		FIVE_NOT_ON	SYS_INT_FLAGS,1
#define		POWER_UP	SYS_INT_FLAGS,2
#define		RUNAWAY_FLAG	SYS_INT_FLAGS,3
#define		MAX_SLEEP_FLAG	SYS_INT_FLAGS,4

SW_CHRG_ON_TIME		EQU	SYS_INT_FLAGS+1
SW_CHRG_OFF_TIME	EQU	SW_CHRG_ON_TIME+1
#define SW_CHRG_FULL		d'20'		;5 SECS
#define	SW_CHRG_ON_INIT		d'4'		;1 SEC
#define	SW_CHRG_OFF_INIT	d'60'		;15 SECS	CHANGED 4-28-99

HV_STATE_FLAGS	EQU	SW_CHRG_OFF_TIME+1
#define		HV_CIP		HV_STATE_FLAGS,0	;HIGH VOLTAGE CHARGE IN PROGRESS
#define		HV_READY	HV_STATE_FLAGS,1	;HIGH VOLTAGE CHARGED AND READY
#define		HV_MAX		HV_STATE_FLAGS,2	;HIGH VOLTAGE CHARGE TO MAX ENERGY
#define		HV_MAX_CIP	HV_STATE_FLAGS,3	;HV CHARGING TO MAX VOLTAGE
#define		HV_ARM		HV_STATE_FLAGS,4	;HV SAFETY ARM
#define		HV_SHOCK	HV_STATE_FLAGS,5	;HV SHOCK!
#define		CHARGE_FAIL	HV_STATE_FLAGS,6	;HV CHARGE ON THE CAP NOT WITHIN RANGE

;COMMUNICATION VARIABLES
RX_PTR		EQU	HV_STATE_FLAGS+1
REPLY_PTR	EQU	RX_PTR+1
RX_COMM_STATE	EQU	REPLY_PTR+1

COMM_STATUS	EQU	RX_COMM_STATE+1
#define	COMM_ERR	COMM_STATUS,0
#define	TX_DONE		COMM_STATUS,1
#define	TX_RQST		COMM_STATUS,2

TX_MSG_LEN	EQU	COMM_STATUS+1
RX_MSG_LEN	EQU	TX_MSG_LEN+1
REPLY_MSG_LEN	EQU	RX_MSG_LEN+1
RX_DATA		EQU	REPLY_MSG_LEN+1

RX_FLAGS	EQU	RX_DATA+1
#define			CHKSUM_OK	RX_FLAGS,0
#define			LAST_CHAR	RX_FLAGS,1

RX_COMMAND	EQU	RX_FLAGS+1
RX_DATA_COUNT	EQU	RX_COMMAND+1
RX_CHK_LOW	EQU	RX_DATA_COUNT+1
RX_CHK_HIGH	EQU	RX_CHK_LOW+1
RX_COUNT_COPY	EQU	RX_CHK_HIGH+1
REC_MSG_LEN	EQU	RX_COUNT_COPY+1

TX_CHK_LOW	EQU	REC_MSG_LEN+1
TX_CHK_HIGH	EQU	TX_CHK_LOW+1

COMM_TIMER	EQU	TX_CHK_HIGH+1
HV_HOLD_TIMER	EQU	COMM_TIMER+1

;*****************************************************************************************
HV_CHARGE_TIME		EQU	HV_HOLD_TIMER+1		;USER SET CHARGE TIME
#define	HV_CHARGE_TIME_INIT	24			;SECS IN 0.25 SEC INCREMENTS 6 SECS

HV_CNTRLV_LOW		EQU	HV_CHARGE_TIME+1
HV_CNTRLV_HIGH		EQU	HV_CNTRLV_LOW+1
#define	HV_CNTRLV_INIT	0

HV_PHASE1_PW		EQU	HV_CNTRLV_HIGH+1
HV_PHASE2_PW		EQU	HV_PHASE1_PW+1

HV_POLARITY		EQU	HV_PHASE2_PW+1
#define	DEFAULT_POLARITY	0

HV_MIN_LOW		EQU	HV_POLARITY+1
HV_MIN_HIGH		EQU	HV_MIN_LOW+1
HV_MAX_LOW		EQU	HV_MIN_HIGH+1
HV_MAX_HIGH		EQU	HV_MAX_LOW+1

DAC_LSB			EQU	HV_MAX_HIGH+1
DAC_MSB			EQU	DAC_LSB+1

AD_1_LSB		EQU	DAC_MSB+1			
AD_1_MSB		EQU	AD_1_LSB+1
AD_2_LSB		EQU	AD_1_MSB+1	
AD_2_MSB		EQU	AD_2_LSB+1

IMPED_AVG_LOW		EQU	AD_2_MSB+1
IMPED_AVG_HIGH		EQU	IMPED_AVG_LOW+1


P1_STARTL		EQU	IMPED_AVG_HIGH+1
P1_STARTH		EQU	P1_STARTL+1

P2_STARTL		EQU	P1_STARTH+1
P2_STARTH		EQU	P2_STARTL+1

P1_ENDL			EQU	P2_STARTH+1
P1_ENDH			EQU	P1_ENDL+1

P2_ENDL			EQU	P1_ENDH+1
P2_ENDH			EQU	P2_ENDL+1

PHASE_TVAL		EQU	P2_ENDH+1

BLINK_ON_TIME		EQU	PHASE_TVAL+1
BLINK_OFF_TIME		EQU	BLINK_ON_TIME+1
#define	BLINK_TIME	d'2'				;2 * 0.25 SECS = 1/2 SEC

RUNAWAY_V		EQU	BLINK_OFF_TIME+1
;*****************************************************************************************
LAST_USED		EQU	RUNAWAY_V
#define	NEXT_LOC		LAST_USED+1

;******************************************************************************************
;RAM BANK 2	STARTS AT 0xA0

#define	RAM_START2	0xA0		;START OF BANK2
					;REMEMBER THE FIRST 2 BYTES IN ALL BANKS MAY BE USED
					;STORE W_TEMP AND STATUS_TEMP, DEPENDING ON WHICH BANK
					;IS CURRENT WHEN AN INTERRUPT IS EXECUTED.

#define	TX_BUF		0xB0		;START OF THE TRANSMIT BUFFER
#define	TX_BUF_LEN	d'16'

#define	RX_BUF		0xC0		;START OF THE RX BUFFER
#define	RX_BUF_LEN	d'16'

#define REPLY_BUF	0xD0		;START OF THE REPLY BUFFER
#define	REPLY_BUF_LEN	d'16'

#define	BUF_SPACE_LEN	d'48'		;TOTAL BUFFER SPACE 48 BYTES
;***********************************************************
; END OF MEMORY DEFINITIONS
;***********************************************************
