;***********************************************************
;file name = QV12.asm 
;
;Written by: Mike Norris; Genesis: DEC. 9,1999
; (c) 2003 David Prutchi and Michael Norris
;
;Purpose: Implement a controller for the QuickView Cardiac Simulator.
;
;Clock Frequency = 10MHZ
;***********************************************************
	LIST	p=16C77,F=INHX8,R=DEC

	#include "P16C77.INC"

	__CONFIG _HS_OSC&_WDT_OFF&_PWRTE_ON&_CP_OFF&_BODEN_ON
	__IDLOCS 0102		;Version 1.2
	ERRORLEVEL 1,-302

	INCLUDE "QVM1.INC"	;MACRO FILE
	INCLUDE "QVP1.INC"	;PORT DEFINITION FILE
	INCLUDE "QVDEFS5.INC"	;MEMORY DEFINITION FILE
;***********************************************************
; THE RESET VECTOR STARTS HERE
;***********************************************************
	ORG	0x00
RESET
	GOTO	STARTUP

;************************************************************
; THE INTERRUPT SERVICE ROUTINE STARTS HERE
;************************************************************
	ORG	0x04
INT_SERV
;************************************************************
;SAVE ALL THE IMPORTANT REGISTERS
; REMEMBER! THE W REG IS SAVED IN WTEMP ON ENTRY, BUT THE RAM BANK CAN
; BE SET ANYWHERE. SO DON'T USE THE FIRST MEM. LOCATION IN ANY BANK.
;*************************************************************
	MOVWF	W_TEMP		;COPY W TO TEMP
	SWAPF	STATUS,W	;SWAP STATUS TO BE SAVED INTO WREG
	CLRF	STATUS		;MAKE BANK = 0		
	MOVWF	STATUS_TEMP	;SAVE STATUS TO TEMP
	MOVF	PCLATH,W	;SAVE THE PAGE REGISTER
	MOVWF	PCLATH_TEMP
	CLRF	PCLATH		;PAGE 0
	BCF	STATUS,IRP	;DATA BANK 0
	MOVF	FSR,W		;SAVE THE FSR REGISTER
	MOVWF	FSR_TEMP

;*******************************************************************
; 1 MS TIMER1 INTERRUPT
;*******************************************************************
	BTFSS	PIR1,TMR1IF	;WAS IT A TIMER 1 INTERRUPT?
	GOTO	TEST_TMR0	;NO THEN CHECK FOR TIMER 0 INTERRUPT

	BCF	T1CON,TMR1ON	;SWITCH THE TIMER OFF
	MOVLW	LOW T1_1MS	;RELOAD THE TIMER
	MOVWF	TMR1L
	MOVLW	HIGH T1_1MS
	MOVWF	TMR1H

	BSF	T1CON,TMR1ON	;SWITCH THE TIMER BACK ON

	BCF	PIR1,TMR1IF	;CLEAR THE INTERRUPT FLAG

;--------------------------------------------------------------------
;DECREMENT THE Z DELAY TIMER IF THE TIMER !=0
;AND START THE IMPEDANCE WAVEFORM OUTPUT WHEN THE DECREMENTED TIMER VALUE
;IS = 0.
;---------------------------------------------------------------------

	MOVF	Z_DELAY_TIMER,W		;GET THE Z_DELAY_TIMER
	BTFSC	_Z			;IS IT ZERO?
	GOTO	TEST_LVP_WIN_TIMER	;YES THEN SKIP THIS

	DECF	Z_DELAY_TIMER,F		;NO THEN DECREMENT THE TIMER
	BTFSS	_Z			;TIMER = 0?
	GOTO	TEST_LVP_WIN_TIMER	;NO THEN SKIP THIS

	MOVLW	Z_DATA_LENGTH		;GET THE LENGTH OF THE WAVEFORM
	MOVWF	Z_COUNT		

	CLRF	Z_INDX_LOW		;CLEAR THE INDEX 
	CLRF	Z_INDX_HIGH
	BSF	Z_FLAG			;TURN THE IMPEDANCE OUTPUT ON.
	
;--------------------------------------------------------------------
;DECREMENT THE LVP WINDOW TIMER, IF THE TIMER !=0
;--------------------------------------------------------------------
TEST_LVP_WIN_TIMER
	MOVF	LVP_WIN_TIMER,W		;TIMER = 0?
	BTFSC	_Z			;YES THEN SKIP THIS
	GOTO	INC_A_AND_V_TIMERS

	DECF	LVP_WIN_TIMER,F		;NO THEN DEC THE TIMER

	BTFSS	_Z			;TIMER = 0?
	GOTO	INC_A_AND_V_TIMERS

	BTFSC	LVP_ERROR		;ERROR FLAG PREVIOUSLY SET?
	GOTO	DO_LVP_ERROR		;YES THEN THIS IS AN ERROR

	BTFSC	LVP_IN			;LVP_IN = 1?
	GOTO	DO_LVP_ERROR		;YES THEN THIS IS AN ERROR

					;NO, LVP_IN=0
	BTFSS	LVP_ARM			;DID LVP GO HIGH INSIDE WINDOW
					;EARLIER?

	GOTO	INC_A_AND_V_TIMERS	;NO THEN DO NOTHING, NO LEDS SHOULD
					;BE TURNED ON.

	BCF	LVP_ARM			;YES THEN CLEAR THE FLAGS
	CALL	LVP_LED_ON		;TURN ON THE LVP OK LED
	GOTO	INC_A_AND_V_TIMERS	;AND EXIT

DO_LVP_ERROR
	BCF	LVP_ERROR
	BCF	LVP_ARM
	CALL	LVP_OUT_LED_ON
	

;--------------------------------------------------------------------
; INCREMENT THE A AND V EVENT TIMERS ONCE EVERY MILLISECOND
;--------------------------------------------------------------------
INC_A_AND_V_TIMERS
	INCF	A_INTERVAL_LOW,F
	BTFSC	_Z
	INCF	A_INTERVAL_HIGH,F

	INCF	V_INTERVAL_LOW,F
	BTFSC	_Z
	INCF	V_INTERVAL_HIGH,F

;--------------------------------------------------------------------
;CHECK THE RV EVENT DELAY TO LV OUTPUT
; IF TIMER = 0 THEN NEXT TEST
; ELSE DEC THE TIMER
;  IF THE TIMER = 0 THEN CALL START_LV_OUT
; ENDIF
;--------------------------------------------------------------------

CHECK_LVS_TIMER
	MOVF	RV_EVENT_TO_LVS_DELAY_TIMER,W	;GET THE TIME
	BTFSC	_Z				;IS IT ZERO?
	GOTO	CHECK_ECG_FLAG			;YES THEN SKIP THIS

	DECF	RV_EVENT_TO_LVS_DELAY_TIMER,F	;DEC TIMER
	BTFSC	_Z				;TIMER = 0?	
	CALL	START_LV_OUT			;YES THEN OUTPUT LVS

;-------------------------------------------------------------------
; CHECK IF WE NEED TO OUTPUT ECG DATA AND Z DATA 
; TO THE D/A CONVERTERS
;-------------------------------------------------------------------
CHECK_ECG_FLAG
	BTFSC	ECG_FLAG			;ECG OUTPUT FLAG SET?
	CALL	OUTPUT_NEXT_ECG			;YES THEN OUTPUT THE NEXT ECG DATA
	
	BTFSC	Z_FLAG				;Z FLAG SET?
	CALL	OUTPUT_Z_DATA			;YES THEN OUTPUT IMPED. DATA

;-------------------------------------------------------------------
; CHECK THE LED TIMERS
;-------------------------------------------------------------------
CHECK_RA_INTRIN_LED_TIME
	MOVF	RA_INTRIN_LED_TIMER,W	;TIMER = 0?
	BTFSC	_Z
	GOTO	CHECK_RA_PACE_LED_TIME	;YES THEN DO NEXT

	DECF	RA_INTRIN_LED_TIMER,F	;NO THEN DEC TIMER
	BTFSC	_Z			;TIME = 0?
	CALL	RA_INTRIN_LED_OFF	;YES THEN TURN OFF THE LED

CHECK_RA_PACE_LED_TIME
	MOVF	RA_PACE_LED_TIMER,W		;TIMER = 0?
	BTFSC	_Z
	GOTO	CHECK_RV_INTRIN_LED_TIME	;NO THEN DO NEXT

	DECF	RA_PACE_LED_TIMER,F		;ELSE TIMER--
	BTFSC	_Z				;TIMER = 0?
	CALL	RA_PACE_LED_OFF			;YES THEN TURN OFF LED

CHECK_RV_INTRIN_LED_TIME
	MOVF	RV_INTRIN_LED_TIMER,W		;TIMER = 0?
	BTFSC	_Z
	GOTO	CHECK_RV_PACE_LED_TIME		;YES THEN DO NEXT

	DECF	RV_INTRIN_LED_TIMER,F		;ELSE TIMER--
	BTFSC	_Z				;TIMER = 0?
	CALL	RV_INTRIN_LED_OFF		;YES THEN TURN OFF LED

CHECK_RV_PACE_LED_TIME
	MOVF	RV_PACE_LED_TIMER,W		;TIMER = 0?
	BTFSC	_Z
	GOTO	CHECK_LV_INTRIN_LED_TIME	;YES THEN DO NEXT

	DECF	RV_PACE_LED_TIMER,F		;ELSE TIMER--
	BTFSC	_Z				;TIMER = 0?
	CALL	RV_PACE_LED_OFF			;YES THEN TURN THE LED OFF

CHECK_LV_INTRIN_LED_TIME
	MOVF	LV_INTRIN_LED_TIMER,W		;TIMER = 0?
	BTFSC	_Z
	GOTO	CHECK_LVP_LED_TIMER		;YES THEN DO NEXT

	DECF	LV_INTRIN_LED_TIMER,F		;ELSE TIMER--
	BTFSC	_Z				;TIMER = 0?
	CALL	LV_LED_OFF			;YES THEN TURN OFF LED

CHECK_LVP_LED_TIMER
	MOVF	LVP_LED_TIMER,W			;TIMER = 0?
	BTFSC	_Z
	GOTO	CHECK_LVP_OUT_LED_TIMER		;YES THEN DO NEXT

	DECF	LVP_LED_TIMER,F			;ELSE TIMER--
	BTFSC	_Z				;TIMER =0?
	CALL	LVP_LED_OFF			;YES THEN TURN OFF LED

CHECK_LVP_OUT_LED_TIMER
	MOVF	LVP_OUT_LED_TIMER,W		;TIMER = 0?
	BTFSC	_Z
	GOTO	TEST_DELAY			;YES THEN DO NEXT

	DECF	LVP_OUT_LED_TIMER,F		;ELSE TIMER--
	BTFSC	_Z				;TIMER = 0?
	CALL	LVP_OUT_LED_OFF			;YES THEN TURN OFF LED


;*******************************************************************
; TEST THE DELAY TIMER
;*******************************************************************
TEST_DELAY
	BTFSS	DELAY_FLAG	;IS THE DELAY FLAG SET?
	GOTO	TEST_TMR0	;NO THEN DO NEXT

	MOVF	DELAY_LOW,W	;LOW BYTE = 0?
	BTFSC	_Z
	DECF	DELAY_HIGH,F	;YES THEN DEC THE HIGH BYTE FIRST

	DECF	DELAY_LOW,F	;DEC THE LOW BYTE

	MOVF	DELAY_LOW,W	;ARE BOTH BYTES = 0?
	IORWF	DELAY_HIGH,W
	BTFSC	_Z
	BCF	DELAY_FLAG	;YES THEN CLEAR THE DELAY FLAG
;***********************************************************************
;TEST FOR TMR0 INTERRUPT
; 200us INTERRUPT
;***********************************************************************
TEST_TMR0
	BTFSS	INTCON,T0IF	;TIMER 0 INTERRUPT?
	GOTO	INT_EXIT	;NO THEN EXIT
 
	MOVLW	T0_200		;RELOAD TMR0 FOR THE NEXT INTERRUPT
	MOVWF	TMR0
	BCF	INTCON,T0IF	;AND CLEAR THE INTERRUPT FLAG

;*******************************************************************
; CHECK FOR ANY EXTERNAL EVENTS
;*******************************************************************
	MOVF	PORTB,W		;READ PORTB
	ANDLW	b'11110000'	;MASK OFF ANY OTHER BITS
	MOVWF	PORTB_TEMP	;SAVE THE RESULT

	XORWF	PORTB_PREV,W	;ARE ALL THE BITS THE SAME AS BEFORE?
	BTFSC	_Z		;RESULT = 0?
	GOTO	TEST_A_OUT	;YES THEN TEST THE A FLAG

;*********************************************************************
; If one of the upper 4 INPUT bits on PORTB has changed then 
; we must determine if the port line for that bit went from a 
; low to high state. That is, was the previous state low and is 
; it high now. If so we set the flag for that bit. This bit
; generates an event in the main loop.
;**********************************************************************
TEST_V_IN
	BTFSC	V_PACE_IN	;V PACE IN = 1?
	GOTO	TEST_V_EN	;YES THEN SEE IF IT IS ENABLED

	BSF	IN_V_ENABLE	;NO THEN ENABLE IT FOR THE NEXT TIME
	GOTO	TEST_A_IN	;AND THEN TEST THE A INPUT LINE

TEST_V_EN
	BTFSS	IN_V_ENABLE	;ARE WE ENABLED AND INPUT = 1?
	GOTO	TEST_A_IN	;NO THEN TEST THE A INPUT LINE

	BSF	V_CAP_EVENT	;YES THEN SET THE CAPTURE FLAG
	BCF	IN_V_ENABLE	;DISABLE THIS INPUT EVENT
	BSF	ECG_PACE_FLAG	;SIGNAL TO SHOW PACE ON ECG

TEST_A_IN
	BTFSC	A_PACE_IN	;A PACE INPUT = 1?
	GOTO	TEST_A_EN	;YES THEN TEST TO SEE IF THE EVENT IS
				;ENABLED
	BSF	IN_A_ENABLE	;NO INPUT = 0, SO ENABLE FOR NEXT TIME
	GOTO	TEST_LVP_IN

TEST_A_EN
	BTFSS	IN_A_ENABLE	;ARE WE ENABLED AND INPUT = 1?
	GOTO	TEST_LVP_IN	;NO THEN TEST THE NEXT INPUT LINE

	BSF	A_CAP_EVENT	;YES THEN SET THE CAPTURE FLAG
	BCF	IN_A_ENABLE	;AND DISABLE THIS INPUT EVENT
	BSF	ECG_PACE_FLAG	;SIGNAL TO SHOW PACE ON ECG

TEST_LVP_IN
	BTFSC	LVP_IN		;LVP INPUT LINE = 1?
	GOTO	TEST_LVP_EN	;YES THEN CHECK TO SEE IF IT IS ENABLED

	BSF	IN_LVP_ENABLE	;NO THEN ENABLE IT FOR THE NEXT TIME
	GOTO	TEST_PVC_IN	;AND TEST THE LVS INPUT LINE

TEST_LVP_EN
	BTFSS	IN_LVP_ENABLE	;ARE WE ENABLED AND THE INPUT = 1?
	GOTO	TEST_PVC_IN	;NO THEN DO NEXT TASK

	BSF	LVP_CAP_EVENT	;YES THEN SIGNAL WE GOT LVP
	BCF	IN_LVP_ENABLE	;AND DISABLE FOR THE NEXT TIME

;------------------------------------------------------------------------
;CHECK TO SEE IF THE LVP LINE JUST WENT HIGH OUTSIDE OF THE 
;LVP WINDOW
;------------------------------------------------------------------------
	BTFSC	LVP_ERROR	;WAS THERE AN ERROR IN THIS CYCLE?
	GOTO	TEST_PVC_IN	;YES THEN SKIP THIS TEST

	BTFSC	LVP_ARM		;LVP ALREADY ARMED?
	GOTO	TEST_PVC_IN	;YES THEN SKIP THIS TEST

	MOVF	LVP_WIN_TIMER,W	;YES LVP_IN JUST WENT FROM LOW TO HIGH
	BTFSC	_Z		;IS THE WINDOW TIMER = 0?
	GOTO	TEST_PVC_IN	;YES THEN SKIP THIS PART

	BSF	LVP_ARM		;NO, THEN ARM THE TEST

;-------------------------------------------------------------------------
TEST_PVC_IN
	BTFSS	LVS_PVC_IN	;LVS_PVC INPUT = 0?
	GOTO	TEST_PVC_EN	;YES THEN TEST THE PVC ENABLE

	BSF	IN_LV_PVC_ENABLE	;ENABLE FOR THE NEXT TIME
	GOTO	SAVE_PORTB

TEST_PVC_EN
	BTFSS	IN_LV_PVC_ENABLE	;ARE WE ENABLED AND THE INPUT = 1?
	GOTO	SAVE_PORTB		;NO THEN DO NEXT TASK

	BSF	LV_PVC_EVENT			;YES THEN SIGNAL WE HAVE A PVC
	BCF	IN_LV_PVC_ENABLE	;AND DISABLE FOR NEXT TIME

SAVE_PORTB
	MOVF	PORTB_TEMP,W		;MAKE THE CURRENT BE THE OLD
	MOVWF	PORTB_PREV

;**********************************************************************	
TEST_A_OUT
	BTFSC	A_FLAG		;A OUTPUT DATA FLAG SET?
	CALL	OUTPUT_A_DATA	;YES THEN OUTPUT THE A CENELAC DATA

	BTFSC	V_FLAG		;V OUTPUT DATA FLAG SET?
	CALL 	OUTPUT_V_DATA	;YES THEN OUTPUT THE V CENELAC DATA

	BTFSC	LVS_FLAG		;LVS OUTPUT DATA FLAG SET?
	CALL	OUTPUT_LVS_DATA	;YES THEN OUTPUT THE LVS CENELAC DATA
			
INT_EXIT
	MOVF	PCLATH_TEMP,W	;RESTORE PCLATH
	MOVWF	PCLATH		
	SWAPF	STATUS_TEMP,W	;RESTORE BANK AND STATUS TO ORIGINAL STATE
	MOVWF	STATUS
	SWAPF	W_TEMP,F	;SWAP W_TEMP INTO WREG
	SWAPF	W_TEMP,W
	RETFIE

;**************************************************************
LVS_TABLE
;A LOOK-UP TABLE FOR THE LVS DELAY SWITCH
;ON ENTRY: EXPECTS AN INDEX VALUE FROM 0 TO 15 TO BE IN THE WREG
;ON EXIT: RETURNS A VALUE FROM 0 TO 150 BASED ON THE INDEX
;**************************************************************
	ADDWF	PCL,F
	RETLW	d'1'		;0
	RETLW	d'10'		;1
	RETLW	d'20'		;2
	RETLW	d'30'		;3
	RETLW	d'40'		;4
	RETLW	d'50'		;5
	RETLW	d'60'		;6
	RETLW	d'70'		;7
	RETLW	d'80'		;8
	RETLW	d'90'		;9
	RETLW	d'100'		;A
	RETLW	d'110'		;B
	RETLW	d'120'		;C
	RETLW	d'130'		;D
	RETLW	d'140'		;E
	RETLW	d'150'		;F

;**************************************************************
; MAIN TABLE
; CALLED FROM THE MAIN LOOP. ALL ROUTINES MUST END WITH A
; GOTO MAIN.
;**************************************************************
MAIN_TABLE
	ADDWF	PCL,F
	GOTO	DO_A_CAP
	GOTO	DO_V_CAP
	GOTO	DO_VA_DELAY
	GOTO	DO_LV_PVC
	GOTO	DO_A_INTRIN
	GOTO	DO_V_INTRIN	
	GOTO	DO_A_INT_POWERUP
	GOTO	DO_V_INT_POWERUP

#define	DO_A_CAP_INDX		0
#define DO_V_CAP_INDX		1

#define	DO_VA_DELAY_INDX	2
#define	DO_LV_PVC_INDX		3
#define	DO_A_INTRIN_INDX	4
#define	DO_V_INTRIN_INDX	5

#define	DO_A_INT_POWERUP_INDX	6
#define	DO_V_INT_POWERUP_INDX	7


;**************************************************************
ECG_STATE_TABLE
;**************************************************************
	ADDWF	PCL,F
	GOTO	GET_IDLE_ECG

	GOTO	GET_P_WAVE
	GOTO	GET_P_HOLD

	GOTO	GET_QRS_WAVE
	GOTO	GET_QRS_HOLD

	GOTO	GET_T_WAVE	
	GOTO	GET_T_HOLD

	GOTO	GET_PVC_WAVE

;*************************************************************
; ECG STATE MACHINE
;*************************************************************
#define	STATE_IDLE	0	;POWER UP STATE

#define	STATE_P		1	;OUTPUT PWAVE
#define	STATE_P_HOLD	2	;OUTPUT THE LAST WORD OF THE PWAVE TABLE (OVER AND OVER)

#define	STATE_QRS	3	;OUTPUT THE QRS WAVE
#define	STATE_QRS_HOLD	4	;OUTPUT THE LAST WORD OF THE QRS TABLE OVER AND OVER

#define	STATE_T		5	;OUTPUT THE T WAVE
#define	STATE_T_HOLD	6	;OUTPUT THE LAST WORD OF THE T WAVE OVER AND OVER	

#define	STATE_PVC	7	;OUTPUT A PVC WAVEFORM


;**************************************************************
; STARTUP (INIT.) ROUTINE STARTS HERE
; CALLED BY THE RESET VECTOR GOTO
;**************************************************************
STARTUP
	BANK_0
;----------------------------------------------------------
;CLEAR ALL VARIABLES
;----------------------------------------------------------
	MOVLW	LAST_USED-RAM_START	;NUMBER OF LOCATIONS TO CLEAR
	MOVWF	RAM_START

	MOVLW	RAM_START+1		;START AT MEM. LOCATION
	MOVWF	FSR

CLR_LOOP
	CLRF	INDF			;CLEAR MEM
	INCF	FSR,F			;ADD 1 TO THE MEM. PTR
	DECFSZ	RAM_START,F		;HAVE WE DONE ALL LOCATIONS?
	GOTO	CLR_LOOP		;NO THEN DO AGAIN

;-------------------------------------------------------------

;DISABLE ALL CS LINES
	BSF	Z_CS_NOT	;ALL CS LINES ARE ACTIVE LOW
	MOVF	PORTC_DATA,W
	MOVWF	PORTC

	BSF	A_CS_NOT
	BSF	V_CS_NOT
	BSF	LVS_CS_NOT

;SETUP CTS HIGH = OFF
	BSF	CTS_OUT		
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

;SETUP THE DATA DIRECTION REGISTERS
	BANK_1
	MOVLW	0xFF
	MOVWF	TRISA

	MOVLW	b'11110000'	;CHANGE PORTB,0 TO INPUT AFTER DEBUG
	MOVWF	TRISB

	MOVLW	b'10010000'
	MOVWF	TRISC

	MOVLW	0x00
	MOVWF	TRISC
	MOVWF	TRISD

;SETUP THE INTERNAL REGISTERS
	MOVLW	b'10000000'	;NO PORTB PULL-UPS, PRESCALE 1:2 (800ns tmr0 CLOCK)
	MOVWF	OPTION_REG

	MOVLW	0xFF		;NO ANALOG INPUTS, ARE ALL DIGITAL I/O
	MOVWF	ADCON1

;SETUP THE SPI REGISTERS
	BANK_0
	MOVLW	SSPCON_INIT	;SPI ENABLED
	MOVWF	SSPCON

	BANK_1
	CLRF	SSPSTAT
	BCF	SSPSTAT,SMP	;DATA SAMPLED AT THE MIDDLE (NOT USED)
	BSF	SSPSTAT,CKE	;DATA TRANSMITTED ON RISING EDGE OF CLK
	BANK_0
	CLRF	SSPBUF
	BCF	PIR1,SSPIF

;SETUP THE INTERNAL DATA VALUES
	MOVLW	A_DATA_LENGTH
	MOVWF	A_COUNT
	MOVLW	V_DATA_LENGTH
	MOVWF	V_COUNT
	MOVLW	LVS_DATA_LENGTH
	MOVWF	LVS_COUNT
	
	MOVLW	ECG_TIME_INIT
	MOVWF	ECG_TIMER

	BSF	POWER_UP_FLAG		;WE ARE IN POWER UP MODE

	BSF	HASH_HIGH		;SIGNAL TO OUTPUT A FULL SCALE
					;VALUE WHEN HASHING THE ECG
					;OUTPUT.


;----------------------------------------------------------------------
;SETUP THE PROGRAMMABLE REGISTERS
;----------------------------------------------------------------------

;SET THE INITIAL AV DELAY
	MOVLW	LOW AV_DELAY_INIT	;INITAL AV DELAY
	MOVWF	AV_DELAY_LOW
	MOVLW	HIGH AV_DELAY_INIT
	MOVWF	AV_DELAY_HIGH

;SET THE INITAL VA DELAY		;INITIAL VA DELAY
	MOVLW	LOW VA_DELAY_INIT
	MOVWF	VA_DELAY_LOW
	MOVLW	HIGH VA_DELAY_INIT
	MOVWF	VA_DELAY_HIGH

;------------------------------------------------------------------------
;SETUP THE POWER ON STATE
;------------------------------------------------------------------------

;SETUP THE AV DELAY
	MOVF	AV_DELAY_LOW,W
	MOVWF	DELAY_LOW
	MOVF	AV_DELAY_HIGH,W
	MOVWF	DELAY_HIGH

;SETUP THE ECG VARIABLES TO OUTPUT THE P WAVE
	MOVLW	STATE_P
	MOVWF	ECG_STATE

	MOVLW	LOW P_WAVE_LEN		;TOTAL NUMBER OF WORDS IN THE P WAVE
	MOVWF	ECG_COUNT_LOW
	MOVLW	HIGH P_WAVE_LEN
	MOVWF	ECG_COUNT_HIGH

;SETUP THE IMPEDANCE VARIABLES TO OUTPUT THE IMPED. DATA

	MOVLW	Z_DATA_LENGTH		;GET THE TABLE LENGTH
	MOVWF	Z_COUNT			;GET THE COUNT

;ENABLE ALL LOCAL HARDWARE EVENT FLAGS
	MOVLW	0xFF
	MOVWF	IN_EVENT_ENABLE_FLAGS

;SETUP THE LV PVC SWITCH ACTION
	MOVLW	DO_LV_PVC_INDX
	MOVWF	LV_PVC_INDX

;SETUP THE POWER UP STATE FOR THE MAIN LOOP
	MOVLW	d'10'		;NO EVENTS ALLOWED
	MOVWF	DELAY_LOW	;TIMEOUT AFTER 10 msecs. AND DO A_INTRINSIC
	CLRF	DELAY_HIGH

	MOVLW	DO_A_INT_POWERUP_INDX
	MOVWF	TIMEOUT_INDX

	BSF	DELAY_FLAG

;SETUP THE PVC SWITCH ROUTINE INDEX POINTER
;WE INITALIZE IT NOW BECAUSE IT ALWAYS POINTS TO THE SAME ROUTIBE
;THIS SHOULD ALWAYS POINT TO THE SAME ROUTINE (FOR NOW)
	MOVLW	DO_LV_PVC_INDX
	MOVWF	LV_PVC_INDX

;INITIALIZE TIMER 1
	CLRF	T1CON		;LOAD TIMER 1 WITH THE 1ms INTERRUPT VALUE
	MOVLW	LOW T1_1MS
	MOVWF	TMR1L
	MOVLW	HIGH T1_1MS
	MOVWF	TMR1H

	BSF	T1CON,TMR1ON	;START TIMER 1 RUNNING

;INITIALIZE TIMER 0
	MOVLW	T0_200		;SETUP TIMER 0 FOR 200us INTERRUPTS
	MOVWF	TMR0
	
	CLRF	INTCON		;CLEAR ALL FLAGS
	BSF	INTCON,T0IE	;ENABLE TIMER 0 INTERRUPTS

	BANK_1			;ENABLE TMR1 INTERRUPTS
	BSF	PIE1,TMR1IE
	BANK_0

	BSF	INTCON,PEIE	;ENABLE PERIPHERAL INTERRUPTS
	BSF	INTCON,GIE	;ENABLE GLOBAL INTERRUPTS
	
;FALLS THROUGH TO MAIN
;************************************************************************
; MAIN LOOP STARTS HERE
;************************************************************************
MAIN

;READ THE LVS DELAY BCD SWITCH
READ_LVS_SW
	CALL	READ_LVS_DELAY	;READ THE BCD SWITCH

	CALL	TEST_LVP_WINDOW	;TEST FOR LVP OUT OF WINDOW

;---------------------------------------------------------------------
; IF A_CAP_EVENT = TRUE	THEN
;	IF A_ACTION_FLAG = TRUE THEN
;		CALL A_CAP_INDX
;	ELSE A_CAP_EVENT = 0
;	ENDIF
; ENDIF
;---------------------------------------------------------------------
MAIN_A
	BTFSS	A_CAP_EVENT	;DID WE HAVE AN A CAPTURE?
	GOTO	MAIN_V		;NO THEN TEST FOR V CAP

	BTFSC	A_ACTION_FLAG	;ARE A EVENTS ENABLED?
	GOTO	MAIN_A_CAP	;YES THEN RUN IT

	BCF	A_CAP_EVENT	;NO THEN CLEAR THE FLAG
	GOTO	MAIN_V		;AND TEST FOR V EVENT

MAIN_A_CAP
	MOVF	A_CAP_INDX,W	;GET THE INDEX
	GOTO	MAIN_TABLE	;AND THEN RUN IT

;---------------------------------------------------------------------
; IF V_CAP_EVENT = TRUE	THEN
;	IF V_ACTION_FLAG = TRUE THEN
;		CALL V_CAP_INDX
;	ELSE V_CAP_EVENT = 0
;	ENDIF
; ENDIF
;---------------------------------------------------------------------

MAIN_V
	BTFSS	V_CAP_EVENT		;DID WE HAVE A V CAP EVENT?
	GOTO	MAIN_PVC		;NO THEN NEXT TEST

	BTFSC	V_ACTION_FLAG		;ARE V EVENTS ENABLED?
	GOTO	MAIN_V_CAP		;YES THEN RUN IT

	BCF	V_CAP_EVENT		;NO THEN CLEAR THE FLAG
	GOTO	MAIN_PVC

MAIN_V_CAP
	MOVF	V_CAP_INDX,W		;ELSE RUN IT
	GOTO	MAIN_TABLE

MAIN_PVC
	BTFSS	LV_PVC_EVENT		;DID WE HAVE A PVC BUTTON PRESS?
	GOTO	MAIN_TOUT		;NO THEN CHECK FOR A TIMEOUT

	BTFSC	LV_PVC_ACTION_FLAG	;ARE PVC EVENTS ENABLED?
	GOTO	LV_PVC_CAP		;YES THEN RUN IT

	BCF	LV_PVC_EVENT		;NO THEN CLEAR THE EVENT FLAG
	GOTO	MAIN_TOUT		;AND TEST FOR TIMEOUT

LV_PVC_CAP
	MOVF	LV_PVC_INDX,W		;YES THEN GET THE INDEX
	GOTO	MAIN_TABLE		;AND DO THE ROUTINE

MAIN_TOUT
	BTFSC	DELAY_FLAG		;IS THE DELAY FLAG SET?
	GOTO	MAIN			;YES THEN WAIT

	MOVF	TIMEOUT_INDX,W		;ELSE RUN THE TIMEOUT ROUTINE
	GOTO	MAIN_TABLE

;*************************************************************************
; MAIN LOOP ENDS HERE
;*************************************************************************


;*************************************************************************
; CHECK TO SEE IF THE LVP INPUT IS HIGH ANYTIME WE ARE OUTSIDE OF THE 
; LVP WINDOW (LVP_WIN_TIMER)
; IF LVP_IN = 1 AND LVP_WIN_TIMER = 0 THEN
;     LVP_ERROR = 1
;     V_CAP_EVENT = 1
; ENDIF
;
; ON ENTRY: EXPECTS NONE
; ON EXIT: RETURNS NONE
;
TEST_LVP_WINDOW
;*************************************************************************
	BTFSC	LVP_ERROR		;DO WE ALREADY HAVE AN ERROR
	RETURN				;YES THEN EXIT

	MOVF	LVP_WIN_TIMER,W		;ARE WE OUTSIDE THE WINDOW?, 
					;TIMER = 0?
	BTFSS	_Z
	RETURN				;NO THEN EXIT

	BTFSS	LVP_IN			;LVP INPUT = 1?
	RETURN				;NO THEN EXIT

	BSF	LVP_ERROR		;YES THEN THIS IS AN ERROR
	BSF	V_CAP_EVENT		;AND WE SHOULD SIGNAL A CAPTURE
					;EVENT.

TEST_LVP_WIN_EXIT
	RETURN

;***************************************************************************
START_V_CEN
; START THE RV CENELAC OUTPUT
;***************************************************************************
	MOVLW	V_DATA_LENGTH			;LOAD THE V IEGM DATA LENGTH
	MOVWF	V_COUNT			

	CLRF	CEN_INDX			;CLEAR THE V POINTER

	BSF	V_FLAG				;START THE V OUTPUT
	RETURN

;***************************************************************************
START_LVS_DELAY
;START THE LEFT VENTRICLE SENSE OUTPUT AFTER A DELAY THAT IS STORE IN LVS_DELAY
;***************************************************************************
	MOVF	LVS_DELAY,W			;GET THE DELAY VALUE
	MOVWF	RV_EVENT_TO_LVS_DELAY_TIMER	;AND STORE IT IN THE TIMER

	RETURN

;***************************************************************************
START_V_CEN_AND_LVS_DELAY
;***************************************************************************
	CALL	START_V_CEN
	CALL	START_LVS_DELAY
	RETURN

;******************************************************************************
START_P_WAVE

	MOVLW	LOW P_WAVE_LEN		;GET THE P WAVE LENGTH
	MOVWF	ECG_COUNT_LOW
	MOVLW	HIGH P_WAVE_LEN
	MOVWF	ECG_COUNT_HIGH

	MOVLW	STATE_P			;STATE = P WAVE
	MOVWF	ECG_STATE

	CLRF	ECG_INDX_LOW		;CLEAR THE INDEX
	CLRF	ECG_INDX_HIGH

	BSF	ECG_FLAG		;ECG OUTPUT ON

	RETURN
;******************************************************************************
START_Z
;SETUP THE IMPEDANCE OUTPUT
;SET THE Z_FLAG WHICH WILL START THE Z OUTPUT IN THE NEXT
;200us INTERRUPT
;******************************************************************************
	BTFSS	PVC_Z_FLAG	;IS THE PVC Z FLAG SET?
	GOTO	START_Z_NORM	;NO THEN OUTPUT A NORMAL Z WAVEFORM

	MOVLW	PVC_Z_DELAY	;YES THEN LOAD THE PVC DELAY
	MOVWF	Z_DELAY_TIMER
	BCF	PVC_Z_FLAG

	RETURN			;AND EXIT

START_Z_NORM
	MOVLW	Z_DELAY		;LOAD THE NORMAL Z DELAY
	MOVWF	Z_DELAY_TIMER

	RETURN

;****************************************************************************
GET_Z_DATA
; GET THE NEXT VALUE FROM THE IMP_TABLE, THE TWO 8 BIT VALUES ARE STORED
; IN TEMP3, TEMP4 (LOW,HIGH)
; IF THE END OF THE TABLE IS REACHED AFTER CALLING THIS ROUTINE, THE 
; INDEX PTR IS CLEARED AND THE DATA COUNT VARIABLE IS RELOADED.
;
; ON ENTRY: EXPECTS NONE
; ON EXIT: RETURNS THE NEXT TWO TABLE VALUES IN TEMP3 AND TEMP4
;****************************************************************************
	BCF	_C			;INDEX *=2
	RLF	Z_INDX_LOW,W
	MOVWF 	TEMP1

	RLF	Z_INDX_HIGH,W
	MOVWF	TEMP2

	MOVLW	HIGH IMP_TABLE		;ADD THE LOW BYTES
	ADDWF	TEMP2,F

	MOVLW	LOW IMP_TABLE		;ADD THE LOW BYTES
	ADDWF	TEMP1,F

	BTFSC	_C			;WAS A CARRY PRODUCED?
	INCF	TEMP2,F			;YES THEN ADD 1 TO THE HIGH RESULT

	MOVLW	HIGH IMP_TABLE_LOAD
	MOVWF	PCLATH

	CALL	IMP_TABLE_LOAD		;GET THE BYTE FROM THE TABLE
	MOVWF	TEMP4			;STORE THE RESULT (HIGH)			

	INCF	TEMP1,F			;PTR++ (INC THE LOW BYTE)
	BTFSC	_Z			;ROLL OVER?
	INCF	TEMP2,F			;YES THEN ADD 1 TO THE HIGH BYTE

	CALL	IMP_TABLE_LOAD		;GET THE BYTE FROM THE TABLE
	MOVWF	TEMP3			;STORE THE RESULT (LOW)
	PAGE_0

	DECF	Z_COUNT,F		;DEC THE WORD COUNT

	BTFSS	_Z			;COUNT = 0?
	GOTO	INC_Z_INDX		;NO THEN INCREMENT THE INDX

	BCF	Z_FLAG			;ELSE STOP THE Z OUTPUT
	GOTO	GET_Z_EXIT		;AND EXIT
	
INC_Z_INDX
	INCF	Z_INDX_LOW,F		;ADD 1 TO THE ARRAY INDEX
	BTFSC	_Z			;DID WE ROLLOVER?
	INCF	Z_INDX_HIGH,F		;YES THEN ADD 1 TO THE HIGH BYTE
GET_Z_EXIT	
	
	RETURN

;****************************************************************************
OUTPUT_Z_DATA
;OUTPUT THE NEXT 12 BIT DATA WORD TO THE IMPEDANCE DAC
;ON ENTRY:NONE
;ON EXIT:NONE
;****************************************************************************
	BCF	Z_CS_NOT		;SET THE Z CS LINE LOW
	MOVF	PORTC_DATA,W
	MOVWF	PORTC

	CALL	GET_Z_DATA

;WAIT FOR THE SPI CHANNEL IF NOT READY FOR NEW DATA
WAIT_Z1
	BTFSS	PIR1,SSPIF		;FLAG = 1?
	GOTO	WAIT_Z1			;NO THEN WAIT

	BCF	PIR1,SSPIF		;YES THEN CLEAR THE FLAG
	MOVF	TEMP4,W			;AND WRITE THE DATA (MSB FIRST)
	MOVWF	SSPBUF

;WAIT FOR THE SPI CHANNEL IF NOT READY FOR NEW DATA
WAIT_Z2
	BTFSS	PIR1,SSPIF		;FLAG = 1?
	GOTO	WAIT_Z2			;NO THEN WAIT

	BCF	PIR1,SSPIF		;YES THEN CLEAR THE FLAG
	MOVF	TEMP3,W			;AND WRITE THE DATA (LSB)
	MOVWF	SSPBUF

;WAIT FOR THE DATA TO BE SHIFTED OUT BEFORE SETTING CS HIGH
WAIT_Z3
	BTFSS	PIR1,SSPIF		;FLAG = 1?
	GOTO	WAIT_Z3			;NO THEN WAIT


;SET THE ECG CS LINE HIGH
	BSF	Z_CS_NOT
	MOVF	PORTC_DATA,W
	MOVWF	PORTC

	RETURN

;****************************************************************************


;****************************************************************************
START_LV_OUT
;***************************************************************************
	CLRF	LVS_INDX
	MOVLW	LVS_DATA_LENGTH
	MOVWF	LVS_COUNT
	BSF	LVS_FLAG				;SET THE OUTPUT FLAG ON

	CALL	LV_LED_ON			;TURN ON THE LED


	RETURN
;****************************************************************************
OUTPUT_ECG_DATA
;OUTPUT THE 12 BIT (2 BYTE) ECG DATA OUT TO THE ECG DAC.
;ON ENTRY: 
; VALUES TO BE OUTPUT IN TEMP3,TEMP4 (LOW,HIGH)
; IF WREG = 0, THEN THE VALUE TO BE OUTPUT IS SAVED AT THE END OF THIS
; ROUTINE.
;ON EXIT: NONE RETURNED
;****************************************************************************

	IORLW	0x00		;WREG = 0?
	BTFSS	_Z
	RETURN			;NO THEN DO NOT OUTPUT A WORD
				;YES THEN OUTPUT THE WORD IN TEMP3,TEMP4

;SET THE ECG CS LINE LOW
	BCF	ECG_CS_NOT
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

;WAIT FOR THE SPI CHANNEL IF NOT READY FOR NEW DATA
WAIT_ECG1
	BTFSS	PIR1,SSPIF		;FLAG = 1?
	GOTO	WAIT_ECG1		;NO THEN WAIT

	BCF	PIR1,SSPIF		;YES THEN CLEAR THE FLAG
	MOVF	TEMP4,W			;AND WRITE THE DATA
	MOVWF	SSPBUF

;WAIT FOR THE SPI CHANNEL IF NOT READY FOR NEW DATA
WAIT_ECG2
	BTFSS	PIR1,SSPIF		;FLAG = 1?
	GOTO	WAIT_ECG2		;NO THEN WAIT

	BCF	PIR1,SSPIF		;YES THEN CLEAR THE FLAG
	MOVF	TEMP3,W			;AND WRITE THE DATA
	MOVWF	SSPBUF

;WAIT FOR THE DATA TO BE SHIFTED OUT BEFORE SETTING CS HIGH
WAIT_ECG3
	BTFSS	PIR1,SSPIF		;FLAG = 1?
	GOTO	WAIT_ECG3		;NO THEN WAIT


;SET THE ECG CS LINE HIGH
	BSF	ECG_CS_NOT
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

	BTFSS	ECG_DONE
	GOTO	OUTPUT_ECG_EXIT

	BCF	ECG_DONE
	BCF	ECG_FLAG

OUTPUT_ECG_EXIT
	RETURN
;***************************************************************************
OUTPUT_NEXT_ECG
;***************************************************************************
	MOVF	ECG_STATE,W		;GET THE ECG STATE
	CALL	ECG_STATE_TABLE		;THIS PUTS THE NEXT ECG VALUE TO 
					;BE OUTPUT IN TEMP3,TEMP4.

;-------------------------------------------------------------
; OUTPUT A HASHED ECG WAVEFORM WHENEVER THE LVP_INPUT IS HIGH
;-------------------------------------------------------------
	BTFSC	HASH_ON			;IS HASHING ON ALREADY?
	GOTO	TEST_HIGH_LOW		;YES THEN CHECK WHICH		

;------------------------------
;NO HASHING IS NOT ON
;------------------------------
	BTFSS	LVP_IN			;IS LVP_IN HIGH?
	GOTO	TEST_PACE		;NO THEN EXIT

	BSF	HASH_ON			;YES THEN SET FLAG

;------------------------------
; YES LVP IN IS HIGH
;------------------------------

TEST_HIGH_LOW
	BTFSC	HASH_HIGH		;OUTPUT A HIGH VALUE?
	GOTO	HASH_HIGH_OUT		;YES THEN OUTPUT A HIGH VALUE

;--------------------------------------------------------------
;OUTPUT A LOW OR A ZERO SCALE VALUE
;--------------------------------------------------------------

	INCF	LVP_COUNT,F		;ADD 1 TO THE COUNT
	MOVF	LVP_COUNT,W
	SUBLW	0x06
	BTFSS	_Z			;COUNT = 6?
	GOTO	LOW_OUT			;NO THEN OUTPUT A LOW VALUE

	BSF	HASH_HIGH		;YES, SIGNAL A HIGH NEXT TIME
	CLRF	LVP_COUNT		;CLEAR THE COUNT
	BCF	HASH_ON			;HASHING IS OFF

	MOVLW	0x08			;WRITE A BASELINE VALUE
	MOVWF	TEMP4
	MOVLW	0x00
	MOVWF	TEMP3	
	
	GOTO	SIGNAL_ECG_OUT

LOW_OUT
	MOVLW	0x00			;ELSE OUTPUT A LOW VALUE
	MOVWF	TEMP3
	MOVLW	0x00
	MOVWF	TEMP4

	GOTO	SIGNAL_ECG_OUT

;-------------------------------------------------------------------
;OUTPUT A HIGH OR FULL SCALE VALUE
;-------------------------------------------------------------------
HASH_HIGH_OUT
	INCF	LVP_COUNT,F		;ADD 1 TO THE LVP COUNT
	MOVF	LVP_COUNT,W
	SUBLW	0x05
	BTFSS	_Z			;COUNT = 5?
	GOTO 	HIGH_OUT		;NO THEN OUTPUT A HIGH VALUE

	BCF	HASH_HIGH		;ELSE SIGNAL TO OUTPUT A LOW
	CLRF	LVP_COUNT		;AND CLEAR THE COUNT

HIGH_OUT
	MOVLW	0xFF			;OUTPUT A HIGH VALUE
	MOVWF	TEMP3
	MOVLW	0xFF
	MOVWF	TEMP4

	GOTO	SIGNAL_ECG_OUT
	
;-------------------------------------------------------------
; THE ECG OUTPUT IS MODIFIED IF A PACING PULSE IS DETECTED
;-------------------------------------------------------------
TEST_PACE
	BTFSS	ECG_PACE_FLAG		;DID WE GET A PACE PULSE
	GOTO	ECG_DATA_OUT		;NO THEN SKIP THIS


	MOVF	ECG_PACE_COUNT,W	;IS THIS THE FIRST TIME THRU?
	BTFSS	_Z
	GOTO	INC_ECG_PACE_COUNT	;NO THEN SKIP THIS
	
	MOVF	ECG_LOW,W		;YES THEN COPY THE PREVIOUS DATA
	MOVWF	ECG_PREV_LOW
	MOVF	ECG_HIGH,W
	MOVWF	ECG_PREV_HIGH

INC_ECG_PACE_COUNT
	INCF	ECG_PACE_COUNT,F	;ADD 1 TO COUNT
	MOVF	ECG_PACE_COUNT,W	;COUNT = 5?
	SUBLW	0x05
	BTFSS	_Z
	GOTO	OUTPUT_PACED_ECG	;NO THEN OUTPUT A FULL SCALE VALUE

	MOVF	ECG_PREV_LOW,W
	MOVWF	TEMP3

	MOVF	ECG_PREV_HIGH,W
	MOVWF	TEMP4

	CLRF	ECG_PACE_COUNT		;YES THEN CLEAR THE COUNT
	BCF	ECG_PACE_FLAG		;CLEAR THE PACE FLAG

	GOTO	SIGNAL_ECG_OUT

OUTPUT_PACED_ECG
	MOVLW	0xFF			;OUTPUT A FULLSCALE VALUE
	MOVWF	TEMP3
	MOVWF	TEMP4

SIGNAL_ECG_OUT
	MOVLW	0x00			;WREG=0, OUTPUT DATA

;----------------------------------------------------------------------
;IF THIS IS A HOLD STATE, THEN A -1 IS RETURNED BY THE ECG_STATE_TABLE
;ROUTINE. OUTPUT_ECG_DATA ONLY OUTPUTS A WORD OF ECG DATA IF THE VALUE
;PASSED TO IT IN WREG IS 0. THE WORD VALUES TO BE OUTPUT ARE IN
;TEMP3, AND TEMP4. (LOW,HIGH)
;----------------------------------------------------------------------
ECG_DATA_OUT
	CALL	OUTPUT_ECG_DATA		;YES THEN OUTPUT THE DATA
	RETURN				;RETURN

;****************************************************************************
OUTPUT_A_DATA
;OUTPUT THE NEXT DATA VALUE IN THE CENELEC TABLE TO THE A IEGM D/A 
;CONVERTER
;ON ENTRY: EXPECTS NONE
;ON EXIT: RETURNS NONE
;****************************************************************************
	CALL	GET_NEXT_CEN

;SET THE CS FOR THE A CENELEC LOW
	BCF	A_CS_NOT
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

;WAIT FOR THE SPI PORT TO BE READY
WAIT_A1
	BTFSS	PIR1,SSPIF		;FLAG = 1?
	GOTO	WAIT_A1			;NO THEN WAIT

	BCF	PIR1,SSPIF		;YES THEN CLEAR THE FLAG
	MOVF	TEMP3,W			;AND WRITE THE DATA
	MOVWF	SSPBUF

	INCF	CEN_INDX,F		;ADD 1 TO THE INDEX

	CALL	GET_NEXT_CEN		

;WAIT FOR THE SPI PORT TO BE READY
WAIT_A2
	BTFSS	PIR1,SSPIF		;FLAG = 1?
	GOTO	WAIT_A2			;NO THEN WAIT

	BCF	PIR1,SSPIF		;YES THEN CLEAR THE FLAG
	MOVF	TEMP3,W			;AND WRITE THE DATA
	MOVWF	SSPBUF

;WAIT FOR THE DATA TO BE SHIFTED OUT
WAIT_A3
	BTFSS	PIR1,SSPIF		;FLAG = 1?
	GOTO	WAIT_A3			;NO THEN WAIT	

;SET THE CS FOR THE A CENELEC HIGH
	BSF	A_CS_NOT
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

	INCF	CEN_INDX,F		;ADD 1 TO THE LOW INDEX 

	DECF	A_COUNT,F		;DEC THE WORD COUNT
	BTFSS	_Z			
	GOTO	EXIT_A			;NO THEN EXIT

	MOVLW	A_DATA_LENGTH		;GET THE TABLE LENGTH
	MOVWF	A_COUNT			;SAVE IT
	BCF	A_FLAG			;AND CLEAR THE DATA OUT FLAG

EXIT_A
	RETURN

;*******************************************************************************
GET_NEXT_CEN
;GET THE NEXT CENELEC WAVEFORM DATA VALUE FROM THE TABLE
;ON ENTRY: EXPECTS NONE
;ON EXIT: THE TABLE VALUE IS RETURNED IN TEMP3
;*******************************************************************************
	MOVLW	HIGH CEN_TABLE
	MOVWF	PCLATH

	MOVF	CEN_INDX,W
	CALL	CEN_TABLE
	MOVWF	TEMP3

	RETURN
;*******************************************************************************	
OUTPUT_V_DATA
;OUTPUT THE NEXT DATA VALUE IN THE CENELEC TABLE TO THE V IEGM D/A 
;CONVERTER
;ON ENTRY: EXPECTS NONE
;ON EXIT: RETURNS NONE
;*******************************************************************************
	CALL	GET_NEXT_CEN

;SET THE CS FOR THE A CENELEC LOW
	BCF	V_CS_NOT
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

;WAIT FOR THE SPI PORT TO BE READY
WAIT_V1
	BTFSS	PIR1,SSPIF		;FLAG = 1?
	GOTO	WAIT_V1			;NO THEN WAIT

	BCF	PIR1,SSPIF		;YES THEN CLEAR THE FLAG
	MOVF	TEMP3,W			;AND WRITE THE DATA
	MOVWF	SSPBUF

	INCF	CEN_INDX,F		;ADD 1 TO THE INDEX

	CALL	GET_NEXT_CEN		

;WAIT FOR THE SPI PORT TO BE READY
WAIT_V2
	BTFSS	PIR1,SSPIF		;FLAG = 1?
	GOTO	WAIT_V2			;NO THEN WAIT

	BCF	PIR1,SSPIF		;YES THEN CLEAR THE FLAG
	MOVF	TEMP3,W			;AND WRITE THE DATA
	MOVWF	SSPBUF

;WAIT FOR THE DATA TO BE SHIFTED OUT
WAIT_V3
	BTFSS	PIR1,SSPIF		;FLAG = 1?
	GOTO	WAIT_V3			;NO THEN WAIT	

;SET THE CS FOR THE A CENELEC HIGH
	BSF	V_CS_NOT
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

	INCF	CEN_INDX,F		;ADD 1 TO THE LOW INDEX 

	DECF	V_COUNT,F		;DEC THE WORD COUNT
	BTFSS	_Z			
	GOTO	EXIT_V			;NO THEN EXIT

	MOVLW	V_DATA_LENGTH		;GET THE TABLE LENGTH
	MOVWF	V_COUNT			;SAVE IT
	BCF	V_FLAG			;AND CLEAR THE DATA OUT FLAG

EXIT_V
	RETURN

;******************************************************************************
OUTPUT_LVS_DATA
;OUTPUT THE LVS OR LV CENELAC DATA TO THE LVS DAC
;ON ENTRY: EXPECTS NONE
;ON EXIT: RETURNS NONE
;******************************************************************************

;SET THE CS FOR THE LVS CENELEC LOW
	BCF	LVS_CS_NOT
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

	MOVLW	HIGH CEN_TABLE		;GET THE UPPER ADDRESS OF THE TABLE
	MOVWF	PCLATH

	MOVF	LVS_INDX,W		;GET THE INDEX PTR
	CALL	CEN_TABLE
	MOVWF	TEMP3

;WAIT FOR THE SPI PORT TO BE READY
WAIT_LVS1
	BTFSS	PIR1,SSPIF		;FLAG = 1?
	GOTO	WAIT_LVS1		;NO THEN WAIT

	BCF	PIR1,SSPIF		;YES THEN CLEAR THE FLAG
	MOVF	TEMP3,W			;AND WRITE THE DATA TO THE DAC
	MOVWF	SSPBUF

	INCF	LVS_INDX,F		;ADD 1 TO THE INDEX

	MOVLW	HIGH CEN_TABLE		;GET THE HIGH ADDRESS
	MOVWF	PCLATH

	MOVF	LVS_INDX,W		;GET THE INDEX (PTR)
	CALL	CEN_TABLE		;GET THE BYTE FROM THE TABLE
	MOVWF	TEMP3			;SAVE IT
		

;WAIT FOR THE SPI PORT TO BE READY
WAIT_LVS2
	BTFSS	PIR1,SSPIF		;FLAG = 1?
	GOTO	WAIT_LVS2		;NO THEN WAIT

	BCF	PIR1,SSPIF		;YES THEN CLEAR THE FLAG
	MOVF	TEMP3,W			;AND WRITE THE DATA
	MOVWF	SSPBUF

;WAIT FOR THE DATA TO BE SHIFTED OUT
WAIT_LVS3
	BTFSS	PIR1,SSPIF		;FLAG = 1?
	GOTO	WAIT_LVS3		;NO THEN WAIT	

;SET THE CS FOR THE A CENELEC HIGH
	BSF	LVS_CS_NOT		;SET THE CS LINE HIGH
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

	INCF	LVS_INDX,F		;ADD 1 TO THE LOW INDEX 

	DECF	LVS_COUNT,F		;DEC THE WORD COUNT
	BTFSS	_Z			
	GOTO	EXIT_LVS			;NO THEN EXIT

	MOVLW	LVS_DATA_LENGTH		;GET THE TABLE LENGTH
	MOVWF	LVS_COUNT		;SAVE IT
	BCF	LVS_FLAG			;AND CLEAR THE DATA OUT FLAG

EXIT_LVS
	RETURN
;******************************************************************************


;******************************************************************************
; OUTPUT THE SAME BYTE AS PREVIOUS (IDLE STATE, DO NOTHING)
;******************************************************************************
GET_P_HOLD
; THIS IS THE OUTPUT AFTER THE P WAVE, HOLD THE DAC OUTPUT AT THE LAST
; VALUE.
;ON ENTRY: EXPECTS NONE
;ON EXIT: RETURNS A -1
;*****************************************************************************

	RETLW	-1	;DO NOTHING, BUT TELL THE CALLING PROGRAM
			;NOT TO OUTPUT ANY DATA TO THE DAC

;******************************************************************************

;******************************************************************************
GET_QRS_HOLD
; THIS IS THE OUTPUT AFTER THE QRS WAVE, HOLD THE DAC OUTPUT AT THE LAST
; VALUE.
; A TIMER IS DECREMENT EACH TIME THIS ROUTINE IS CALLED, THIS TIMER CONTROLS
; THE DELAY OF THE END OF THE QRS WAVE TO THE T WAVE.
;
;ON ENTRY: EXPECTS NONE
;ON EXIT: RETURNS NONE
;******************************************************************************
	DECF	QRS_TO_T_TIMER,F	;DEC THE QRS TO T WAVE DELAY TIMER
					;COUNT = 0?
	BTFSS	_Z			;NO THEN WAIT
	GOTO	EXIT_QRS_HOLD

	MOVLW	STATE_T			;GO TO T WAVE STATE
	MOVWF	ECG_STATE
	
	MOVLW	LOW T_WAVE_LEN		;LOAD THE T WAVE LENGTH
	MOVWF	ECG_COUNT_LOW
	MOVLW	HIGH T_WAVE_LEN
	MOVWF	ECG_COUNT_HIGH

	CLRF	ECG_INDX_LOW		;AND CLEAR THE INDEX
	CLRF    ECG_INDX_HIGH
	BSF 	ECG_FLAG

EXIT_QRS_HOLD
	RETLW	-1
;******************************************************************************
; OUTPUT THE SAME BYTE AS PREVIOUS (IDLE STATE, DO NOTHING)
;******************************************************************************
GET_T_HOLD
	RETLW	-1
; THIS IS THE OUTPUT AFTER THE P WAVE, HOLD THE DAC OUTPUT AT THE LAST
; VALUE.
;ON ENTRY: EXPECTS NONE
;ON EXIT: RETURNS A -1
;*****************************************************************************

;******************************************************************************
; OUTPUT THE SAME BYTE AS PREVIOUS (IDLE STATE, DO NOTHING)
;******************************************************************************
GET_IDLE_ECG
	RETLW	-1
; THIS IS THE OUTPUT AFTER THE P WAVE, HOLD THE DAC OUTPUT AT THE LAST
; VALUE.
;ON ENTRY: EXPECTS NONE
;ON EXIT: RETURNS A -1
;*****************************************************************************

;******************************************************************************
; OUTPUT THE P WAVE DATA
;
; ON ENTRY: EXPECTS NONE
; ON EXIT: RETURNS NONE
;******************************************************************************
GET_P_WAVE

	BCF	_C			;INDEX *=2
	RLF	ECG_INDX_LOW,W
	MOVWF 	TEMP1

	RLF	ECG_INDX_HIGH,W
	MOVWF	TEMP2

	MOVLW	HIGH P_WAVE		;ADD THE HIGH BYTES
	ADDWF	TEMP2,F

	MOVLW	LOW P_WAVE		;ADD THE LOW BYTES
	ADDWF	TEMP1,F

	BTFSC	_C			;WAS A CARRY PRODUCED?
	INCF	TEMP2,F			;YES THEN ADD 1 TO THE HIGH RESULT

	MOVLW	HIGH ECG_LOAD
	MOVWF	PCLATH

	CALL	ECG_LOAD		;GET THE BYTE FROM THE TABLE
	MOVWF	TEMP4			;STORE THE RESULT (HIGH)			
	MOVWF	ECG_HIGH		;SAVE A COPY

	INCF	TEMP1,F			;PTR++ (INC THE LOW BYTE)
	BTFSC	_Z			;ROLL OVER?
	INCF	TEMP2,F			;YES THEN ADD 1 TO THE HIGH BYTE

	CALL	ECG_LOAD		;GET THE BYTE FROM THE TABLE
	MOVWF	TEMP3			;STORE THE RESULT (LOW)
	MOVWF	ECG_LOW			;SAVE A COPY

	PAGE_0				;MAKE SURE WE ARE IN PAGE 0 BEFORE
					;WE EXIT
	MOVF	ECG_COUNT_LOW,W		;LOW BYTE = 0?
	BTFSC	_Z
	DECF	ECG_COUNT_HIGH,F	;DEC THE HIGH BYTE
	DECF	ECG_COUNT_LOW,F		;DEC THE LOW BYTE
	
	MOVF	ECG_COUNT_LOW,W		;ARE BOTH BYTES = 0?
	IORWF	ECG_COUNT_HIGH,W

	BTFSS	_Z			;COUNT = 0?
	GOTO	INC_P_INDX		;NO THEN INCREMENT THE INDX

	MOVLW	STATE_P_HOLD		;GO TO NEXT STATE
	MOVWF	ECG_STATE
	GOTO	P_EXIT
	
INC_P_INDX
	INCF	ECG_INDX_LOW,F		;ADD 1 TO THE ARRAY INDEX
	BTFSC	_Z			;DID WE ROLLOVER?
	INCF	ECG_INDX_HIGH,F		;YES THEN ADD 1 TO THE HIGH BYTE
P_EXIT	
	RETLW	0

;******************************************************************************
; OUTPUT THE QRS WAVEFORM DATA
;
; ON ENTRY: EXPECTS NONE
; ON EXIT: RETURNS NONE
;******************************************************************************
GET_QRS_WAVE
	BCF	_C			;INDEX *=2
	RLF	ECG_INDX_LOW,W
	MOVWF 	TEMP1

	RLF	ECG_INDX_HIGH,W
	MOVWF	TEMP2

	BTFSS	ECG_V_PACE_FLAG		;WAS THIS A PACED ECG?
	GOTO	V_INTRIN_OUT		;NO THEN NORMAL INTRINSIC OUT
	
	MOVLW	HIGH	V_PACE_TABLE	;ELSE USE THE V PACING WAVEFORM
	ADDWF	TEMP2,F

	MOVLW	LOW	V_PACE_TABLE
	ADDWF	TEMP1,F
	GOTO	TEST_QRS_C


V_INTRIN_OUT
	MOVLW	HIGH QRS_WAVE		;ADD THE LOW BYTES
	ADDWF	TEMP2,F

	MOVLW	LOW QRS_WAVE		;ADD THE LOW BYTES
	ADDWF	TEMP1,F

TEST_QRS_C
	BTFSC	_C			;WAS A CARRY PRODUCED?
	INCF	TEMP2,F			;YES THEN ADD 1 TO THE HIGH RESULT

	MOVLW	HIGH ECG_LOAD
	MOVWF	PCLATH

	CALL	ECG_LOAD		;GET THE BYTE FROM THE TABLE
	MOVWF	TEMP4			;STORE THE RESULT (HIGH)			
	MOVWF	ECG_HIGH		;SAVE A COPY

	INCF	TEMP1,F			;PTR++ (INC THE LOW BYTE)
	BTFSC	_Z			;ROLL OVER?
	INCF	TEMP2,F			;YES THEN ADD 1 TO THE HIGH BYTE

	CALL	ECG_LOAD		;GET THE BYTE FROM THE TABLE
	MOVWF	TEMP3			;STORE THE RESULT (LOW)
	MOVWF	ECG_LOW			;SAVE A COPY

	PAGE_0

	MOVF	ECG_COUNT_LOW,W		;LOW WORD COUNT = 0?
	BTFSC	_Z
	DECF	ECG_COUNT_HIGH,F	;YES THEN DEC THE HIGH COUNT
	DECF	ECG_COUNT_LOW,F		;DEC THE LOW WORD COUNT

	MOVF	ECG_COUNT_LOW,W		;ARE BOTH BYTES = 0?
	IORWF	ECG_COUNT_HIGH,W

	BTFSS	_Z			;COUNT = 0?
	GOTO	INC_QRS_INDX		;NO THEN INCREMENT THE INDX

	MOVLW	STATE_QRS_HOLD
	MOVWF	ECG_STATE		;YES THEN NEXT STATE (QRS HOLD)


	MOVLW	QRS_TO_T_DELAY		;SETUP THE DELAY BEFORE THE T WAVE

	BTFSC	ECG_V_PACE_FLAG
	MOVLW	d'1'
	MOVWF	QRS_TO_T_TIMER

	BCF	ECG_V_PACE_FLAG

	GOTO	QRS_EXIT
	
INC_QRS_INDX
	INCF	ECG_INDX_LOW,F		;ADD 1 TO THE ARRAY INDEX
	BTFSC	_Z			;DID WE ROLLOVER?
	INCF	ECG_INDX_HIGH,F		;YES THEN ADD 1 TO THE HIGH BYTE
QRS_EXIT	
	RETLW	0

;******************************************************************************
; OUTPUT THE T WAVE DATA
;
; ON ENTRY: EXPECTS NONE
; ON EXIT: RETURNS NONE
;******************************************************************************
GET_T_WAVE
	BCF	_C			;INDEX *=2
	RLF	ECG_INDX_LOW,W
	MOVWF 	TEMP1

	RLF	ECG_INDX_HIGH,W
	MOVWF	TEMP2

	MOVLW	HIGH T_WAVE		;ADD THE LOW BYTES
	ADDWF	TEMP2,F

	MOVLW	LOW T_WAVE		;ADD THE LOW BYTES
	ADDWF	TEMP1,F

	BTFSC	_C			;WAS A CARRY PRODUCED?
	INCF	TEMP2,F			;YES THEN ADD 1 TO THE HIGH RESULT

	MOVLW	HIGH ECG_LOAD
	MOVWF	PCLATH

	CALL	ECG_LOAD		;GET THE BYTE FROM THE TABLE
	MOVWF	TEMP4			;STORE THE RESULT (HIGH)			
	MOVWF	ECG_HIGH		;SAVE A COPY

	INCF	TEMP1,F			;PTR++ (INC THE LOW BYTE)
	BTFSC	_Z			;ROLL OVER?
	INCF	TEMP2,F			;YES THEN ADD 1 TO THE HIGH BYTE

	CALL	ECG_LOAD		;GET THE BYTE FROM THE TABLE
	MOVWF	TEMP3			;STORE THE RESULT (LOW)
	MOVWF	ECG_LOW			;SAVE A COPY
	PAGE_0

	MOVF	ECG_COUNT_LOW,W		;LOW WORD COUNT = 0?
	BTFSC	_Z
	DECF	ECG_COUNT_HIGH,F	;YES THEN BORROW 1 FROM THE HIGH BYTE
	DECF	ECG_COUNT_LOW,F		;DEC THE LOW WORD COUNT

	MOVF	ECG_COUNT_LOW,W		;ARE BOTH BYTES = 0?
	IORWF	ECG_COUNT_HIGH,W

	BTFSS	_Z			;COUNT = 0?
	GOTO	INC_T_INDX		;NO THEN INCREMENT THE INDX

	BSF	ECG_DONE		;ELSE SIGNAL WE ARE DONE

	GOTO	T_EXIT			;AND EXIT
	
INC_T_INDX
	INCF	ECG_INDX_LOW,F		;ADD 1 TO THE ARRAY INDEX
	BTFSC	_Z			;DID WE ROLLOVER?
	INCF	ECG_INDX_HIGH,F		;YES THEN ADD 1 TO THE HIGH BYTE
T_EXIT	
	RETLW	0

;******************************************************************************
GET_PVC_WAVE
;ON ENTRY: EXPECTS NONE
;ON EXIT: RETURNS NONE
;RETURNS THE PVC DATA WORD IN TEMP3, AND TEMP4 (LOW,HIGH)
;******************************************************************************
#define	ECG_STEP	d'27'

	MOVF	PVC_COUNT,W	;IS THE COUNT = 75?
	SUBLW	d'73'		;K-W, 73-x
	BTFSC	_C		;RESULT POSITIVE?
	GOTO	ADD_STEP	;YES THEN ADD STEP
	
	MOVLW	ECG_STEP
	SUBWF	PVC_RAMP_LOW,F
	BTFSS	_C
	DECF	PVC_RAMP_HIGH,F
	GOTO	LOAD_ECG_DATA

ADD_STEP
	MOVLW	ECG_STEP
	ADDWF	PVC_RAMP_LOW,F
	BTFSC	_C
	INCF	PVC_RAMP_HIGH,F

LOAD_ECG_DATA
	MOVF	PVC_RAMP_LOW,W
	MOVWF	TEMP3
	MOVF	PVC_RAMP_HIGH,W
	MOVWF	TEMP4

;	MOVF	PVC_COUNT,W
	SUBLW	d'149'
	BTFSS	_Z
	RETLW	0

	BCF	PVC_RQST
	CLRF	ECG_COUNT_LOW
	CLRF	ECG_COUNT_HIGH
	CLRF	ECG_STATE

	MOVLW	0x80
	MOVWF	TEMP4
	CLRF	TEMP3

	RETLW	0

;******************************************************************************
READ_LVS_DELAY
; READ THE LVS DELAY HEX SWITCH AND STORE THE VALUE IN THE VARIABLE LVS_DELAY
; ON ENTRY: EXPECTS NONE
; ON EXIT: RETURNS NONE
;******************************************************************************
	MOVF	PORTA,W		;READ THE LVS DELAY SWITCH
	ANDLW	0x0F		;MASK OFF THE UPPER 4 BITS
	CALL	LVS_TABLE	;GET THE DELAY VALUE FROM THE TABLE
	MOVWF	LVS_DELAY	;STORE THE DELAY VALUE

	SUBLW	0x01		;IS THE DELAY = 1?
	BTFSS	_Z		
	GOTO	SHIFT_LVS_DELAY	;NO THEN SKIP THIS

	MOVLW	0x01		;YES THEN USE A DELAY OF 1
	MOVWF	LVS_IN_INH_DELAY
	RETURN

SHIFT_LVS_DELAY
	RRF	LVS_DELAY,W	;1/2 OF THE LEFT VENTRICLE SENSE 
				;DELAY IS USED FOR THE LVS IN THE 
				;INHIBIT WINDOW PVC

	MOVWF	LVS_IN_INH_DELAY
	RETURN			;AND EXIT


;************************************************************************
CLEAR_A_TIMER
	CLRF	A_INTERVAL_LOW
	CLRF	A_INTERVAL_HIGH
	RETURN

;************************************************************************
CLEAR_V_TIMER
	CLRF	V_INTERVAL_LOW
	CLRF	V_INTERVAL_HIGH
	RETURN

;************************************************************************
; LED ON ROUTINES
;************************************************************************
RA_INTRIN_LED_ON
	MOVLW	LED_TIME		;LOAD THE TIME ON TIMER
	MOVWF	RA_INTRIN_LED_TIMER	

	BSF	RA_INTRIN_LED		;TURN ON THE LED
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

	RETURN
;************************************************************************
RA_PACE_LED_ON
	MOVLW	LED_TIME		;LOAD THE TIME ON TIMER
	MOVWF	RA_PACE_LED_TIMER	

	BSF	RA_PACE_LED		;TURN ON THE LED
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

	RETURN

;************************************************************************
RV_INTRIN_LED_ON
	MOVLW	LED_TIME		;LOAD THE TIME ON TIMER
	MOVWF	RV_INTRIN_LED_TIMER	

	BSF	RV_INTRIN_LED		;TURN ON THE LED
	MOVF	PORTB_DATA,W
	MOVWF	PORTB

	RETURN

;************************************************************************
RV_PACE_LED_ON
	MOVLW	LED_TIME		;LOAD THE TIME ON TIMER
	MOVWF	RV_PACE_LED_TIMER	

	BSF	RV_PACE_LED		;TURN ON THE LED
	MOVF	PORTB_DATA,W
	MOVWF	PORTB

	RETURN

;************************************************************************
LV_LED_ON
	MOVLW	LED_TIME		;LOAD THE TIME ON TIMER
	MOVWF	LV_INTRIN_LED_TIMER	

	BSF	LV_INTRIN_LED		;TURN ON THE LED
	MOVF	PORTB_DATA,W
	MOVWF	PORTB

	RETURN

;************************************************************************
LVP_LED_ON
	MOVLW	LED_TIME		;LOAD THE TIME ON TIMER
	MOVWF	LVP_LED_TIMER	

	BSF	LVP_LED			;TURN ON THE LED
	MOVF	PORTC_DATA,W
	MOVWF	PORTC

	RETURN


;************************************************************************
LVP_OUT_LED_ON
	MOVLW	LED_TIME		;LOAD THE TIME ON TIMER
	MOVWF	LVP_OUT_LED_TIMER	

	BSF	LVP_OUT_LED		;TURN ON THE LED
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

	BSF	V_CAP_EVENT

	RETURN


;************************************************************************
; LED OFF ROUTINES
;************************************************************************
RA_INTRIN_LED_OFF

	BCF	RA_INTRIN_LED		;TURN OFF THE LED
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

	RETURN
;************************************************************************
RA_PACE_LED_OFF

	BCF	RA_PACE_LED		;TURN OFF THE LED
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

	RETURN

;************************************************************************
RV_INTRIN_LED_OFF

	BCF	RV_INTRIN_LED		;TURN OFF THE LED
	MOVF	PORTB_DATA,W
	MOVWF	PORTB

	RETURN

;************************************************************************
RV_PACE_LED_OFF

	BCF	RV_PACE_LED		;TURN OFF THE LED
	MOVF	PORTB_DATA,W
	MOVWF	PORTB

	RETURN

;************************************************************************
LV_LED_OFF

	BCF	LV_INTRIN_LED		;TURN OFF THE LED
	MOVF	PORTB_DATA,W
	MOVWF	PORTB

	RETURN

;************************************************************************
LVP_LED_OFF

	BCF	LVP_LED			;TURN OFF THE LED
	MOVF	PORTC_DATA,W
	MOVWF	PORTC

	RETURN


;************************************************************************
LVP_OUT_LED_OFF

	BCF	LVP_OUT_LED		;TURN OFF THE LED
	MOVF	PORTD_DATA,W
	MOVWF	PORTD

	RETURN

;************************************************************************
; ROUTINES CALLED FROM THE MAIN LOOP THRU THE MAIN TABLE
;************************************************************************


;*****************************************************************
DO_LV_PVC
;*****************************************************************
;SET THE ECG SIGNAL TO PVC OUTPUT
	BCF	LV_PVC_EVENT		;CLEAR THE PVC FLAG

	BTFSC	PVC_RQST		;IS THERE AN EVENT ALREADY PENDING?
	GOTO	MAIN			;YES THEN EXIT

	BSF	PVC_RQST		;SIGNAL THAT A PVC IS PENDING
	CLRF	PVC_BEAT_COUNT

	CLRF	PVC_TYPE		;RESET THE PVC TYPE TO ZERO
	GOTO	MAIN
	
;*******************************************************************
DO_VA_DELAY
;*******************************************************************
	BCF	INTCON,GIE		;DISABLE INTERRUPTS

	BTFSS	GOT_A_EVENT		;DID WE GET AN A EVENT IN THIS CYCLE
	GOTO	USE_V_TIMER		;NO THEN USE THE V TIMER VALUES

	MOVF	A_INTERVAL_LOW,W	;ELSE USE THE A TIMER VALUE
	MOVWF	TEMP3

	MOVF	A_INTERVAL_HIGH,W
	MOVWF	TEMP4

	GOTO	CALC_VA_DELAY

USE_V_TIMER
	BCF	GOT_A_EVENT
	MOVF	V_INTERVAL_LOW,W
	MOVWF	TEMP3

	MOVF	V_INTERVAL_HIGH,W
	MOVWF	TEMP4

;-----------------------------------------------------------------------
; CALCULATE THE VA DELAY AND LOAD THE DELAY TIMER
;-----------------------------------------------------------------------
CALC_VA_DELAY
	MOVLW	LOW RATE_MS		;PACING RATE - A OR V INTERVAL
	MOVWF	DELAY_LOW

	MOVLW	HIGH RATE_MS
	MOVWF	DELAY_HIGH

	MOVF	TEMP4,W			;SUBTRACT THE HIGH BYTES
	SUBWF	DELAY_HIGH,F
	
	MOVF	TEMP3,W			;SUBTRACT THE LOW BYTES
	SUBWF	DELAY_LOW,F

	BTFSS	_C			;RESULT POSITIVE?
	DECF	DELAY_HIGH,F		;NO THEN SUBTRACT THE BORROW

	BSF	DELAY_FLAG
;-----------------------------------------------------------------------

;-----------------------------------------------------------------------
;WAS THE PVC SWITCH PUSHBUTTON PRESSED?
;-----------------------------------------------------------------------
	BTFSS	PVC_RQST		;IS THERE A PVC EVENT PENDING?
	GOTO	DO_NORM			;NO THEN SKIP THIS

	MOVF	PVC_BEAT_COUNT,W	;GET THE BEAT COUNT
	BTFSC	_Z			;IS THE BEAT COUNT = 0?
	GOTO	DO_PVC			;YES THEN DO THE NEXT PVC

	DECF	PVC_BEAT_COUNT,F	;NO THEN DEC THE COUNT
	GOTO	DO_NORM			;AND DO A NORMAL BEAT WITH NO PVC

DO_PVC
	MOVLW	d'10'			;LOAD THE BEAT COUNT
	MOVWF	PVC_BEAT_COUNT

;----------------------------------------------------------------------
; DETERMINE WHICH PVC TYPE AND SET THE APPROPRIATE FLAGS
;----------------------------------------------------------------------

;----------------------------------------------------------------------
; CHECK FOR A PVC TYPE OF V INTRINSIC WITH NO A INRINSIC
; PVC_TYPE = 0
;----------------------------------------------------------------------
	MOVF	PVC_TYPE,W		;IS THE PVC TYPE = PVC_V_NO_A
	SUBLW	PVC_V_NO_A
	BTFSS	_Z
	GOTO	TEST_LV_IN_A
	
	BSF	PVC_V_NO_A_FLAG		;SIGNAL TO THE V_CAP ROUTINE
					;THAT THIS WAS A PVC

	CLRF	ACTION_FLAGS		;CLEAR ALL EVENTS

	MOVLW	DO_V_CAP_INDX		;DO A V CAPTURE ON TIMEOUT
	MOVWF	TIMEOUT_INDX

	INCF	PVC_TYPE,F		;ADD 1 TO THE PVC TYPE
	GOTO	VA_EXIT			;AND EXIT

;------------------------------------------------------------------
; CHECK FOR A PVC TYPE OF LV (LVS OUTPUT) DURING AV DELAY
; PVC_TYPE = 1
;------------------------------------------------------------------
TEST_LV_IN_A
	MOVF	PVC_TYPE,W		;PVC TYPE = LV IN THE AV DELAY?
	SUBLW	PVC_LV_IN_AV
	BTFSS	_Z
	GOTO	TEST_SHORT_AV		;NO THEN CHECK NEXT TYPE

	BSF	LV_IN_A_FLAG		;YES THEN SET THE LV_IN_A FLAG

	GOTO	PVC_EXIT		;AND EXIT

;------------------------------------------------------------------
; CHECK FOR A PVC TYPE OF SHORT AV DELAY < 40ms
; PVC_TYPE = 2
;------------------------------------------------------------------
TEST_SHORT_AV
	MOVF	PVC_TYPE,W		;PVC TYPE = SHORT AV DELAY?
	SUBLW	PVC_SHORT_AV
	BTFSS	_Z
	GOTO	TEST_LVS_IN_INH		;NO THEN CHECK THE NEXT TYPE

	BSF	SHORT_AV_FLAG		;YES THEN SET THE SHORT AV FLAG

	GOTO	PVC_EXIT

;------------------------------------------------------------------
; CHECK FOR A PVC TYPE OF LV EVENT WITHIN LVS INHIBIT WINDOW
; PVC_TYPE = 3
;------------------------------------------------------------------
TEST_LVS_IN_INH
	MOVF	PVC_TYPE,W
	SUBLW	PVC_LV_IN_LVS
	BTFSS	_Z
	GOTO	TEST_NO_SURFACE_A

	BSF	LV_IN_LVS_INH_FLAG

	GOTO	PVC_EXIT

;------------------------------------------------------------------
; CHECK FOR A PVC TYPE OF NO SURFACE ECG A EVENT
; PVC_TYPE = 4
;------------------------------------------------------------------
TEST_NO_SURFACE_A
	MOVF	PVC_TYPE,W
	SUBLW	PVC_SURFACE
	BTFSS	_Z
	GOTO	TEST_PVC_Z

	BSF	PVC_SUR_FLAG
	GOTO	PVC_EXIT

;------------------------------------------------------------------
; CHECK FOR A PVC TYPE OF SHIFT THE IMPEDANCE WAVEFORM
; PVC_TYPE = 5
;------------------------------------------------------------------
TEST_PVC_Z
	CALL	TRIG_PORTB		;DEBUG ONLY	

	MOVF	PVC_TYPE,W		;GET THE PVC TYPE
	SUBLW	PVC_Z_SHIFT		;IS IT A Z SHIFT TYPE
	BTFSC	_Z
	BSF	PVC_Z_FLAG		;YES THEN SET THE Z SHIFT FLAG


CLEAR_PVC_TYPE	
	CLRF	PVC_TYPE		;YES THEN CLEAR THE TYPE BACK TO 0
	BCF	PVC_RQST	
	GOTO	DO_NORM			;EXIT

PVC_EXIT
	INCF	PVC_TYPE,F		;ADD 1 TO THE PVC TYPE


;-----------------------------------------------------------------------

DO_NORM

	CLRF	ACTION_FLAGS		;CLEAR OLD ACTIONS
	BSF	A_ACTION_FLAG		;ALLOW A CAPTURE
	BSF	V_ACTION_FLAG		;ALLOW V CAPTURE
	BSF	LV_PVC_ACTION_FLAG	;ALLOW PVCs

	CLRF	LAST_EVENT_FLAGS	;CLEAR OLD STATUS INFO

	MOVLW	DO_A_CAP_INDX		;DO A CAP ON A EVENT
	MOVWF	A_CAP_INDX

	MOVLW	DO_V_CAP_INDX		;DO V CAPTURE ON V EVENT
	MOVWF	V_CAP_INDX

	MOVLW	DO_A_INTRIN_INDX	;DO A INTRINSIC ON TIMEOUT
	MOVWF	TIMEOUT_INDX

VA_EXIT
	BCF	GOT_A_EVENT

	BSF	INTCON,GIE
	GOTO	MAIN

;*********************************************************************
DO_A_INT_POWERUP
;TWO CYCLES OF A INTRINISC AND V INTRINSIC ARE RUN AFTER POWER UP
;SO THAT ALL OF THE TIMERS WILL HAVE "GOOD" VALUES FOR CALCULATING
;CORRECT A-V AND V-A DELAYS.
;*********************************************************************
	BCF	INTCON,GIE		;INTERRUPTS OFF	

	CALL	RA_INTRIN_LED_ON	;TURN ON THE RA INTRIN LED

	CALL	CLEAR_A_TIMER

	CLRF	LAST_EVENT_FLAGS	;SIGNAL LAST EVENT
	BSF	A_INT_FLAG

	BSF	GOT_A_EVENT		;SIGNAL WE GOT AN A EVENT
					;FOR THIS CYCLE

;SETUP TO OUTPUT THE A CENELEC
	MOVLW	A_DATA_LENGTH		;LOAD THE A IEGM DATA LENGTH
	MOVWF	A_COUNT			

	CLRF	CEN_INDX		;CLEAR THE A POINTER

	BSF	A_FLAG

;SETUP TO OUTPUT THE START OF THE ECG WAVEFORM (P WAVE)

	CALL	START_P_WAVE

	CLRF	ACTION_FLAGS		;DON'T ALLOW ANY EVENTS FOR NOW

;TIMEOUT DELAY IS THE AV DELAY
	MOVF	AV_DELAY_LOW,W
	MOVWF	DELAY_LOW
	
	MOVF	AV_DELAY_HIGH,W
	MOVWF	DELAY_HIGH

	INCF	POWER_UP_COUNTER,F	;ADD 1 TO THE POWER UP COUNTER

;DO V INTRINSIC POWERUP ON TIMEOUT
	MOVLW	DO_V_INT_POWERUP_INDX
	MOVWF	TIMEOUT_INDX

	BSF	DELAY_FLAG		;ENABLE THE DELAY
	
	BSF	INTCON,GIE		;ENABLE INTERRUPTS
	GOTO	MAIN

;*********************************************************************
DO_V_INT_POWERUP
;TWO CYCLES OF A INTRINISC AND V INTRINSIC ARE RUN AFTER POWER UP
;SO THAT ALL OF THE TIMERS WILL HAVE "GOOD" VALUES FOR CALCULATING
;CORRECT A-V AND V-A DELAYS.
;*********************************************************************

	BCF	INTCON,GIE		;INTERRUPTS OFF	

	CALL	RV_INTRIN_LED_ON	;TURN ON THE RV INTRIN LED

	CALL	CLEAR_V_TIMER		;CLEAR THE V INTERVAL TIMER

	CLRF	LAST_EVENT_FLAGS	;SET LAST EVENT
	BSF	V_INT_FLAG

	CALL	START_V_CEN_AND_LVS_DELAY

;SETUP TO OUTPUT THE START OF THE ECG WAVEFORM (QRS WAVE)
	BCF	ECG_V_PACE_FLAG
	MOVLW	STATE_QRS		;STATE = QRS WAVE
	MOVWF	ECG_STATE

	MOVLW	LOW QRS_WAVE_LEN	;GET THE QRS WAVE LENGTH
	MOVWF	ECG_COUNT_LOW
	MOVLW	HIGH QRS_WAVE_LEN
	MOVWF	ECG_COUNT_HIGH

	CLRF	ECG_INDX_LOW
	CLRF	ECG_INDX_HIGH

	BSF	ECG_FLAG

	CLRF	ACTION_FLAGS		;DISABLE ALL ACTIONS

	MOVF	POWER_UP_COUNTER,W	;IS THIS THE 2ND TIME THROUGH?
	SUBLW	0x02
	BTFSC	_Z			
	GOTO	V_INTRIN_EXIT		;YES THEN TO A NORMAL RUN

;USE VA DELAY FOR THE TIMEOUT DELAY
	MOVLW	LOW VA_DELAY_INIT
	MOVWF	DELAY_LOW

	MOVLW	HIGH VA_DELAY_INIT
	MOVWF	DELAY_HIGH

	MOVLW	DO_A_INT_POWERUP_INDX	;DO ANOTHER A INTRINSIC
	MOVWF	TIMEOUT_INDX		;ON TIMEOUT
	

EXIT_A_POWERUP
	BSF	DELAY_FLAG
	BSF	INTCON,GIE

	GOTO	MAIN

;************************************************************************
DO_A_CAP
; WE HAVE AN A CAPTURE EVENT
;************************************************************************
;STOP ALL INTERRUPTS
	BCF	INTCON,GIE		;DISABLE INTERRUPTS

	CALL	RA_PACE_LED_ON		;TURN ON THE LED

	CALL	CLEAR_A_TIMER
	
	BCF	A_CAP_EVENT		;CLEAR THE A EVENT FLAG

	CLRF	LAST_EVENT_FLAGS	;AND SIGNAL AN A CAPTURE
	BSF	A_CAP_FLAG

	BSF	GOT_A_EVENT		;SIGNAL WE GOT AN A EVENT FOR THIS
					;CYCLE

;-------------------------------------------------------------------
;DID WE GET A PVC TYPE OF LV EVENT WITHIN THE AV DELAY?
;-------------------------------------------------------------------
TEST_A_CAP_LV_IN_A
	BTFSS	LV_IN_A_FLAG		;DID WE GET LV IN AV DELAY
	GOTO	A_CAP_ECG		;NO THEN SKIP THIS

	MOVLW	d'100'			;USE A DELAY OF 100ms
	MOVWF	RV_EVENT_TO_LVS_DELAY_TIMER

;-------------------------------------------------------------------

;------------------------------------------------------------
;SETUP TO OUTPUT THE START OF THE ECG WAVEFORM (P WAVE)
;------------------------------------------------------------
A_CAP_ECG
	CALL	START_P_WAVE

;CLEAR ALL ACTION FLAGS
	CLRF	ACTION_FLAGS		;DISABLE ALL EVENTS
	BSF	V_ACTION_FLAG		;ALLOW V EVENTS
	BSF	LV_PVC_ACTION_FLAG

;--------------------------------------------------------------
;DO V CAP ON THE V CAPTURE EVENT
;--------------------------------------------------------------
	MOVLW	DO_V_CAP_INDX		;DO AN V CAPTURE ON A V EVENT
	MOVWF	V_CAP_INDX

;--------------------------------------------------------------
; DID WE GET A PVC TYPE OF SHORT AV DELAY
;--------------------------------------------------------------
	BTFSS	SHORT_AV_FLAG		;DO WE HAVE A PVC TYPE OF SHORT AV DELAY?
	GOTO	A_CAP_NORM_AV		;NO THEN DO A NORMAL AV DELAY

	BCF	SHORT_AV_FLAG		;ELSE CLEAR THE FLAG
	MOVLW	SHORT_AV_DELAY		;LOAD A SHORT AV DELAY
	MOVWF	DELAY_LOW		;LOW VALUE
	CLRF	DELAY_HIGH		;HIGH VALUE = 0
	GOTO	A_CAP_V_IN

;TIMEOUT DELAY IS THE AV DELAY		
A_CAP_NORM_AV
	MOVF	AV_DELAY_LOW,W
	MOVWF	DELAY_LOW
	
	MOVF	AV_DELAY_HIGH,W
	MOVWF	DELAY_HIGH

;------------------------------------------------------------------
; DID WE GET A PVC TYPE FOR A PVC SURFACE ECG
;------------------------------------------------------------------
	BTFSS	PVC_SUR_FLAG		;PVC (SURFACE ECG)
	GOTO	A_CAP_V_IN		;NO THEN DO NORMAL

	BCF	PVC_SUR_FLAG		;CLEAR THIS FLAG
	BSF	PVC_V_NO_A_FLAG		;AND SIGNAL VCAP THIS IS A PVC
	MOVLW	DO_V_CAP_INDX		;YES THEN DO A V CAP EVENT
	MOVWF	TIMEOUT_INDX		;ON TIMEOUT

	CLRF	ACTION_FLAGS		;ALLOW NO ACTIONS

	GOTO	A_CAP_SET_DELAY

;DO V INTRINSIC ON TIMEOUT
A_CAP_V_IN
	MOVLW	DO_V_INTRIN_INDX
	MOVWF	TIMEOUT_INDX

A_CAP_SET_DELAY
	BSF	DELAY_FLAG
	
;ENABLE ALL INTERRUPTS
	BSF	INTCON,GIE
	GOTO 	MAIN

;**********************************************************
DO_V_CAP
;**********************************************************
;STOP ALL INTERRUPTS
	BCF	INTCON,GIE

	CALL	RV_PACE_LED_ON		;TURN ON THE LED

	CALL	CLEAR_V_TIMER		;CLEAR THE V INTERVAL TIMER

	BCF	V_CAP_EVENT		;CLEAR THE EVENT FLAG

;----------------------------------------------------------------
;INITIALIZE THE LVP TIMER
;----------------------------------------------------------------
	MOVLW	LVP_WIN_INIT
	MOVWF	LVP_WIN_TIMER
;----------------------------------------------------------------

	CLRF	LAST_EVENT_FLAGS
;--------------------------------------------------------------------

;--------------------------------------------------------------------
;CHECK FOR A EARLY LV EVENT (LV_IN_A FLAG), WHERE THE LV SIGNAL
;WAS ALREADY OUTPUT.
;--------------------------------------------------------------------
	BTFSS	LV_IN_A_FLAG		;LV IN AV DELAY?
	GOTO	TEST_VCAP_PVC		;NO THEN NEXT TEST

	BCF	LV_IN_A_FLAG		;YES THEN CLEAR THE FLAG
	GOTO	V_CAP_OK		;AND DO A NORMAL V CAPTURE
;--------------------------------------------------------------------
; CHECK FOR A PVC EVENT OF V WITH NO A EVENT
;--------------------------------------------------------------------
TEST_VCAP_PVC
	BTFSS	PVC_V_NO_A_FLAG			;WAS THIS FROM A PVC?
	GOTO	VCAP_TEST_LV			;NO THEN DID WE HAVE A PVC
						;TYPE OF LV IN LVS INH.?

	BSF	PVC_FLAG			;YES THEN SIGNAL PVC

	CALL	START_V_CEN_AND_LVS_DELAY 	;OUTPUT THE V CENELAC
						;AND LVS

	BCF	PVC_V_NO_A_FLAG			;CLEAR THE PVC FLAG

	GOTO	V_CAP_START_ECG			;AND START THE ECG OUTPUT

;----------------------------------------------------------------------
; CHECK FOR A PVC EVENT TYPE OF LV EVENT WITHIN THE LVS INHIBIT WINDOW
;----------------------------------------------------------------------

VCAP_TEST_LV
	BTFSS	LV_IN_LVS_INH_FLAG		;DID WE HAVE A LV IN INH?
	GOTO	V_CAP_OK			;NO THEN DO NORM

	BSF	PVC_FLAG			;SIGNAL PVC

	MOVF	LVS_IN_INH_DELAY,W
	MOVWF	RV_EVENT_TO_LVS_DELAY_TIMER

	BCF	LV_IN_LVS_INH_FLAG		;CLEAR THE PVC FLAG
	GOTO	V_CAP_START_ECG			;AND EXIT

;----------------------------------------------------------------------
; NO PVC EVENTS, SO DO A NORMAL V CAPTURE
;----------------------------------------------------------------------
V_CAP_OK
	BSF	V_CAP_FLAG			;SIGNAL LAST EVENT A V CAP.

	CALL	START_LVS_DELAY			;START LVS DELAY TIMER
;------------------------------------------------------------------
;SETUP TO OUTPUT THE START OF THE ECG WAVEFORM (PACED WAVE)
;-----------------------------------------------------------------
V_CAP_START_ECG
	CALL	START_Z			;START THE IMPEDANCE OUT

	BSF	ECG_V_PACE_FLAG
	MOVLW	STATE_QRS		;STATE = V_PACE
	MOVWF	ECG_STATE

	MOVLW	LOW V_PACE_LEN		;GET THE V PACE LENGTH
	MOVWF	ECG_COUNT_LOW
	MOVLW	HIGH V_PACE_LEN
	MOVWF	ECG_COUNT_HIGH

	CLRF	ECG_INDX_LOW
	CLRF	ECG_INDX_HIGH

	BSF	ECG_FLAG

	CLRF	ACTION_FLAGS			;DISABLE ALL ACTIONS
	BSF	LV_PVC_ACTION_FLAG

;TIMEOUT IS THE V REFRACTORY
	MOVLW	LOW V_REF
	MOVWF	DELAY_LOW
	
	MOVLW	HIGH V_REF
	MOVWF	DELAY_HIGH

;DO VA DELAY ON TIMEOUT
	MOVLW	DO_VA_DELAY_INDX
	MOVWF	TIMEOUT_INDX

	BSF	DELAY_FLAG

	BSF	INTCON,GIE
	GOTO	MAIN

;*********************************************************************
DO_A_INTRIN
;THIS IS THE A INTRINSIC ROUTINE. THIS ROUTINE IS RUN AFTER A TIMEOUT
;FOR THE V-A DELAY.
;*********************************************************************
	BCF	INTCON,GIE		;INTERRUPTS OFF	
	
	CALL	RA_INTRIN_LED_ON	;TURN ON THE RA INTRIN LED

	CALL	CLEAR_A_TIMER

	CLRF	LAST_EVENT_FLAGS	;SET LAST EVENT
	BSF	A_INT_FLAG

	BSF	GOT_A_EVENT		;SIGNAL WE GOT AN A EVENT FOR
					;THIS CYCLE

;-------------------------------------------------------------------
;DID WE GET A PVC TYPE OF LV EVENT WITHIN THE AV DELAY?
;-------------------------------------------------------------------
	BTFSS	LV_IN_A_FLAG		;DID WE GET LV IN AV DELAY
	GOTO	A_IN_CEN		;NO THEN SKIP THIS

	MOVLW	d'100'			;DELAY 100 ms
	MOVWF	RV_EVENT_TO_LVS_DELAY_TIMER

;-------------------------------------------------------------------

;SETUP TO OUTPUT THE A CENELEC
A_IN_CEN
	MOVLW	A_DATA_LENGTH		;LOAD THE A IEGM DATA LENGTH
	MOVWF	A_COUNT			

	CLRF	CEN_INDX		;CLEAR THE A POINTER

	BSF	A_FLAG			;START THE A CENELAC OUTPUT

;SETUP TO OUTPUT THE START OF THE ECG WAVEFORM (P WAVE)

	CALL	START_P_WAVE

	CLRF	ACTION_FLAGS		;DON'T ALLOW ANY EVENTS FOR NOW
	BSF	V_ACTION_FLAG		;ALLOW V EVENTS
	BSF	LV_PVC_ACTION_FLAG

;DO V CAP ON THE V CAPTURE EVENT
	MOVLW	DO_V_CAP_INDX		;DO AN V CAPTURE ON A V EVENT
	MOVWF	V_CAP_INDX

;-------------------------------------------------------------------
; PVC TYPE OF SHORT AV DELAY?
;-------------------------------------------------------------------
	BTFSS	SHORT_AV_FLAG		;PVC FOR SHORT AV DELAY SET?
	GOTO	A_INT_OK		;NO THEN TEST FOR A SURFACE PVC

	BCF	SHORT_AV_FLAG		;YES THEN CLEAR THE FLAG
	MOVLW	d'25'			;AND LOAD AN AV DELAY OF 25ms
	MOVWF	DELAY_LOW
	CLRF	DELAY_HIGH

	GOTO	A_INT_VTOUT


;TIMEOUT DELAY IS THE AV DELAY
A_INT_OK
	MOVF	AV_DELAY_LOW,W
	MOVWF	DELAY_LOW
	
	MOVF	AV_DELAY_HIGH,W
	MOVWF	DELAY_HIGH

;-------------------------------------------------------------------
; PVC TYPE OF SURFACE ECG PVC?
;-------------------------------------------------------------------
	BTFSS	PVC_SUR_FLAG		;SURFACE ECG PVC?
	GOTO	A_INT_VTOUT		;NO THEN LOAD NORMAL ACTION
					

	BCF	PVC_SUR_FLAG		;CLEAR THE FLAG
	BSF	PVC_V_NO_A_FLAG		;SIGNAL DO_V_CAP THIS IS A PVC
	MOVLW	DO_V_CAP_INDX		;DO V CAP AFTER AV DELAY
	MOVWF	TIMEOUT_INDX
	
	CLRF	ACTION_FLAGS		;ALLOW NO EVENT RESPONSES

	GOTO	A_INT_SET_DELAY

;DO V INTRINSIC ON TIMEOUT
A_INT_VTOUT
	MOVLW	DO_V_INTRIN_INDX
	MOVWF	TIMEOUT_INDX

A_INT_SET_DELAY
	BSF	DELAY_FLAG		;ENABLE THE DELAY
	
	BSF	INTCON,GIE		;ENABLE INTERRUPTS
	GOTO	MAIN

;*********************************************************************
DO_V_INTRIN
;*********************************************************************
	BCF	INTCON,GIE		;INTERRUPTS OFF	

	CALL	RV_INTRIN_LED_ON	;TURN ON THE RV INTRIN LED

	CALL	CLEAR_V_TIMER		;CLEAR THE V INTERVAL TIMER

;-------------------------------------------------------------
;INITIALIZE THE LVP TIMER
;-------------------------------------------------------------
	MOVLW	LVP_WIN_INIT
	MOVWF	LVP_WIN_TIMER


	CLRF	LAST_EVENT_FLAGS	;CLEAR LAST EVENT FLAG
	BSF	V_INT_FLAG		;SIGNAL LAST EVENT WAS AN INTRINSIC
					;EVENT
;-------------------------------------------------------------
;TEST FOR A PVC TYPE WHERE THE LV OUTPUT SHOULD BE IN THE LVS 
;INHIBIT WINDOW.
;-------------------------------------------------------------
	BTFSS	LV_IN_LVS_INH_FLAG	;PVC REQUESTED?
	GOTO	V_INTRIN_OK		;NO THEN DO NORMAL

	BCF	LV_IN_LVS_INH_FLAG	;YES THEN CLEAR THE FLAG

;-----------------------------------------
;START THE V IEGM (CENELAC) OUTPUT
;-----------------------------------------
	CALL	START_V_CEN

	MOVF	LVS_IN_INH_DELAY,W
	MOVWF	RV_EVENT_TO_LVS_DELAY_TIMER

	GOTO	V_INTRIN_ECG		;AND START THE ECG OUTPUT

;--------------------------------------------
V_INTRIN_OK
	BTFSS	LV_IN_A_FLAG		;DID WE HAVE A LV EVENT IN THE AV
	GOTO    V_INT_BOTH		;DELAY?	NO THEN OUTPUT BOTH

	BCF	LV_IN_A_FLAG		;YES THEN CLEAR THE FLAG
	CALL	START_V_CEN		;AND OUTPUT THE V CEN. ONLY
	GOTO	V_INTRIN_ECG

V_INT_BOTH
	CALL	START_V_CEN_AND_LVS_DELAY	;OUTPUT BOTH V CEN. AND THE 
						;LEFT VENTRICLE SENSE SIGNAL

;-------------------------------------------------------------
;SETUP TO OUTPUT THE START OF THE ECG WAVEFORM (QRS WAVE)
;-------------------------------------------------------------
V_INTRIN_ECG
	BCF	ECG_V_PACE_FLAG
	MOVLW	STATE_QRS		;STATE = QRS WAVE
	MOVWF	ECG_STATE

	MOVLW	LOW QRS_WAVE_LEN	;GET THE QRS WAVE LENGTH
	MOVWF	ECG_COUNT_LOW
	MOVLW	HIGH QRS_WAVE_LEN
	MOVWF   ECG_COUNT_HIGH

	CLRF	ECG_INDX_LOW
	CLRF	ECG_INDX_HIGH

	BSF	ECG_FLAG

	CLRF	ACTION_FLAGS		;DISABLE ALL ACTIONS
	BSF	LV_PVC_ACTION_FLAG

;******************************************************************
; THIS PART OF THE ROUTINE IS ALSO JUMPED TO BY THE V INTRIN. POWER UP
; ROUTINE.
;******************************************************************
V_INTRIN_EXIT
	CALL	START_Z

;DELAY IS THE V REFRACTORY
	MOVLW	LOW V_REF
	MOVWF	DELAY_LOW
	
	MOVLW	HIGH V_REF
	MOVWF	DELAY_HIGH

;DO VA DELAY ON TIMEOUT
	MOVLW	DO_VA_DELAY_INDX
	MOVWF	TIMEOUT_INDX

	BSF	DELAY_FLAG
	
	BSF	INTCON,GIE
	GOTO	MAIN

;************************************************************************
TRIG_PORTB
; THIS ROUTINE IS USED FOR DEBUG ONLY
;************************************************************************
	BCF	PORTB,0		;MAKE SURE PORT IS LOW
	BSF	PORTB,0		;MAKE LINE HIGH
	BCF	PORTB,0		;AND THEN LOW AGAIN
	RETURN

;******************************************************************************
; DATA (TABLES) FILES ARE HERE
;******************************************************************************
;PATH= C:\Windows\desktop\qv\
;Note: the path should point to where the files are located on your system
	
	include "C:\Windows\desktop\qv\CEN1.ASM"		;A AND V DAC FILE	
	include "C:\windows\desktop\qv\ECG7.ASM"		;ECG DAC FILE
	include "C:\windows\desktop\qv\TWAV1.ASM"		;ECG DAC FILE
	include "C:\windows\desktop\qv\VPAC1.ASM"		;ECG DAC FILE						
	include "C:\windows\desktop\qv\IMP2.ASM"		;IMPEDANCE DAC FILE	


	END


