;*************************************************************
; QUICKVIEW DEFINITIONS FILE
; QVDEFS5.INC
; (c) 2003 David Prutchi and Michael Norris
;
;		
;**************************************************************
; DEFINES
;**************************************************************
#define	SET_IRP0	BCF	STATUS,IRP
#define	SET_IRP1	BSF	STATUS,IRP

#define	_Z		STATUS,Z
#define _C		STATUS,C

#define	T0_200		d'15'
#define	T1_1MS		d'63054'

#define	SSPCON_INIT		b'00100000'	;SPI PORT CONFIG VALUE
						;ENABLE THE PORT, CLOCK PARITY LOW,
						;MASTER MODE, FOSC/4

;***********************************************************
; DATA MEMORY DEFINITIONS START HERE
;***********************************************************
RAM_START	EQU	0x20
W_TEMP		EQU	RAM_START
STATUS_TEMP	EQU	W_TEMP+1
FSR_TEMP	EQU	STATUS_TEMP+1
PCLATH_TEMP	EQU	FSR_TEMP+1

PORTA_DATA	EQU	PCLATH_TEMP+1
PORTB_DATA	EQU	PORTA_DATA+1
PORTC_DATA	EQU	PORTB_DATA+1
PORTD_DATA	EQU	PORTC_DATA+1
PORTE_DATA	EQU	PORTD_DATA+1

OUTPUT_FLAGS	EQU	PORTE_DATA+1		;FLAGS FOR DATA OUTPUT TO D/A CONVERTERS
#define	ECG_FLAG	OUTPUT_FLAGS,0		;ECG WAVEFORM
#define	A_FLAG		OUTPUT_FLAGS,1		;A CENELEC
#define	V_FLAG		OUTPUT_FLAGS,2		;V CENELEC
#define	LVS_FLAG	OUTPUT_FLAGS,3		;LVS CENELEC
#define	Z_FLAG		OUTPUT_FLAGS,4		;Z WAVEFORM
#define	ECG_DONE	OUTPUT_FLAGS,5		;SIGNALS THE OUTPUT ROUTINE TO TURN OFF
						;THE ECG OUTPUT AFTER THE CURRENT WORD IS 
						;WRITTEN TO THE DAC.

Z_COUNT		EQU	OUTPUT_FLAGS+1
A_COUNT		EQU	Z_COUNT+1		
V_COUNT		EQU	A_COUNT+1
LVS_COUNT	EQU	V_COUNT+1

ECG_COUNT_LOW	EQU	LVS_COUNT+1	;WORD COUNT
ECG_COUNT_HIGH	EQU	ECG_COUNT_LOW+1


;NUMBER OF 12 BIT WORDS
#define Z_DATA_LENGTH		d'251'	;WORDS	
#define	A_DATA_LENGTH		d'75'
#define	V_DATA_LENGTH		d'75'
#define	LVS_DATA_LENGTH		d'75'

;INDEX TO EACH OF THE DATA TABLES
ECG_INDX_LOW	EQU	ECG_COUNT_HIGH+1	;INDEX TO A 16 BIT TABLE (ARRAY IN ROM)
ECG_INDX_HIGH	EQU	ECG_INDX_LOW+1

Z_INDX_LOW	EQU	ECG_INDX_HIGH+1		;16 BITS		
Z_INDX_HIGH	EQU	Z_INDX_LOW+1

CEN_INDX	EQU	Z_INDX_HIGH+1
LVS_INDX		EQU	CEN_INDX+1

;SCRATCH PAD REGISTERS USED BY THE INTERRUPT ROUTINES
TEMP1	EQU	LVS_INDX+1
TEMP2	EQU	TEMP1+1
TEMP3	EQU	TEMP2+1
TEMP4	EQU	TEMP3+1

; SCRATCH PAD REGISTERS USED BY SUBROUTINES CALLED BY THE MAIN LOOP
; SHOULD NOT BE USED BY THE INTERRUPT SUBROUTINES
SPAD1	EQU	TEMP4+1
SPAD2	EQU	SPAD1+1
SPAD3	EQU	SPAD2+1
SPAD4	EQU	SPAD3+1

ECG_TIMER	EQU	SPAD4+1
#define	ECG_TIME_INIT	5

LVS_DELAY	EQU	ECG_TIMER+1

PORTB_PREV	EQU	LVS_DELAY+1
PORTB_TEMP	EQU	PORTB_PREV+1

EVENT_FLAGS	EQU	PORTB_TEMP+1
#define	A_CAP_EVENT	EVENT_FLAGS,0
#define	A_NON_EVENT	EVENT_FLAGS,1
#define	V_CAP_EVENT	EVENT_FLAGS,2
#define	V_NON_EVENT	EVENT_FLAGS,3
#define	LVP_CAP_EVENT	EVENT_FLAGS,4
#define LV_PVC_EVENT	EVENT_FLAGS,5
#define	LVP_ARM		EVENT_FLAGS,6
#define LVP_ERROR	EVENT_FLAGS,7


;ENABLE FLAGS FOR EXTERNAL (HARDWARE) EVENTS
;USED ONLY IN THE INTERRUPT ROUTINE
IN_EVENT_ENABLE_FLAGS	EQU	EVENT_FLAGS+1
#define IN_V_ENABLE		IN_EVENT_ENABLE_FLAGS,0
#define	IN_A_ENABLE		IN_EVENT_ENABLE_FLAGS,1
#define	IN_LVP_ENABLE		IN_EVENT_ENABLE_FLAGS,2
#define	IN_LV_PVC_ENABLE	IN_EVENT_ENABLE_FLAGS,3

;ACTION FLAGS ARE USED TO CONTROL WHICH EVENTS THE MAIN LOOP SHOULD
;RESPOND TO.
ACTION_FLAGS		EQU	IN_EVENT_ENABLE_FLAGS+1
#define	A_ACTION_FLAG		ACTION_FLAGS,0
#define V_ACTION_FLAG		ACTION_FLAGS,1
#define LVP_ACTION_FLAG		ACTION_FLAGS,2
#define LV_PVC_ACTION_FLAG	ACTION_FLAGS,3

;POINTERS FOR MAIN LOOP TO THE MAIN TABLE
A_CAP_INDX		EQU	ACTION_FLAGS+1
V_CAP_INDX		EQU	A_CAP_INDX+1
LV_PVC_INDX		EQU	V_CAP_INDX+1
TIMEOUT_INDX		EQU	LV_PVC_INDX+1

;THE DELAY TIMER USED BY THE MAIN LOOP
DELAY_LOW		EQU	TIMEOUT_INDX+1
DELAY_HIGH		EQU	DELAY_LOW+1

SYSTEM_FLAGS		EQU	DELAY_HIGH+1
#define	DELAY_FLAG	SYSTEM_FLAGS,0
#define	POWER_UP_FLAG	SYSTEM_FLAGS,1
#define	GOT_A_EVENT	SYSTEM_FLAGS,2
#define	PVC_RQST	SYSTEM_FLAGS,3


;PREVIOUS EVENT FLAGS
LAST_EVENT_FLAGS	EQU	SYSTEM_FLAGS+1
#define	A_CAP_FLAG	LAST_EVENT_FLAGS,0
#define A_INT_FLAG	LAST_EVENT_FLAGS,1
#define V_CAP_FLAG	LAST_EVENT_FLAGS,2
#define V_INT_FLAG	LAST_EVENT_FLAGS,3
#define PVC_FLAG	LAST_EVENT_FLAGS,4

;DELAY TIMERS FOR THE LEDS. HOLDS THE TIME THAT THE 
;LEDS ARE ON.
RA_INTRIN_LED_TIMER	EQU	LAST_EVENT_FLAGS+1
RA_PACE_LED_TIMER	EQU	RA_INTRIN_LED_TIMER+1
RV_INTRIN_LED_TIMER	EQU	RA_PACE_LED_TIMER+1
RV_PACE_LED_TIMER	EQU	RV_INTRIN_LED_TIMER+1
LV_INTRIN_LED_TIMER	EQU	RV_PACE_LED_TIMER+1
LVP_LED_TIMER		EQU	LV_INTRIN_LED_TIMER+1
LVP_OUT_LED_TIMER	EQU	LVP_LED_TIMER+1

#define	LED_TIME	d'100'


;A TO V DELAY TIME
AV_DELAY_LOW		EQU	LVP_OUT_LED_TIMER+1
AV_DELAY_HIGH		EQU	AV_DELAY_LOW+1
#define	AV_DELAY_INIT	d'200'
#define	SHORT_AV_DELAY	d'30'

;V TO A DELAY TIME
VA_DELAY_LOW		EQU	AV_DELAY_HIGH+1
VA_DELAY_HIGH		EQU	VA_DELAY_LOW+1
#define VA_DELAY_INIT	d'600'
#define	V_REF		d'250'
#define	RATE_MS		d'850'

;THE ECG STATE MACHINE STATE
ECG_STATE		EQU	VA_DELAY_HIGH+1

;The length of each of the waveform tables
#define P_WAVE_LEN	d'51' 			;102 BYTES / 2 = WORD LENGTH
#define	QRS_WAVE_LEN	d'148'			;296 BYTES / 2
#define	T_WAVE_LEN	d'151'			;302 BYTES / 2
#define V_PACE_LEN	d'300'			;600 BYTES / 2


;DELAY TIMER FOR THE END OF THE QRS WAVE TO THE START OF THE T WAVE
QRS_TO_T_TIMER		EQU	ECG_STATE+1
#define QRS_TO_T_DELAY	d'150'

#define	LVP_WIN_INIT	d'180'


;------------------------------------------------------------------
;THE DELAY TIMER FOR AN RV EVENT TO THE TIME THAT THE LV OUTPUT
;OCCURS.
;-------------------------------------------------------------------
RV_EVENT_TO_LVS_DELAY_TIMER	EQU	QRS_TO_T_TIMER+1

;HOLDS THE CURRENT TIME SINCE THE LAST A EVENT (IN ms)
A_INTERVAL_LOW			EQU	RV_EVENT_TO_LVS_DELAY_TIMER+1
A_INTERVAL_HIGH			EQU	A_INTERVAL_LOW+1

;HOLDS THE CURRENT TIME SINCE THE LAST V EVENT (IN ms)
V_INTERVAL_LOW			EQU	A_INTERVAL_HIGH+1
V_INTERVAL_HIGH			EQU	V_INTERVAL_LOW+1

POWER_UP_COUNTER		EQU	V_INTERVAL_HIGH+1


;FLAGS USED BY THE ECG OUTPUT ROUTINE
ECG_FLAGS			EQU	POWER_UP_COUNTER+1
#define	ECG_PACE_FLAG		ECG_FLAGS,0
#define ECG_V_PACE_FLAG		ECG_FLAGS,1
#define	ECG_PVC_FLAG		ECG_FLAGS,2


#define	HASH_HIGH		ECG_FLAGS,5
#define HASH_ON			ECG_FLAGS,6
#define	ECG_HOLD_FLAG		ECG_FLAGS,7

;A COPY OF THE LAST ECG DATA WORD IS STORED HERE
ECG_LOW			EQU	ECG_FLAGS+1
ECG_HIGH		EQU	ECG_LOW+1

ECG_PREV_LOW		EQU	ECG_HIGH+1
ECG_PREV_HIGH		EQU	ECG_PREV_LOW+1

ECG_PACE_COUNT		EQU	ECG_PREV_HIGH+1

LVP_COUNT		EQU	ECG_PACE_COUNT+1

LVP_WIN_TIMER		EQU	LVP_COUNT+1

;----------------------------------------------------------------------------------
; THE Z OR IMPEDANCE WAVEFORM NORMALLY STARTS 66ms AFTER THE QRS WAVE STARTS,
; THIS PLACES THE START OF THE IMPEDANCE WAVEFORM, AT THE START OF R WAVE + 30ms.
; THE PVC DELAY IS THE SAME DELAY BUT WITH THE EXTRA 30ms REMOVED.
;-----------------------------------------------------------------------------------
Z_DELAY_TIMER		EQU	LVP_WIN_TIMER+1
#define PVC_Z_DELAY	d'66'			;66 ms Q->R DELAY
#define	Z_DELAY		d'96'			;Q->R DELAY + 30ms

;-----------------------------------------------------------
; VARIABLES USED BY THE PVC ROUTINE
;-----------------------------------------------------------
PVC_FLAGS		EQU	Z_DELAY_TIMER+1
#define PVC_V_NO_A_FLAG		PVC_FLAGS,0
#define	LV_IN_A_FLAG		PVC_FLAGS,1
#define SHORT_AV_FLAG		PVC_FLAGS,2
#define	LV_IN_LVS_INH_FLAG	PVC_FLAGS,3
#define	PVC_SUR_FLAG		PVC_FLAGS,4
#define	PVC_Z_FLAG		PVC_FLAGS,5	

PVC_BEAT_COUNT		EQU	PVC_FLAGS+1
LVS_IN_INH_DELAY		EQU	PVC_BEAT_COUNT+1	

PVC_TYPE		EQU	LVS_IN_INH_DELAY+1
#define	PVC_V_NO_A	0
#define	PVC_LV_IN_AV	1
#define	PVC_SHORT_AV	2
#define	PVC_LV_IN_LVS	3
#define	PVC_SURFACE	4
#define PVC_Z_SHIFT	5
#define LAST_PVC	PVC_Z_SHIFT+1

PVC_RAMP_LOW	EQU	PVC_TYPE+1
PVC_RAMP_HIGH	EQU	PVC_RAMP_LOW+1

PVC_COUNT		EQU	PVC_RAMP_HIGH+1
#define	LAST_USED	PVC_COUNT
;***********************************************************
; END OF DATA MEMORY DEFINITIONS
;***********************************************************

